C     ------------------------------
      SUBROUTINE Anelast_Earth (ZDR)
C BTIDE Displacements * Unrelaxed -> relaxed tide spectrum
C     ------------------------------
C>
C Computes the effect of anelastic relaxation on the displacement spectrum.
C_
      IMPLICIT    REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2   IFR(0:9,1200)
      COMPLEX*16  ZDR(1200,3)
      COMMON     /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IFR
      NC=0
      NN=0
      DO 102 L=0,1
      DO 101 I=1,NV
      N=IFR(0,I)
      IF (N.GT.2) THEN
         NN=NN+L
         GOTO 101
      ENDIF
      Z=ZANEL(L,FRE(I),ZZ)
      IF (L.EQ.0) ZDR(I,1)=ZDR(I,1)*Z
      IF (L.EQ.1) THEN
                  ZDR(I,2)=ZDR(I,2)*Z
                  ZDR(I,3)=ZDR(I,3)*Z
                  NC=NC+1
      ENDIF
101   CONTINUE
102   CONTINUE
      PRINT '(A,I5,A,I5)','0<ANELAS>>> Number of constituents affected:'
     &,NC,', unchanged:',NN
      RETURN
      END
C
C
C     -------------------------------------------
      SUBROUTINE Anelast_Comb (ZDR,MC,ZCOMB,ZOUT)
C BTIDE Mixed component * Anelastic relaxed spectrum
C     -------------------------------------------
C>
C ZDR(1200,MC) - complex*16 - a tide spectrum
C ZCOMB        - complex*16 external function name
C ZOUT         - complex*16 - the tide mixed component output
C
C Combines the effect of anelastic relaxation for a mixed component.
C This routine calls a user-written external function ZCOMB with the
C following arguments for each partial tide i:
C
C Z=ZCOMB(N,M,ZA,ZZ,BL,MC,ZD)
C
C where:
C N,M    integer    - degree and order of tide
C ZA(3) complex*16  - relaxations of H,L,K at the frequency of the tide
C ZZ(3) complex*16  - their derivatives w r t the phase at the reference frequ.
C BL(3)  real*8     - the Basic Love numbers H,L,K
C MC     integer    - column dimension of ZDR = dimension of ZD
C ZD(MC) complex*16 - copy of ZDR(i,j),j=1,MC of tide i.
C
C More help....
C______________________________________________________________________________
C
C Example:    Computing the anelastic effect on areal-strain tides
C
C CALL Astro (...)
C CALL Strain_Earth (...)
C CALL Anelast_Comb (ZDR,3,ZAS,ZOUT)
C  ...
C User-written procedure:
C
C Complex*16 Function ZAS(N,M,ZA,ZZ,BL,MC,ZD):
C real*8 BL(3)
C complex*16 ZA(3), ZZ(3), ZD(MC)
C If (N.eq.2) then
C  ZAS=(2.d0*BL(1)*ZA(1)-dfloat(N*(N+1))*BL(2)*ZA(2)) * (ZD(1)+ZD(2)) /
C      (2.d0*BL(1)      -dfloat(N*(N+1))*BL(2))
C else
C  ZAS=(1.d0,0.0d0)
C endif
C return
C end
C
C ... realising that areal strain = ZD(1)+ZD(2)  relaxes as  2h-n(n+1)l
C
C Then, ZOUT contains the areal-strain spectrum with visco-elasticity.
C______________________________________________________________________________
C
      IMPLICIT    REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2   IFR(0:9,1200)
      COMPLEX*16  ZDR(1200,MC),ZOUT(1200),ZD(10),ZZ(3),ZA(3)
      COMMON     /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IFR
      COMMON     /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE
      KNOT=0
      KANEL=0
      print '(2a)','0<ANEL_C>>> Calling user-specified Love-number '
     &,     'combination with anelasticity'
      DO 111 I=1,3
111   ZD(I)=(0.0D0,0.0D0)
      DO 101 I=1,NV
      N=IFR(0,I)
      M=IFR(1,I)
      DO 102 L=1,3
      IF (N.GT.2) THEN
         ZZ(L)=(0.0d0,0.0d0)
         ZA(L)=(1.0D0,0.0D0)
      ELSE
         ZA(L)=ZANEL(L-1,FRE(I),ZZ(L))
      ENDIF
102   CONTINUE
      IF (N.EQ.2) THEN
         KANEL=KANEL+1
      ELSE
         KNOT=KNOT+1
      ENDIF
      DO 103 K=1,MC
103   ZD(K)=ZDR(I,K)
      ZOUT(I)=ZCOMB(N,M,ZA,ZZ,BL,MC,ZD)
101   CONTINUE
      PRINT '(A,2I4)'
     &,     ' <ANEL_C>>> Tides affected / not affected :'
     &,      KANEL,KNOT
      RETURN
      END
C
C
C     -------------------------------------
      SUBROUTINE DZDPHI (ZDR,IREMOV,ZDDRDP)
C BTIDE Displacements * Derivative relaxation w.r.t phase parameter
C     -------------------------------------
C>
C Computes the derivative of the displacement spectrum with respect to
C the lag angle PHI0 (PHI0 refers to the reference frequency F0)
C
C IREMOV - integer - Signals whether ZDR has already been subjected to
C                    subroutine Anelast_Earth (then IREMOV has to be set
C                    to a value greater than zero before call).
C Input:  ZDR
C Output: ZDDRDP
C_
      IMPLICIT    REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2   IFR(0:9,1200)
      COMPLEX*16  ZDR(1200,3), ZDDRDP(1200,3)
      COMMON     /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IFR
      DO 101 L=0,1
      DO 101 I=1,NV
      N=IFR(0,I)
      IF (N.EQ.2) THEN
         Z=ZANEL(L,FRE(I),ZZ)
         IF (IREMOV.GT.0) ZZ=ZZ/Z
         IF (L.EQ.0) ZDDRDP(I,1)=ZDR(I,1)*ZZ
         IF (L.EQ.1) THEN
                     ZDDRDP(I,2)=ZDR(I,2)*ZZ
                     ZDDRDP(I,3)=ZDR(I,3)*ZZ
         ENDIF
      ELSE IF (L.EQ.0) THEN
         DO 103 K=1,3
103      ZDDRDP(I,K)=(0.0D0,0.0D0)
      ENDIF
101   CONTINUE
      PRINT '(2A)'
     &,'0<DZDPHI>>> Returns derivative of anelastic spectrum w.r.t. lag'
     &,' parameter PHI0'
      RETURN
      END
C
C
C     ----------------------------------
      COMPLEX*16 FUNCTION ZANEL (L,F,ZZ)
C BTIDE Anelasticity * Love No. Relaxation function at frequency
C     ----------------------------------
C>
C Computes the anelastic response of the earth according to a parameterized
C version of the rheological model of Zschau and Wang (1985), Wang (1986).
C
C L     - integer    - code for the Love number for which anelasticity is
C                      requested:
C                  0 - H = radial displacement
C                  1 - L = tangential displacement
C                  2 - K = secondary potential
C
C F     - real*8     - frequency (rad/h)
C
C ZZ    - Complex*16 - RETURNED: Derivative dZANEL/dPHI0(L) = derivative
C                      w.r.t. the tide lag parameter PHI0 (the lag at the
C                      reference frequency F0).
C
C ZANEL - complex*16 - returns the complex response ratio of anelastic
C                      relaxation w r t a basic frequency BSF
C                      (relative admittance and phase).
C                      I.e. CDABS( ZANEL(i,F_cal) ) = VCAL(i)
C
C BSF can be redefined using CALL AELBSF (F)
C_
      IMPLICIT  REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      REAL*8    PHI0(0:2),PHI1(0:2), P(0:2),SUM2(0:2)
     &,         ATTBSF(0:2),CFCBSF(0:2),VCAL(0:2),FCAL(0:2)
      LOGICAL   LBSF,LFCAL,LCALL(0:2)
      CHARACTER COMP(0:2)*12,COMBSF*12
      COMMON    /MPAR/  PI, RAD
      COMMON    /WANGP/ PHI0,PHI1,P,BRPER,FBR,F0,BSF,FCAL,SUM2
     &,                 ATTBSF,CFCBSF,VCAL
     &,                 LCALL,COMBSF,COMP
      DATA      EPS/5.D-6/
C
      IF (L.GT.2.OR.L.LT.0) THEN
         PRINT '(A,I6)',' <ZANEL->T> Illegal component number:',L
         STOP
      ENDIF
      IF (.NOT.LCALL(L)) GOTO 101
C
C ---> Compute frequency independent constants >-------------------->
      NERR=0
      FBR=RAD*0.04107/BRPER
      OBR=FBR/F0
      PHI1(L)=PHI0(L)*OBR**(P(L)-1.D0)
C
C SUM2 ( ... is due to the pole in the Hilbert transform integral)
      TSUM=0.0D0
      AO=EPS
      N=-1
201   N=N+1
      IF (N.GT.100000) THEN
         PRINT '(2A)',' <ZANEL->E> Bad convergence SUM2, component '
     &,       COMP(L),' <ZANEL->I> Check the following parameters:'
         CALL Show_Prm ('ANEL')
         PRINT '(A,1P,2D12.4)',' <ZANEL>I> Err, Abs(SUM2) =',ERR,TSUM
         GOTO 202
      ENDIF
      AN=1.D0/(DFLOAT(N*N+2*N)+P(L)*(1.0D0-P(L)/2.0D0))
      TSUM=TSUM+AN
      Q=DABS(AN/AO)
      IF (N.GT.2.AND.Q.LT.1D0) THEN
         ERR=DABS(AO)/(1.D0-Q)
         IF (ERR.LT.EPS*DABS(TSUM)) GOTO 202
      ENDIF
      AO=AN
      GOTO 201
202   SUM2(L)=TSUM*2.D0/PI*(P(L)-1.D0)
C
C Compute ATT and PHI at BSF = Basic frequency
      LBSF=.TRUE.
      O=BSF/F0
      OOB=BSF/FBR
      IF (OOB.LE.1.D0) THEN
         PRINT '(A,1P,D12.4,A)',
     &       ' <ZANEL->T> Basic frequency at/below breakpoint :'
     &,         BSF/RAD,' (deg/h). '
     &,      ' <ZANEL->I> Check the following parameters:'
         CALL Show_Prm ('ANEL')
         STOP
      ENDIF
      GOTO 102
C              ---> 102 , compute ATT and PHI, and return here again: --,
C                                                                        )
C  <--------------------------------------------------------------------`
211   ATTBSF(L)=ATT
      PHIBSF=PHI
      LBSF=.FALSE.
      LFCAL=.TRUE.
      O=FCAL(L)/F0
      OOB=FCAL(L)/FBR
      GOTO 102
C              ---> 102 , compute ATT and PHI, and return here again: --,
C                                                                        )
C  <--------------------------------------------------------------------`
212   CFCBSF(L)=VCAL(L)/EXP(ATT-ATTBSF(L))
      LFCAL=.FALSE.
      LCALL(L)=.FALSE.
      PRINT 1001, COMP(L),COMBSF,BSF/RAD,PHIBSF/RAD,FCAL(L)/RAD,PHI/RAD
     &,           CFCBSF(L),SUM2(L)
     &,           FBR/PI*12.D0*365.25D0
1001  FORMAT ('0<ZANEL->>> Constants stored for tide component ',A12/
     &' <ZANEL->>>'/
     &' <ZANEL->>> Basic frq.',A12,': ',T43,F10.6,' deg/h, '
     &,'phase:',T68,F8.4,' deg'/
     &' <ZANEL->>> Calibration frequency:',T43,F10.6,' deg/h, phase:'
     &,T68,F8.4,' deg'/
     &' <ZANEL->>> Calibration constant:',T43,F10.6/
     &' <ZANEL->>> Pole factor ALPHA'':',T43,F10.6/
     &' <ZANEL->>> Model valid above breakpoint:',T43,F10.6,' cyc/y')
C
C -------------------------------------------< End compute const <--
C
101   CONTINUE
      O=F/F0
      OOB=F/FBR
      IF (OOB.LE.1.D0) THEN
         PRINT '(A,1P,D12.4,A)',
     &       ' <ZANEL->W> Frequency at/below breakpoint :',F/RAD
     &,      ' (deg/h). Unity response returned.'
         ZANEL=(1.0D0,0.0D0)
         RETURN
      ENDIF
C
C Compute  PHI = Phase  and  ATT = Attenuation                  102
C
C                          Phase:
102   PHI=PHI0(L)*O**P(L)
C
      SUM1=0.0D0
      OOB2N=1.0D0
C                    Attenuation:
C Summation ...
      AO=EPS
      SUM1=0.0D0
      N=-1
111   N=N+1
      IF (N.GT.100) THEN
         NERR=NERR+1
         PRINT '(A,1P,D12.4)'
     &,  ' <ZANEL->E> Bad convergence SUM1, freq.=',F/RAD,' deg/h.'
         IF (NERR.GT.10) THEN
            PRINT '(A)',' <ZANEL->T> SUM1 something wrong. Check:'
            CALL Show_Prm ('ANEL')
            STOP
         ELSE
            GOTO 112
         ENDIF
      ENDIF
      AN=OOB2N/(DFLOAT(2*N)+P(L))
      SUM1=SUM1+AN
      Q=DABS(AN/AO)
      IF (N.GT.2.AND.Q.LT.1D0) THEN
         ERR=DABS(AO)/(1.D0-Q)
         IF (ERR.LT.EPS*DABS(SUM1)) GOTO 112
      ENDIF
      AO=AN
      OOB2N=OOB2N/OOB/OOB
      GOTO 111
112   CONTINUE
C             ... end summation
C
      ATT = PHI1(L)*O/PI*DLOG(DABS(O+OBR)/(O-OBR))
     &    - PHI1(L)*OBR*2.D0/PI*SUM1
     &    - PHI*SUM2(L)
C                                                    back to 211, BSF ?
C                                                      "   " 212, F0  ?
      IF (LBSF) GOTO 211
      IF (LFCAL)  GOTO 212
C
C Relative to attenuation at Basic frequency (N.B. use DCMPLX if available):
      Z=CMPLX(ATT-ATTBSF(L),PHI)
C
C Return complex transfer function value (N.B. use CDEXP if available):
      ZA=CFCBSF(L)*EXP(Z)
      ZZ=ZA*Z/PHI0(L)
      ZANEL=ZA
      RETURN
      END
