      subroutine Astronomic_THD (iunc)
c
c Tamura tide harmonic development in J2000.
c Calls Astron_arg.
c Processes ETERNA data file ETCPOT.DAT on file unit IUNC.
c
c Returns spectral coefficients and argument numbers in   common /CASTSP/ 
c
c Parameter setting entries:
c --------------------------
c
c The static "tide" is switched off by default. To include it,
c
c    Call Set_Permanent_Tide (.true.)
c
c
c Before this routine can be called, the following call is
c required:  Call Set_EP_J2000 (idate)
c
c
c The THD file can be preconnected; in this case, this routine will
c not close it at return time. Otherwise, the file will be opened 
c with the file name announced by
c
c    Call Astronomic_THD_File (iun,filename)
c
c or /home/hgs/Ttide/etgtab/etcpot.dat by default (filename=' ')
c
c Thus, the file unit number can be announced with
c
c    Call Astronomic_THD_File (iun,' ')
c
c e.g. in Main, and used with 
c
c    Call Astronomic_THD (-1)
c
c e.g. in a subroutine as an alternative to Call Astronomic_THD (iun)
c
c
c In ocean loading applications, the tide potential can be reduced to the
c essential terms:
c
c    Call Use_THD_Oceanload (qq,threshold)
c
c qq=.true. will weed out all partial tides with harmonic degree greater 
c than two. The threshold specifies a tide potential amplitude [m^2/s^2] 
c below which the partical tide is neglected. (Defaults: .false., 0.0)
c
c Special: 
c   Entry Tamura_CTE_style (qq, qqq)
c
c to sacrifice Tamura's Venus and Jupiter terms (which need additional
c argument equations);
c and to phase-wrap all arguments to the interval [0,360) 
c
      implicit real*8 (a-h,o-z)
      integer*2 qaa,ia7
      character sym*4, dsym*7, hend*8, trg*8, fn*64, cc*(*), darwin*4
     &,         f1101*80
      logical   q0f,qi0f,qq,qqq,qmsg(10),qop,qufl,qraw,q7,qnoneg
      dimension WNM(2:5,0:5),E(8),EP(8),DRA(2:5),QAA(0:9),DARWIN(1200)
     &,         rawamp(1200)
     &,         ia7(0:9,1200)

      include 'tide_spectrum.fca'
c
c New! we keep the 0:9 structure, since we have defined IAAW
      equivalence (iaa(0,1),ia7(0,1))
c
      COMMON   /EJ2000/ DTUJD,DTUT20,DTDT20,DDT,ITY,ITM,ITD
      COMMON   /MPAR/   PI,RAD
      COMMON   /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG
     &,                 RHOW
      COMMON   /CIAAW/  IAAW

      SAVE
      DATA RMMOON/0.01230002d0/, PRLX/3422.448d0/, ampmin/0.d0/
     &,    HEND/'C*******'/, QI0F/.FALSE./, QMSG/10*.true./
     &,    qufl/.false./
     &,    fn/'/home/hgs/Ttide/etgtab/etcpot.dat '/, iun/-1/
     &,    iuout/-1/, q7/.false./, qnoneg/.false./
     &,    f1101/'(i5,10i3,1p,e16.8,0p,f12.5,f14.10,1x,a4,1x,a7)'/

      print '(a)','0<AstTid>>> TAMURA (1987) TIDE HARMONIC DEVELOPMENT'
      if (iunc.gt.0) iun=iunc
      RAD=DATAN(1.d0)/45.d0
      PI=RAD*180.d0
      
      if (iun.lt.0) then
         print '(a)',' <AstThd>T> Invalid file unit number.'
         stop '<AstThd>T> No FUN at all.'
      endif
      
      inquire (unit=iun,opened=qop)
      if (.not.qop) then
         lfn=index_l_nonblank(fn)
         print '(a,i2.2,1x,a)'
     &,        ' <AstThd>>> Opening file #',iun,fn(:lfn)
         open (unit=iun,file=fn,status='OLD',form='FORMATTED'
     &,        err=901,iostat=ios)
         call save_file_name (iun,fn)
      else
         inquire (unit=iun,name=fn)
         lfn=index_l_nonblank(fn)
         print '(a,i2.21x,a)'
     &,        ' <AstThd>>> Already open #',iun,fn(:lfn)
         lfn=index_l_nonblank(fn)
      endif
C
C Factors for use with conventional spherical harmonics
c
c Remember: WE NEVER USE THOSE GEODETIC COEFFICIENTS
c           WE USE THE DEFINITION OF ABRAMOVITZ-STEGUN 
c           WE DON'T FLIP SIGNS
c Careful with P20!
      WNM(2,0)=-1.0D0
      WNM(2,1)= 1.5D0
      WNM(2,2)= 3.0D0
      WNM(3,0)=-1.0D0/DSQRT( 5.0D0)
      WNM(3,1)=-8.0D0/DSQRT(15.0D0)
      WNM(3,2)=10.0D0/DSQRT( 3.0D0)
      WNM(3,3)=15.0D0
      WNM(4,0)= 1.0D0
      WNM(4,1)=-5.D0*(3.D0+DSQRT(393.D0))
     &              *DSQRT(390.D0+2.D0*DSQRT(393.D0))/896.D0
      WNM(4,2)=-135.0D0/14.0D0
      WNM(4,3)= 315.0D0*DSQRT(3.0D0)/16.0D0
      WNM(4,4)= 105.0D0
      WNM(5,0)= 1.0D0
      WNM(5,1)=-2.0588572D0            ! X_max = sqrt((3-2*sqrt(11/21))/5)
      WNM(5,2)=-10.47079983D0          ! X_max = sqrt((2-sqrt(7/3))/5)
      WNM(5,3)=140.D0/3.D0/DSQRT(1.5D0)
      WNM(5,4)=3024.D0/5.D0/DSQRT(5.D0)
      WNM(5,5)=945.D0
C
      CALL ASTRON_ARG (E,EP)
      DJ2000=DTUT20+1.d0/36525.d0           ! That's worth a check
C
C compute Doodson
      Doodsn=0.75d0*GM*RMMOON/RE*(R0/RE)**2*(PRLX*RAD/3600.d0)**3
c
      DRA(2)=(RE/R0)**2
      DRA(3)=DRA(2)*RE/R0
      DRA(4)=DRA(2)*DRA(2)
      DRA(5)=DRA(2)*DRA(3)
      print '(a,1p,2(d14.6,a))'
     &,     ' <AstThd>>> Doodson const: ',Doodsn,' m^2/s^2, (re/r0)**2:'
     &,      dra(2)
c  
c Process the file
      rewind (iun)
8001  format (a8)
8002  format (i5)
8003  format (i4,4x,10i2,t62,f8.6,f9.6)
800   read (iun,8001) trg
      if (trg.ne.hend) goto 800
      read (iun,8002) nv
      iv=0
801   read (iun,8003,end=802) iw,qaa,dht,dhtp
      if (iw.le.0) goto 801
      iv1=iv+1
      dc2=0.0d0
      dc3=0.0d0
      q0f=.false.
      do 811 i=1,8
         q0f=q0f.or.qaa(i).ne.0
         if (q7) then
            if (qaa(7).ne.0.or.qaa(8).ne.0) goto 801 ! away with Venus&Jupiter!
            if (i.le.6) ia7(i,iv1)=qaa(i)
         else
            iaa(i,iv1)=qaa(i)
         endif
         if (i.gt.0) then
            k=qaa(i)
            dc2=dc2+e(i)*dfloat(k)
            dc3=dc3+ep(i)*dfloat(k)
         endif
 811  continue
      if (q0f.or.qi0f) then
         n=qaa(0)
         if (n.gt.5) then
            if (qmsg(n)) then
               print '(a)'
     &,        ' <AstThd>W> File contains SH degree ',n
               qmsg(n)=.false.
            endif
         else
            iv=iv+1
            if (q7) then
               ia7(0,iv)=qaa(0)
               ia7(7,iv)=qaa(9)
            else
               iaa(0,iv)=qaa(0)
               iaa(9,iv)=qaa(9)
            endif
            m=qaa(1)
            k=qaa(9)
            fre(iv)=dc3*rad
            dht=dht+dhtp*dj2000
            rawamp(iv)=dra(n)*dht
            av=doodsn*dra(n)*dht/wnm(n,m)
            if (qnoneg.and.av.lt.0.d0) then
               k=k+2
               av=-av
            endif
            amp(iv)=av
            dc2=dc2+90*k
            dc2=dmod(dc2+720.d0,360.0d0)
            if (dc2.lt.0.d0) dc2=dc2+360.d0
            pha(iv)=dc2*rad
            if (qufl) then
               if (qaa(0).ne.2
     &             .or.abs(amp(iv)).lt.ampmin) iv=iv-1
            endif
            if (.not.q0f) print '(a,1p,4d12.4/a,d12.4,3x,10i3)'
     &,' <AstThd>>> Perm.Tide: D,dra,dht,wnm: '
     &,              doodsn,dra(n),dht,wnm(n,m)
     &,' <AstThd>>>            Pot.Amplitude: ',amp(iv),(qaa(k),k=0,9)
         endif
      else
         print '(a,10i2,1p,d13.4)',' <AstThd>>> Skip ',qaa,dht
      endif
      goto 801

802   continue
      print '(a,2i5,a)',' <AstThd>>> ',nv,iv,' spectral lines.'
      nv=iv
      if (.not.qop) then
         close (iun)
         print '(a,i2)',' <AstThd>>> Closing file unit ',iun
      endif
      return      

901   lfn=index_l_nonblank(fn)
      print '(a,i2,2h ",a,2h" ,a,i4)'
     &,     ' <AstThd>T> File open ',iun,fn(:lfn),' failed. IOSTAT=',ios
      print '(a,2h !)'
     &,' <AstThd>I> User, employ Call Astronomic_THD_File(IUN,FILENAME)'
      stop 'due to OPEN ERR in Astronomic_Thd'

      entry Set_Permanent_Tide (qq)
      qi0f=qq
      print '(a,l2)'
     &,     ' <SPermT>>> Permanent tide is included (T/F) ? -',qi0f
      return

      entry Use_THD_Oceanload (qq,ampc)
      qufl=qq
      ampmin=ampc
      return

      entry Astronomic_THD_File (iunc,cc)
      iaaw=9
      iun=iunc
      if (cc(1:1).ne.' ') fn=cc
      return
      
      entry Write_Astronomic_THD (iunc,qraw)
c
c Astronomic_THD is requested to write the development to file unit IUNC
c
c "raw amp" designates a potential number that is not yet multiplied
c with Doodson's constant nor with the (r/r0)**n term nor divided
c with the normalizing factor (not adapted to conventional
c Associated Legendres). 
c 
      print '(a,2i5,a)',' <WAstTHD>>> Unit ',iunc,nv,' lines'
      write (iunc,'(a)') f1101
      do i=1,nv
         if (qraw) then
            aw=rawamp(i)
         else
            aw=dabs(amp(i))
         endif
         pw=pha(i)/rad
         if (amp(i).lt.0.d0) pw=dmod(pw+540.d0,360.d0)
         if (q7) then
            do k=0,6
               qaa(k)=ia7(k,i)
            enddo
            qaa(9)=ia7(7,i)
            qaa(8)=0
            qaa(7)=0
         else
            do k=0,9
               qaa(k)=iaa(k,i)
            enddo
         endif
         call symbol_of_tide (qaa,sym)
         call doodson_symbol (qaa,dsym)
         write (iunc,f1101) i,(qaa(k),k=0,9)
     &,                    aw,pw,fre(i)*12.d0/pi,sym,dsym
      enddo
      return
      
      entry tamura_cte_style (qq,qqq)
c
c qq  .true.: Astronomic_THD is to skip Venus and Jupiter for the benefit
c             of easy implementation of Tamura's development into code 
c             that copes only with 7-astro-arguments.
c
c qqq .true.: Astronomic_THD is to convert AMP and PHA such that  
c             amplitudes in COMMON /CASTSP/ are all positive.
c
      q7=qq
      qnoneg=qqq
      if (q7) then
         print '(a)',' <AstThd>>> Using 7 astronom.arguments, '
     & //' ignoring Venus and Jupiter.'
         iaaw=7
      else
         print '(a)',' <AstThd>>> Using 9 astronom.arguments, '
     & //' including Venus and Jupiter.'
         iaaw=9
      endif
      if (qnoneg) then
         print '(a)',' <AstThd>>> Positive amplitudes thoughout.'
      else
         print '(a)',' <AstThd>>> Amplitudes may be negative valued.'
      endif
      return
      end
