C
C     --------------------------------------
      SUBROUTINE Displacement_tide (X,Y,ZDR)
C Displacements * Wahr, 1981
C     --------------------------------------
C
C After CALL ASTRO(...), Ela_Ell_Earth can be called to return...
C ZDR = tide displacement spectrum for a station at...
C X = east longitude
C Y = colatitude Y (geocentric coordinates, units = radians).
C
C Elastic, elliptical earth model of WAHR (1981) with liquid core,
C refined parameters according to NEUBERG et al. (1987).
C ________________________________________________________________________
C
C ENTRY D_Displacement_Tide (X,Y,ZDR,ZO,ZM)
C
C returns a linear combination   ZO*ZDR.in + ZM*ZDR.new = ZDR.out
C supposedly after an earth or site parameter has been changed. This call
C provides a way to obtain derivatives w r t a parameter:
C
C Call Displacement_tide (x,y,zdr)
C Call get_ndrfrq (f)
C      fnew = f*(1.001)
C      zdf = dcmplx(1d0/(fnew-f),0.0d0)
C Call ndrfrq (fnew)
C Call D_Displacement_tide (x,y,zdr,-zdf,zdf)
C
C Then zdr(returned) is the partial derivative of dZ/df_NDR
C
C
C Call Inertial_Tide (q_acceleration, q_coriolis)
C
C      To be called before Displacement_tide, requests accelerations
C      due to displacements. Units = nm/s^2
C 
C _________________________________________________________________________
C
C ZDR   - complex*16 - Array of length (1200,3) (NV c.f. common /CASTSP/),
C                      the second index k (column) specifying the component.
C                      The columns furnish  Up, East, North  in units of meter.
C
C In order to obtain the tide displacement, component k, at time T, perform
C
C   DR(k) = Real ( Sum (i=1,IW) BL(k) * ZDR(i,k) * Exp(ZI*FRE(i)*T) )
C
C where
C       k = component, 1 - radial, 2 - east, 3 - north
C      BL = Basic Love number (you find them in common /EPAR/)
C      ZI = imaginary unit
C     FRE = Frequency array, found in common /CASTSP/
C       T = time from initial epoch in hours.
C __________________________________________________________________________
C
C ENTRY SP_Displacement_Tide (X,Y,IAA_P,ZDRSP)
C
C returns a single displacement partial tide, given that tides argument
C numbers (IAA_P). 
C At return, message option is switched off. If repeated messages are
C desired, call Displacement_Tide_MSG (.TRUE.) before SP_Displacement_Tide.
C
C ENTRY F_Displacement_Tide (X,Y,II,QQ,ZDRSP)
C
C is similar. Bin index of tide in spectrum is supplied: II
C QQ=.true. presumes spherical harmonics and other geographical
C parameters can be used from the previous call. QQ=.false.: re-init.
C
C
      IMPLICIT        REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2       IAA, IAA_P(0:9)
c      INTEGER*2       IATR(0:6)
c      LOGICAL         QTR
      INTEGER         ICO(2:4,0:4)
      REAL*8          P(2:4,0:4), DP(2:4,0:4)
     &,               CORDH(2:4,0:4), CORDL(2:4,0:4), CORDK(2:4,0:4)
      REAL*8          WLP0(0:2),WWP0(0:2),WZ0(0:2),WKP0(0:2),WSLP(0:2)
     &,               WT(0:5)
      COMPLEX*16      ZV(3), ZLO(0:4), ZDR(1200,*), ZDRSP(*)
      LOGICAL         QMSG,QQ,QSP,QCDNAN, QACC, QCORIO
      EQUIVALENCE    (ZV(1),ZH), (ZV(2),ZLE), (ZV(3),ZLN) 

      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IAA(0:9,1200)
      COMMON /MPAR/   PI,RAD
C
C Earth parameters:   Mean radius, equat.radius, equat.gravity, rot.velocity,
C                     Basic Love numbers:
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW
C
C Near diurnal resonance (NDR) parameters:
      COMMON /COREP/  ONDR,QNDR,FREFC
     &,               SEHNDR,SELNDR,SEKNDR,ZOHNDR,ZOLNDR,ZOKNDR
C
C Love number ratios for spherical harmonic degrees and orders w r t  n=2, m=1
      COMMON /SHDOP/  CORDH, CORDL, CORDK
C
C Spherical harmonic coupling parameters; Wahr-model
      COMMON /WAHRP/  WWP0,WLP0,WZ0,WY,WWP1,WLP1,WZ1,WS3
     &,               WKP0,WSLP
     &,               WT
C
      SAVE
      DATA  ZI/(0.0D0,1.0D0)/, QMSG/.TRUE./, QACC,QCORIO/2*.false./
c     &,     IATR/2,2,0,0,0,0,0/
C
C Reference frequency for resonance formula = O1
C        FREFC/0.243352D0/
C initialized in BLOCK DATA, common /COREP/
C
      QSP=.FALSE.
      ZO=(0.0D0,0.0D0)
      ZM=(1.0D0,0.0D0)
      IV=0
      GOTO 1
C
C     -------------------------------------------
      ENTRY D_Displacement_Tide (X,Y,ZDR,ZOD,ZMD)
C Displacements * Lin.comb. after parameter change
C     -------------------------------------------
      QSP=.FALSE.
      ZO=ZOD
      ZM=ZMD
      IV=0
      GOTO 1
C
C     -------------------------------------------
      ENTRY F_Displacement_Tide (X,Y,II,QQ,ZDRSP)
C Displacements * Singe partial tide, fast (QQ=true)
C     -------------------------------------------
      QSP=.TRUE.
      IV=II
      IF (QQ) GOTO 100
      GOTO 1
C
C     --------------------------------------------
      ENTRY SP_Displacement_Tide (X,Y,IAA_P,ZDRSP)
C Displacements * Singe partial tide
C     --------------------------------------------
      QSP=.TRUE.
      IV=NR_OF_TIDE(IAA_P,9)
C
    1 CONTINUE

      OME=EROT/3.6d3 ! rad/s
      IF (Y/RAD.LT.1.D-3.OR.Y/RAD.GT.179.99D0) THEN
         PRINT '(A,F10.4)','0<DiTide>W> Invalid colatitude :'
     &,   Y/RAD,'. Adjusted.'
         Y=DMIN1(179.99*RAD,DMAX1(1.D-3*RAD,Y))
      ENDIF
      FRENDR=(1.D0+1.D0/ONDR)*EROT
      ZFRNDR=CMPLX(FRENDR,FRENDR/QNDR/2.D0)
C                                       (N.B. use DCMPLX if available)
      TRMIN=1.D10
      TTMIN=0

      MCACT=INQ_ACTIVE_COMP()
      IF (MCACT.GT.1) TTMIN=1.D10
C
C Spherical harmonics for the site:
      IF (QMSG) THEN
         PRINT '(A,2F12.6)'
     &,        '0<DiTide>>> Site longitude / colatitude:',X/RAD,Y/RAD
         PRINT '(A)',' <DiTide>>> (P(N,M), DP(N,M), M=0,N):'
      ENDIF
      DO 20 N=2,4
         DO 21 M=0,N
            ICO(N,M)=0
            P(N,M)=PNM(N,M,Y)
            IF (MCACT.GT.1) DP(N,M)=DPNMX(N,M,Y)
 21      CONTINUE
         IF (QMSG) PRINT '(I3,5(2X,2F9.4))'
     &,                   N,(P(N,M),DP(N,M),M=0,N)
 20      CONTINUE
      DO 22 M=0,4
         DM=DFLOAT(M)
 22      ZLO(M)=EXP(ZI*DM*X)
C                          (N.B. use CDEXP instead of EXP if available)
      SINY=DSIN(Y)
C
C ------------------------- Step through the astron.spectrum -->>>
 100  IF (.NOT.QSP) THEN
         IV=IV+1
         IF (IV.GT.NV) GOTO 101
      ENDIF
      N=IAA(0,IV)
C
C Discard sph.harm.degree > 4 ( M5/M2 is  5 ppm)
      IF (N.GT.4.OR.N.LT.2) THEN
         IF (QSP) THEN
            DO IC=1,MCACT
               ZDRSP(IC)=(0.0D0,0.0D0)
            ENDDO
            RETURN
         ELSE
            DO IC=1,MCACT
               ZDR(IV,IC)=(0.0D0,0.0D0)
            ENDDO
         ENDIF
         GOTO 100
      ENDIF
      M=IAA(1,IV)
      ICO(N,M)=ICO(N,M)+1
      DM=DFLOAT(M)
C
      IF (N.EQ.2) THEN
C
C Leading terms
C
         IF (M.EQ.1) THEN
C
C Liquid core effects
C
            ZCLQCH=(SEHNDR*(FRE(IV)-FREFC)
     &            + ZOHNDR*EROT)/(FRE(IV)-ZFRNDR)
            ZCLQCL=(SELNDR*(FRE(IV)-FREFC)
     &            + ZOLNDR*EROT)/(FRE(IV)-ZFRNDR)
            ZL2N  = ZCLQCL*((1.D0+WZ1*P(2,0))*DP(2,1) + WLP1*DP(4,1)
     &            + WWP1*DM*P(3,1)/SINY)
            ZL2E  = ZCLQCL*(((1.D0+WZ1*P(2,0))*P(2,1) + WLP1*P(4,1))
     &              *DM/SINY
     &            + WWP1*DP(3,1))
C
         ELSE
C
C Long period or semidiurnal
C
            ZCLQCH=(0.0D0,0.0D0)
            ZCLQCL=(0.0D0,0.0D0)
            ZL2N  =(0.0D0,0.0D0)
            ZL2E  =(0.0D0,0.0D0)
         ENDIF
         CL1N  = WZ0(M)*P(2,0)*DP(N,M) + WLP0(M)*DP(4,M)
     &         + WWP0(M)*DM*P(3,M)/SINY
         CL1E  =(WZ0(M)*P(2,0)*P(N,M) + WLP0(M)*P(4,M))*DM/SINY
     &         + WWP0(M)*DP(3,M)
         CH1   = 1.D0+WY*P(2,0)
C
      ELSE
C
C Spherical harmonic degree > 2
         CH1   = 1.0D0
         CL1N  = 0.0D0
         CL1E  = 0.0D0
         ZL2N  =(0.0D0,0.0D0)
         ZL2E  =(0.0D0,0.0D0)
         ZCLQCH=(0.0D0,0.0D0)
C
      ENDIF
C
C Put together: potential amplitude / g * (spherical eff. + ellips.perturb.)
C N.B.: Unit vector North = - unit vector colatitude
      XAMP=AMP(IV)/GE
      ZH = XAMP*CORDH(N,M)*(1.D0+ZCLQCH)*CH1*P(N,M)
      ZLN=-XAMP*CORDL(N,M)*(DP(N,M)+CL1N+ZL2N)
      ZLE= XAMP*ZI*CORDL(N,M)*(P(N,M)*DM/SINY+CL1E+ZL2E)
      ZPHA=EXP(ZI*PHA(IV))
C                          (N.B. use CDEXP instead of EXP if available)
      IF (QSP) THEN
         DO IC=1,MCACT
            ZDRSP(IC)=ZV(IC)*ZLO(M)*ZPHA
         ENDDO
         GOTO 101
      ENDIF

      DO IC=1,MCACT
         if (qcdnan(zdr(iv,ic)).and.abs(zo).lt.1.d-40) 
     &      ZDR(IV,IC)=(0.0d0,0.0d0)
         ZDR(IV,IC) = ZV(IC)*ZLO(M)*ZPHA*ZM + ZO*ZDR(IV,IC)
      ENDDO
C     
c      QTR=.TRUE.
c      DO 901 I=0,6
c      QTR=QTR.AND.IAA(I,IV).EQ.IATR(I)
c901   CONTINUE
c      IF (QTR) THEN
c         PRINT '(3H0Tr,2F10.4,1P,D12.4/(3X,1P,2D12.4))'
c     &,  AMP(IV),PHA(IV)/RAD,XAMP,ZH,ZLO(M),ZPHA,ZM,ZO
c      ENDIF
C
C Check smallest amplitude (N.B. use CDABS if available):
      AR=ABS(ZDR(IV,1))
      IF (MCACT.GT.1) THEN
         AT=ZDR(IV,2)*DCONJG(ZDR(IV,2))+ZDR(IV,3)*DCONJG(ZDR(IV,3))
         AT=DSQRT(AT)
         TTMIN=DMIN1(AT,TTMIN)
      ENDIF
      TRMIN=DMIN1(AR,TRMIN)
      IF (.NOT.QSP) GOTO 100
C
C ------------------------------------------------ End loop ---<<<
101   CONTINUE
      IF (QMSG) THEN
         PRINT '(A,1P,2D10.2,A)'
     &,' <DiTide>>> Min. tide amplitudes (radial, tangential) :'
     &, TRMIN*0.6, TTMIN*0.085,' m'
         PRINT '(A)',' <DiTide>>>'
     &,' <DiTide>>> Number of tides in the different bands:'
     &,' <DiTide>>>  N / M=0    1    2    3    4'
         DO 111 N=2,4
 111     PRINT '(12H <DiTide>>> ,I2,1X,5I5)', N,(ICO(N,M),M=0,N)
         PRINT '(11H <DiTide>>>)'
         IF (QSP) QMSG=.FALSE.
      ENDIF
      RETURN
      
      ENTRY Displacement_Tide_MSG (QQ)
      QMSG=QQ
      RETURN
      END

      SUBROUTINE Inertial_tide (QACC,QCORIO,X,Y,ZDR,UNITS)
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      
      DIMENSION ZDR(1200,*),ZACC(3),XL(3)
      INTEGER*2 IAA
      LOGICAL   QACC,QCORIO,Q_BLMULT
      CHARACTER UNITS*8

      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IAA(0:9,1200)
      COMMON /MPAR/   PI,RAD
C
C Earth parameters:   Mean radius, equat.radius, equat.gravity, rot.velocity,
C                     Basic Love numbers:
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW

      DATA ZI/(0.0D0,1.0D0)/

      IF (.NOT.QACC.AND..NOT.QCORIO) RETURN

      UNITS='[nm/s^2]'
      OME=EROT/3.6D3
      amx=-1.d0

      MCACT=INQ_ACTIVE_COMP()
      IF (MCACT.LT.2) THEN 
         PRINT '(A,I2)',' <InerTi>E> Need at least 2 comps, not:',MCACT
         PRINT '(A)'  ,' <InerTi>I> Need 1 & 2 i.e. R & E'
         STOP
      ENDIF
      
      IF (Q_BLMULT()) THEN
         XL(1)=BL(1)
         XL(2)=BL(2)
         XL(3)=BL(2)
      ELSE
         XL(1)=1.d0
         XL(2)=1.d0
         XL(3)=1.d0
      ENDIF

      DO IV=1,NV
      ZACC(1)=(0.d0,0.d0)
      ZACC(2)=(0.d0,0.d0)
      ZACC(3)=(0.d0,0.d0)
      IF (QACC) THEN
         OMT=FRE(IV)/3.6D3
         DO IC=1,MCACT
            ZACC(IC)=-ZDR(IV,IC)*OMT**2*XL(IC)
         ENDDO
c         print *,fre(iv)/rad/15.d0,zdr(iv,1),omt**2,xl(1)
      ENDIF
      IF (QCORIO) THEN
         OMT=FRE(IV)/3.6D3
         ZACC(1)=ZACC(1)-2.d0*ZDR(IV,2)*SIN(Y)*ZI*OME*OMT*XL(2)
         IF (MCACT.EQ.3) THEN
            ZACC(2)=ZACC(2)-( ZDR(IV,3)*XL(3)*COS(Y)
     &                       -ZDR(IV,1)*XL(1)*SIN(Y))
     &                     *2.d0*ZI*OME*OMT
            ZACC(3)=ZACC(3)+2.d0*ZDR(IV,2)*COS(Y)*ZI*OME*OMT*XL(2)
         ENDIF
      ENDIF
      DO IC=1,MCACT
         ZDR(IV,IC)=ZACC(IC)*1.d9 ! nm/s^2
         a=cdabs(zdr(iv,ic))
         if (a.gt.amx) then
            icm=ic
            ivm=iv
         endif
         amx=dmax1(a,amx)
      ENDDO
      
      ENDDO

      CALL BLMULT ('NO')
      print '(a,i2,3f10.4,1x,2l2)',' <InerTi>>> ',MCACT,XL,qacc,qcorio
      print '(a,1p,d12.4,a,d12.4,a,i2,i5,a,7i3)'
     &,' <InerTi>>> OMEGA_EROT=',OME
     &, ' max d^2u/dt^2 = ',amx,' ic,iv=',icm,ivm
     &,' arg: ',(iaa(k,ivm),k=0,6)
      RETURN
      END

C
C     -----------------------------
      SUBROUTINE AVE_TIDE (ZDR,T)
C Tide potential * Effective average over T [h]
C     -----------------------------
      IMPLICIT    REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2   IFR(0:9,1200)
      COMPLEX*16  ZDR(1200,*)
      COMMON     /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IFR
      if (t.le.1d-4) return
      n=inq_active_comp()
      print '(a,1p,d12.4,a)',' <AveTid>>> Average over ',t,' [h]'
      do iv=1,nv
         fa=fre(iv)*t/2.d0
         ff=dsin(fa)/fa
         do i=1,n
            zdr(iv,i)=zdr(iv,i)*ff
         enddo
      enddo
      return
      end
C
C     ---------------------
      SUBROUTINE BL_Z (ZDR)
C Displacements * Multiply spectrum with basic Love numbers
C     ---------------------
C
C Multiplies the displacement spectrum with the Basic Love numbers
C and turns BLMULT-option off.
C
      IMPLICIT    REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2   IFR(0:9,1200)
      COMPLEX*16  ZDR(1200,*)
      CHARACTER   NAME*6
      LOGICAL     Q_BLMULT

      COMMON     /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IFR
      COMMON     /EPAR/   R0,RA,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG
     &,                   RHOW
      SAVE
      MCACT=INQ_ACTIVE_COMP()
      MC=MCACT
      if (mc.le.0.or.mc.gt.3) mc=3
      IOFF=0
      print *,'<BL_Z-->>> No. of components active: ', mcact
      NAME='BL_Z--'
      GOTO 1
C
      ENTRY BL_P (ZDR)
C Secondary potential * Multiply spectrum with basic Love number
      MC=1
      IOFF=2
      NAME='BL_P--'
C
1     IF (.NOT.Q_BLMULT()) THEN
         print '(3a)'
     &,' <',NAME,'>>> Skipped since option was OFF'
         return
      endif

      DO 101 J=1,MC
      JJ=MIN0(2,J)
      XLOVE=BL(JJ+IOFF)
      DO 101 I=1,NV
101   ZDR(I,J)=ZDR(I,J)*XLOVE
      PRINT '(4A)'
     &,' <',NAME,'>>> Spectrum multiplied with Basic Love nmbrs.'
      print '(3A,2I3,3F10.4)'
     &,     ' <',NAME,'>>> MC, IOFF, BL= '
     &,      MC,IOFF,(BL(JJ+IOFF),JJ=1,MIN0(2,MC))

      CALL BLMULT ('NO ')
      RETURN
      END
