      double precision function dgcdis (x1,y1,x2,y2,vrad)
c
c computes distance along great circle between (lon,lat) = 
c (x1,y1) and (x2,y2)
c vrad (input) contols metric units:
c vrad = 1.d0   - locations and returned function value are in degrees.
c vrad = pi/180 - locations ... in radians.
c
c dgcdis_c assumes the coordinate y1 unchanged w.r.t. previous call.
c Therefore it's a little faster, but it needs initialisation with a 
c call  d=dgcdis(x1,y1,x2,y2,vrad)
c then  d=dgcdis_c(x2,y2)
c
c a=dgazi() can be called after d=dgcdis[_c](..) 
c It's always a bit slow.
c
      implicit real*8 (a-h,o-z)
      complex*16 zaz
      logical qe,qt,qa
      common /mpar/ pi,rad
      save
      if (dabs(pi-3.d0).gt..2d0) then
         rad=datan(1.d0)/45.d0
         pi=180*rad
      endif

      xx1=x1
      xx2=x2
      yy1=y1
      yy2=y2
      
      urad=rad/vrad
      bcy=dcos(y1*urad)
      qe=.false.
      qt=.false.
      goto 1

      entry dgcdis_c (x2,y2)
      bcy=dcos(yy1*urad)
      xx2=x2
      yy2=y2
      qe=.true.
      qt=.false.
      goto 1

      entry dgcdis_t (x2,y2)
      xx2=x2
      yy2=y2
      qe=.false.
      qt=.true.

 1    syh=dsin((y2-yy1)*urad/2.d0)
      sxh=dsin((x2-xx1)*urad/2.d0)
      scy=dcos(y2*urad)
      ccc=bcy*scy
      sqh=syh*syh+sxh*sxh*ccc
      sth=dsqrt(sqh)
      if (qt) then
         dgcdis_t=sth
         return
      endif
      qa=.false.
      goto 2

      entry dgcazi_t ()
      qa=.true.

 2    dgc=2*dasin(sth)
      if (qa) goto 3
      if (qe) then
         dgcdis_c=dgc/urad
      else
         dgcdis=dgc/urad
      endif
      return

      entry dgcazi () ! returns the last gcdis-call's azimuth
c
c remainder calculates azimuth 
 3    cosdis=dcos(dgc)
      sindis=dsin(dgc)
      ssx=dsin(xx2*urad)
      bsx=dsin(xx1*urad)
      scx=dcos(xx2*urad)
      bcx=dcos(xx1*urad)
      ssy=dsin(yy2*urad)
      bsy=dsin(yy1*urad)
c      scy=dcos(yy2*urad)
c      bcy=dcos(yy1*urad)
c      slosb=bcx*ssx-bsx*scx
      slosb=bsx*scx-bcx*ssx
      zaz=dcmplx((bsy-cosdis*ssy)/sindis/scy
     &,          bcy*slosb/sindis)
      azi=dfatg(zaz)
      dgcazi=azi/urad
      return

      entry dgazi () ! returns the last gcdis-call's azimuth
c                      at the "other" point
c
c remainder calculates azimuth 
 4    cosdis=dcos(dgc)
      sindis=dsin(dgc)
      ssx=dsin(xx1*urad)
      bsx=dsin(xx2*urad)
      scx=dcos(xx1*urad)
      bcx=dcos(xx2*urad)
      ssy=dsin(yy1*urad)
      bsy=dsin(yy2*urad)
      scy=dcos(yy1*urad)
      bcy=dcos(yy2*urad)
c      slosb=bcx*ssx-bsx*scx
      slosb=bsx*scx-bcx*ssx
      zaz=dcmplx((bsy-cosdis*ssy)/sindis/scy
     &,          bcy*slosb/sindis)
      azi=dfatg(zaz)
      dgazi=azi/urad
      return
      end

      real function gcdis (x1,y1,x2,y2,rad)
      real*8 dgcdis
      gcdis=dgcdis(dble(x1),dble(y1),dble(x2),dble(y2),dble(rad))
      return
      end

C:Provides  real function gcdis F=gcdis.f
C:Provides  double precision function dgcdis F=gcdis.f
