C     -------------------------------------
      REAL*8 FUNCTION Colatitude (X,Y,XRAD)
C UTIDE S * Convert geographic to geocentric
C     -------------------------------------
C>
C Converts geographic to geocentric angles.
C
C X,Y  - real*8 - on call:   longitude, latitude
C                 on return: longitude, geocentric latitude in radians.
C XRAD - real*8 - if 1.0d0:  input values are in degrees
C                 if RAD:    input values are in radians
C Returned value:            geocentric colatitude in radians.
C_
C Use, e.g.:
C READ *, X,V
C Y=Colatitude (X,V,1.0d0)
C CALL Ela_Ell_Earth (X,Y,ZSP)
C
      IMPLICIT REAL*8 (A-H,O-Z)
      logical qpr
      save
      COMMON /CGEOC/  mmsg,jmsg
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW
      COMMON /MPAR/   PI,RAD
      Y=Y*RAD/XRAD
      X=X*RAD/XRAD
      W=DATAN((1.D0-ESQUAR)*DTAN(Y))
      V=PI/2.D0-W
      Colatitude=V
      jmsg=jmsg+1
      qpr=jmsg.le.mmsg
      if (.not.qpr) return
      PRINT '(A,T48,2F12.5)',' <COLAT->>> Input  long.,  geograf.lat.: '
     &,                        X/RAD,Y/RAD
      PRINT '(A,T60,2F12.5)',' <COLAT->>> Output geocntr.lat., colat.: '
     &,                        W/RAD,V/RAD
      RETURN
      END
C
C
C     ----------------------------------------
      REAL*8 FUNCTION Geog_Latitude (X,Y,XRAD)
C UTIDE S * Convert geocentric to  geographic
C     ----------------------------------------
C>
C Converts geocentric to geographic angles.
C
C X,Y  - real*8 - on call:   geocentric longitude, colatitude in radians.
C                 on return: longitude, geocentric colatitude
C XRAD - real*8 - if 1.0d0:  returned values are in degrees
C                 if RAD:    returned values are in radians
C Returned value:            geographic latitude
C_
C Use, e.g.:
C READ *, X,Y,Z
C CALL XYZ_TO_GEOC (X,Y,Z,AX,AY)
C AW=GEOG_LATITUDE (AX,AY,1.D0)
C CALL OCEAN_LOAD_TIDE (AX,AW, Close_Enough, L_unit, Option(:3), Zspectrum)
C
      IMPLICIT REAL*8 (A-H,O-Z)
      logical qpr
      COMMON /CGEOC/  mmsg,jmsg
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW
      COMMON /MPAR/   PI,RAD
      save
      jmsg=jmsg+1
      qpr=jmsg.le.mmsg
      if (qpr) PRINT '(A,T48,2F12.5)'
     &,              ' <GEOGLA>>> Input long.  geocntr.colat.: '
     &,                X/RAD,Y/RAD
      X=X*XRAD/RAD
      W=DATAN(1.D0/(1.D0-ESQUAR)*DTAN(PI/2.D0-Y))
      Geog_Latitude=W*XRAD/RAD
      Y=Y*XRAD/RAD
      if (qpr) PRINT '(A,T60,2F12.5,:/A,F12.5)'
     &,              ' <GEOGLA>>> Output geograf.lat.  geocntr.colat.: '
     &,                W/RAD,Y/XRAD
CC     &,' <GEOGLA>>> Units: semicircle=',180.d0*XRAD
      RETURN
      END
C
C
C     -------------------------------------
      SUBROUTINE XYZ_TO_GEOC (SX,SY,SZ,X,Y)
C UTIDE S * Convert XYZ to geocentric angles
C     -------------------------------------
C>
C Converts geocentric  x y z  to  longitude, colatitude
C_
      IMPLICIT REAL*8 (A-H,O-Z)
      logical qpr
      COMMON /CGEOC/  mmsg,jmsg
      COMMON /MPAR/   PI,RAD
      save
      X=DFATG(dcmplx(SX,SY))
      S=DSQRT(SX*SX+SY*SY)
      Y=PI/2.D0-DATAN(SZ/S)
      jmsg=jmsg+1
      qpr=jmsg.le.mmsg
      if (qpr) PRINT '(A,T24,3F12.1)',' <XYZGEO>>> X Y Z (m):'
     &,                                 SX,SY,SZ
      if (qpr) PRINT '(A,T48,2F12.5)',' <XYZGEO>>> Long. Colat.:'
     &,                                 X/RAD,Y/RAD
      RETURN
      END
C
C
C     -------------------------------------
      SUBROUTINE GEOC_TO_XYZ (X,Y,SX,SY,SZ)
C UTIDE S * Convert geocentric angles to XYZ
C     -------------------------------------
C>
C Converts geocentric coordinates to XYZ on the IERS-ellipsoid.
C
C X,Y      - real*8 -    input: longitude, colatitude (rad)
C SX,SY,SZ - real*8 - returned: X,Y,Z (m)
C_
      IMPLICIT REAL*8 (A-H,O-Z)
      logical qpr
      COMMON /CGEOC/  mmsg,jmsg
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW
      COMMON /MPAR/   PI,RAD
      save
      ABSQ=(FE/(FE-1.D0))**2
      IF (DABS(90.D0-Y).LT.1.D-3) THEN
         SZ=0.D0
         SX=RE*DCOS(X)
         SY=RE*DSIN(X)
      ELSE
         TY=DTAN(Y)
         TYSQ=TY*TY
         SZ=RE/DSQRT(TYSQ+ABSQ)
         SX=RE/DSQRT(ABSQ/TYSQ+1.0D0)
         SY=SX*DSIN(X)
         SX=SX*DCOS(X)
      ENDIF
      jmsg=jmsg+1
      qpr=jmsg.le.mmsg
      if (qpr) PRINT '(A,T48,2F12.5)',' <GEOXYZ>>> Long. Colat.:'
     &,                                 X/RAD,Y/RAD
      if (qpr) PRINT '(A,T24,3F12.1)',' <GEOXYZ>>> X Y Z (m):'
     &,                                 SX,SY,SZ
      RETURN
      END

      SUBROUTINE XYZ_TO_GEOG (X,Y,Z,H,XLON,XLAT)
C UTIDE S * Convert XYZ to geographic
C>
C Given XYZ coordinates, this routine returns
C H    - height above IERS ellipsoid  [m]
C XLON - longitude on  "       "     [rad]
C XLAT - latitude  "   "       "       "
C
C according to IERS Standards.
C
C Critical: ESQUAR and FE=1/F must be consistent,
C           ESQUAR=2F-F^2, 1/F=298.257
C           (1993: F definition has higher priority than ESQUAR)
C_
      IMPLICIT REAL*8 (A-H,O-Z)
      LOGICAL QPR
      COMMON /CGEOC/ mmsg,jmsg
      COMMON /EPAR/  R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW
      COMMON /MPAR/  PI,RAD
      save
      F1=1.D0-1.D0/FE
      PQ=X*X+Y*Y
      P=DSQRT(PQ)
      R=DSQRT(PQ+Z*Z)
      COSP=0.D0
      XLON=DFATG(DCMPLX(X,Y))
      IF (P.GT.1.D-12) THEN
         TANM=Z/P*(F1+ESQUAR*RE/R)
         IF (DABS(Z).GT.1.D-12) THEN
            COSM=1.D0/DSQRT(1.D0+TANM*TANM)
            SINM=COSM*TANM
            TANP=(Z*F1+ESQUAR*RE*SINM**3)/F1/(P-ESQUAR*RE*COSM**3)
            COSP=1.D0/DSQRT(1.D0+TANP*TANP)
            SINP=COSP*TANP
            XLAT=DATAN(TANP)
         ELSE
            XLAT=0.D0
            SINP=0.D0
            COSP=1.D0
         ENDIF
      ELSEIF (Z.GT.0.D0) THEN
         XLAT=PI/2.D0
         SINP=1.D0
      ELSE
         XLAT=-PI/2.D0
         SINP=-1.D0
      ENDIF
      H=P*COSP+Z*SINP-RE*DSQRT(1.D0-ESQUAR*SINP*SINP)
      jmsg=jmsg+1
      qpr=jmsg.le.mmsg
      IF (QPR) PRINT '(A,F12.6,2H m,F12.6,F9.4,8H deg <- ,3F12.3,2H m)'
     &, ' <XYGeog>>> H lon lat :',H,XLON/RAD,XLAT/RAD,X,Y,Z
      RETURN
      END

      SUBROUTINE GEOG_TO_XYZ (DH,DLON,DLAT,X,Y,Z)
C UTIDE S * Convert geographic to XYZ (from Wenzel)
C
C Entry GEOG_TO_GEOC converts to Radius, Longitude, Latitude
C                                 [m]      [rad]     [rad]
      IMPLICIT REAL*8 (A-H,O-Z)
      LOGICAL QPR,QGEOC
      COMMON /CGEOC/ mmsg,jmsg
      COMMON /EPAR/  R0,DEA,GE,EROT,BL(3),GM,GRVLAT,DEE,FE,BG,RHOW
      COMMON /MPAR/  DPI,DRAD
      SAVE
      QGEOC=.false.
      GOTO 1

      ENTRY GEOG_TO_GEOC (DH,DLON,DLAT,X,Y,Z)
      QGEOC=.true.

 1    F1=1.D0-1.D0/FE
C----------------------------------------------------------------------C
C     DEFINITION OF ELLIPSOIDAL PARAMETERS OF GEODETIC REFERENCE       C
C     SYSTEM 1980.                                                     C
C     DEA  IS MAJOR SEMI AXIS IN METER.                                C
C     DEE  IS SQUARE OF FIRST ECCENTRICITY (DIMENSIONLESS).            C
C----------------------------------------------------------------------C
      DEA=6378137.00D0
      DEE=6.69438002290D-3
      DR0=DEA*(1.D0-DEE/6.D0-5.D0*DEE**2/72.D0-DEE**3*55.D0/1296.D0)
      DCLAT=DCOS(DLAT)
      DSLAT=DSIN(DLAT)
      DN=DEA/DSQRT(1.D0-DEE*DSLAT**2)
      DPSI=DATAN(((DN*(1.D0-DEE)+DH)*DSLAT)/((DN+DH)*DCLAT))
      DR1=DSQRT((DN+DH)**2*DCLAT**2+(DN*(1.D0-DEE)+DH)**2*DSLAT**2)
      DHC=DH
      DLONC=DLON
      DLATC=DLAT
      IF (QGEOC) THEN
         X=DR1
         Y=DLONC
         Z=DPSI     ! LATITUDE!
         RETURN
      ENDIF
      X=DR1*DCOS(DPSI)*DCOS(DLONC)
      Y=DR1*DCOS(DPSI)*DSIN(DLONC)
      Z=DR1*DSIN(DPSI)
      jmsg=jmsg+1
      qpr=jmsg.le.mmsg
      IF (QPR) PRINT '(A,F12.6,2H m,F12.6,F9.4,8H deg -> ,3F12.3,2H m)'
     &, ' <GeogXY>>> H lon lat :',DHC,DLONC/DRAD,DLATC/DRAD,X,Y,Z
C      if (qpr) print *,'dpsi=',dpsi/drad
      RETURN
      END

      subroutine geoc_msg (nmsg)
c UTIDE S * set geocoord message limit
      save
      COMMON /CGEOC/ mmsg,jmsg
      jmsg=0
      mmsg=nmsg
      return
      end

      block data bgeoc
      COMMON /CGEOC/ mmsg,jmsg
      data mmsg/9999999/,jmsg/0/
      end

C:Provides  subroutine xyz_to_geoc F=geocoord.f
C:Provides  subroutine geoc_to_xyz F=geocoord.f
C:Provides  subroutine xyz_to_geog F=geocoord.f
C:Provides  subroutine geog_to_xyz F=geocoord.f
C:Provides  subroutine geoc_msg F=geocoord.f
C:Provides  real*8 function colatitude F=geocoord.f
C:Provides  real*8 function geog_latitude F=geocoord.f
C:Provides  block data bgeoc F=geocoord.f
