      subroutine get_ol_rec (iun,opt,site,monu,xlon,ylat,salt,z
c OL BLQ * Get an OL record
     &,                      sitel,cf,cc,*)
c>
c SITE - char**  - name
c MONU - char**  - monument number
c XLON - real*4  - site location, longitude
c YLAT - real*4  - ..., latitude (deg)
c SITEL- char*48 - site long name
c CF   - char*77 - comment concerning data history
c CC   - char*77 - comment concerning program(mer)
c OPT  - char**  - search options (first encounter)
c                ' '- (empty) take next record
c                 N - like ' ', but impose site, monu and location from
c                     the call line.
c                 S - find by SITE (name).
c                 U - like S, do UpperCase on names first.
c                 M - find by MONUment number.
c                 T - find by SITE .OR. MONU.
c                 V - like T, do UpperCase on names first.
c                SM - find by SITE .AND. MONU. 
c             X Y P - find by coordinates less then ACCuracy (deg)
c                     X - longitude, Y - latitude, P - both (by distance)
c                     CALL Get_Ol_LocAcc (ACCuracy) to redefine value.
c
c                 O - Same TANG orientation between file and Z(.,.)
c                     (which is the unusual case !)
c                 E - Suppress EOF-Warning
c
c                 search option: best
c                 C - find by closest distance to xlon,ylat, min ACC
c                     CALL Get_Ol_LocAcc (ACCuracy) to redefine value.
c
c                 Additional options after a colon: (under construction)
c                +P - extended printing (under construction)
c             +A -A - switch on/off auto scale, 
c                     I.e. apply 1/factor for a record that
c                     contains  SCF=factor  in the CF record
c                     (first $$-line below SITE MONU)
c
c Z(11,3) - complex - Returned: load coeff for tide 1..11, components
c                     Rad, West, South  or
c                     Rad, East, North  if 'O' in OPT
c
c Subroutine
c Call put_ol_rec (...,opt,...,cf,cc)
c OPT     - char*1  - O - give file data Rad,East,North orientiation.
c
c OPT H will write HARPOS records, where comments CF CC appear only once.
c OPT HC will refresh writing of comments CF CC
c
c
c Parameter setting entries:
c
c Call get_ol_locacc (acc)
c ACC     - real    - Site location accuracy with OPT e {C|P|X|Y} (deg)  [0.01]
c
c Call put_ol_orient (qq)
c QQ      - logical - .true. with OPT='O..' - suppress messages about
c                     changed orientation.
c
c Call put_ol_phacut (angle)
c ANGLE   - real    - Phase values in file; phase circle is 
c                     cut at the specified angle                        [180.0]
c
c Call put_ol_ampfac (ampfac)    Call get_ol_ampfac (ampfac)
c AMPFAC  - real    - File is written with amplitudes multiplied 
c                     by ampfac                                           [1.0]
c
c Call put_ol_comps (n_components,string)  write 1 or 3 components
c N_COMPONENTS - integer - number of components (1 or 3), 
c                          specifying 2 will result in 3                    [3]
c STRING       - char**  - text string like 'RADI TANG'                   [' ']
c
c Call put_ol_mti (n_tides)           
c N_TIDES      - integer - write 1 to N_TIDES partial tides                [11]
c
c Call put_ol_sitefmt (format)
c FORMAT       - char**  - format code for writing site and monument
c                          The monument code ought to be positioned
c                          with the T code, and the site be written
c                          with indefinte length; see the default
c                          (avoids to write site name right-justified).
c                                                              ['(2x,a,t13,a)']
c
c Call end_ol_rec (iun)
c                        - Write appropriate end-header record.
c_
      include   'colrec.fca'
      character site*(*), monu*(*)
     &,         sitel*48, cc*77, cf*77, rec*77, opt*(*)
      complex   z(*)

      real*8    xst,yst,xss,yss,dgcdis,cdis,cacc
      character sitf*24, monf*16, sitt*24, buff*80, fn*64, blank*8
      complex   zrad,polar

      dimension buff(11), rer(n_oltides,3)
      logical   qrs,qcs,qsd,qfcs,qo,qor_index,qneof,qmsg,qaus
     &,         qerrec,qerrecs,qerrecf,q_substring,qmonu

      real*8           pi,rad
      common    /mpar/ pi,rad

      complex*8 zvc(n_oltides,3)
      common /czolre/ zvc

      real*8 xc,yc,zc
      common /csitrc/ xc,yc,zc
      
      save

      data    zrad/(0.0,0.01745329252)/, acc/0.01/
     &,       blank/'        '/,qmsg/.false./
     &,       qerrecs,qerrecf/2*.false./,qaus/.false./


      if (iun.le.0) then
         do j=1,olnrw
            do k=1,olmti
               z(k+(j-1)*olmti)=zvc(k,j)
            enddo
         enddo
         return
      endif

      lo=index(opt,':')-1
      if (lo.lt.0) lo=len(opt)
      if (index(opt(lo:),'+P').gt.0) qmsg=.true.
      if (index(opt(lo:),'-P').gt.0) qmsg=.false.
      if (index(opt(lo:),'+A').gt.0) qaus=.true.
      if (index(opt(lo:),'-A').gt.0) qaus=.false.
      
      lm=index_l_nonblank(monu)
      if (qmsg) then
         print '(2a,2f10.4,a,2i5)'
     &,        ' <GOlRec>>> Opt=',opt(:lo),xlon,ylat
     &,             ',  MTI,NRW=',olmti,olnrw
         print '(3a)',' <GOlRec>>> call line Site,monu=',site,monu(:lm)
      endif
      if (qor_index(opt(:lo),'lL',i)) then
         if (qerrecs) then
            do i=1,olmti
               do j=1,olnrw
                  z(i+(j-1)*olmti)=rer(i,j)
               enddo
            enddo
            call replace_in_string (sitel,'!Conf.Obs','!Observed',1,0)
         else
            print '(a)',' <GOlRec>E> No error record for this site' 
         endif
         return
      endif
      qo=qor_index(opt(:lo),'oO',i)
      qneof=qor_index(opt(:lo),'eE',i)
      call put_ol_orient (qo)
      if (qor_index(opt(:lo),'STUV',ier)) then
         ls=index_l_nonblank(site)
c         call replace_in_string (site,'_',' ',1,ls)
         call replace_in_string (site,'$',' ',1,ls)
         call replace_in_string (site,']',' ',1,ls)
      endif
      qcs=index(opt(:lo),'C').gt.0
      qsd=index(opt(:lo),'P').gt.0
      if (qcs.or.qsd) then
         olcdis=1.d30
         xss=xlon*rad
         yss=ylat*rad
         cacc=acc*rad
         qfcs=.false.
      endif
c
c Read a block
1     do 111 i=1,11
111   read (iun,'(a)',end=901) buff(i)
c
c Get site coordinates
      read (buff(5),2103) rec
      lsl=index(rec,'lon/lat:')-1
      j=lsl+9
      read (rec(j:),'(2f10.0,f8.0)',end=109,err=109) xst,yst,ast
c
c we'll need that eventually      
c      if (q_substring(rec,'RADI',' ',sbs,i,k)) then
c         if (sbs.eq.'TANG') then
c            call put_ol_comps (2,'RADI TANG ')
c         else
c            call put_ol_comps (1,'RADI ')
c         endif
c      elseif (q_substring(rec,'RADI',' ',sbs,i,k)) then
c         call put_ol_comps (1,'GRAV ') 
c      endif        
      
      if ((qcs.or.qsd).and..not.qerrecf) then
         cdis=dgcdis (xss,yss,xst*rad,yst*rad,rad)
         olcdis=dmin1(olcdis,cdis)
         if (cdis.gt.cacc) goto 1
         if (qcs) cacc=dmin1(cacc,cdis)
         qfcs=.true.
      endif
      qerrec =index(buff(5),'!Conf.').gt.0
      qerrecf=index(buff(5),'!Observed').gt.0

      xs=xst
      ys=yst
      as=ast
      sitel=rec(:lsl)
c      read (buff(1),2101)
      l1=index_l_nonblank(buff(2))
      l2=max0(len(site)+3,index_l(buff(2)(:l1),' '))
      if (l2.lt.3) l2=l1+1
      sitf=buff(2)(3:l2-1)
      if (l2.lt.l1) then
         monf=buff(2)(l2+1:l1)
         qmonu=.true.
      else
         monf=' '
         qmonu=.false.
      endif
c      read (buff(2),2102) sitf,monf
      read (buff(3),2103) cf
      read (buff(4),2103) cc
      qerrecs=.false.

      ampsca=1.
      if (q_substring(cf,'SCF=','$] ',rec,i,ii)) then
         read (rec(i:),*) ampsca
         if (.not.qaus) ampsca=1.
         print '(a,1p,e12.4,a,l1)'
     &,        ' <GOlRec>>> SCF assumed for only this record: '
     &,          ampsca,' since QAUS=',qaus
      endif

      do 112 j=1,olnrw
112   read (buff(5+j),2104) (olamp(i,j),i=1,olmti)
      do 113 j=1,olnrw
113   read (buff(8+j),2104) (olpha(i,j),i=1,olmti)
      do 202 j=1,olnrw
      do 201 i=1,olmti
      if (.not.qerrec.or.(qerrec.and..not.(qcs.or.qsd))) then
      if (i.gt.olmti) then
         z(i+(j-1)*olmti)=(0.0,0.0)
      else
         olamp(i,j)=olamp(i,j)/olampfac/ampsca
         if (.not.qo.and.j.gt.1) then
            z(i+(j-1)*olmti)=-olamp(i,j)*exp(zrad*olpha(i,j))
         else
            z(i+(j-1)*olmti)=olamp(i,j)*exp(zrad*olpha(i,j))
         endif
      endif
      else
      if (i.gt.olmti) then
         rer(i,j)=(0.0,0.0)
      else
         olamp(i,j)=olamp(i,j)/olampfac/ampsca
         rer(i,j)=olamp(i,j)
         qerrecs=.true.
      endif
      endif
201   continue
202   continue
      if (qerrecs) print *,'CFL data has been stored'
      if (qerrecs) 
     &  call replace_in_string (sitel,'!Conf.Obs','!Observed',1,0)
      
c      if (qmsg) print '(a)',' <getolr>>> values assigned o.k.'
      if (index(opt(:lo),'N').gt.0) goto 198
2101  format (1h )
c2102  format (2x,a8,a6)
2103  format (3x,a)
2104  format (1x,11f7.0)
      if (qerrecf.and.(qcs.or.qsd)) goto 1
      qrs=.true.
      if (qor_index(opt(:lo),'SUTV',inx)) then
         sitt=sitf
         if (index(opt(:lo),'UV').gt.0) call ucase(sitf,sitt)
         if (inx.gt.2) then
            qrs=qrs
     &         .and.(site(:ls).eq.sitt(:ls).and.site(:ls).ne.blank(:ls))
     &          .or.(index(monf,monu(:lm)).gt.0
     &                               .and.index_l_nonblank(monu).gt.0)
     &              .or.(monu(:lm).eq.monf(:lm)
     &                               .and.index_l_nonblank(monu).gt.0)
ccc            print '(1h>,a,1h<)', site(:ls),sitt(:ls),monu,monf
         else
            qrs=qrs.and.site(:ls).eq.sitt(:ls)
         endif
         if (.not.qrs) goto 1
      endif
      if (index(opt(:lo),'M').gt.0) then
         qrs=qrs.and.index(monf,monu(:lm)).gt.0
         if (.not.qrs) goto 1
      endif
      if (index(opt(:lo),'X').gt.0) then
         qrs=qrs.and.abs(xs-xlon).lt.acc
         if (.not.qrs) goto 1
      endif
      if (index(opt(:lo),'Y').gt.0) then
         qrs=qrs.and.abs(ys-ylat).lt.acc
         if (.not.qrs) goto 1
      endif

      site=sitf
      monu=monf
      if (.not.qmonu) monu=' '
      xlon=xs
      ylat=ys
      salt=as

      if (qcs) goto 1

 199  dism=cacc/rad
 198  if (qmsg) then
         lf=index_l_nonblank(cf)
         lc=index_l_nonblank(cc)
         print '(2a)',' <GOlRec>>> ',cf(:lf),' <GOlRec>>> ',cc(:lc)
         print '(5a)',' <GOlRec>>> ',site,monu,' ',sitel
         print '(2(a,2f10.4,f8.2),a)'
     &,        ' <GOlRec>>> lon/lat: ',xlon,ylat,salt
     &,        '; min.dist., loc.acc.:',dism,acc,' [deg]'
         print '(a,2i5)',' <GOlRec>>> Rows, columns ',olnrw,olmti
         do i=1,olmti
            print '(2a,3(3p,f10.2,0p,f8.1))'
     &,           ' <GOlRec>>> ',oltides(i)
     &,           (polar(z(i+(j-1)*olmti),0.0),j=1,3)
         enddo
      endif
      call geog_to_xyz (dble(salt),xlon*rad,ylat*rad,xc,yc,zc)
      return

901   if ((qsd.or.qcs).and.qfcs) goto 199
      if (.not.qneof.and.(qsd.or.qcs)) then
         print *,'<GOlRec>>> EOF ',iun,' ',opt,' ',site,monu,xlon,ylat
      endif
      if (qmsg) print *,'<GOlRec>>> EOF ',iun
      return 1

109   print '(a/1x,a)',' <GOlRec>T> Bad site location record:'
     &, rec
      fn=' '
c      call get_file_name (iun,fn)
      lfn=index(fn//' ',' ')-1
      print '(a,i2,1x,a)',' <GOlRec>I> File=',iun,fn(:lfn)
      print '(a)',' <GOlRec>I> Buffer was:',(buff(i),i=1,olmti)
      stop

      entry get_ol_locacc (xx)
      acc=xx
      return
      end
      
      real function get_ol_distance (degrees)
      include 'colrec.fca'
      get_ol_distance=olcdis*degrees
      return
      end

      subroutine put_ol_rec (iun,opt,site,monu,xlon,ylat,salt,z
     &,                      sitel,cf,cc)

      include 'colrec.fca'
      character   site*(*),monu*(*),
     &            sitel*48,cf*77,cc*77, rec*80, opt*(*)
     &,           comps*16, f2102*48, buff*80, dsymb*7, opt4*4
      dimension   iuns(10), osign(3)
      real*8      dalt,dlon,dlat
      complex     z(*), zv, polar
      logical     qo, qor_index, qq, qr, qri, qpr, qblq, qharpos
      integer*2   argn(8)
c
      real*8      xc,yc,zc
      common /csitrc/ xc,yc,zc
      real*8      pi,rad
      common /mpar/   pi,rad
      save

      data phacut/180.0/, qr/.false./, comps/' '/, lcm/0/
     &, qpr/.false./, f2102/'(2x,a,t13,a)'/
     &, qblq/.true./, qharpos/.false./
     &, iuns/10*0/

      print *,'<polrec>d> olmti=',olmti

      qo=qor_index(opt,'oO',i)
      if (qo) then ! this is for HARPOS
c                    if opt = 'O' we have to flip the sign
c                    (in BLQ it is flipped if opt =/= 'O' )
         osign(1)=1.d0
         osign(2)=-1.d0
         osign(3)=-1.d0
      else
         osign(1)=1.d0
         osign(2)=1.d0
         osign(3)=1.d0
      endif
      qri=qor_index(opt,'rR',i)
      qpr=qor_index(opt,'pP',i)
      qharpos=qor_index(opt,'hH',i)
      if (qpr) then
         print *,'<puorec>d> qri,qpr,qharpos=',qri,qpr,qharpos
         print *,'<puorec>d> olmti=',olmti
      endif
      if (qharpos) then
         qblq=.false.
         if (qor_index(opt,'cC',i)) then
            do i=1,10
               iuns(i)=0
            enddo
         endif
      endif
      
      if (qo.neqv.qr) then
         print '(a,l1)',' <PuORec>w> Orientation Qo=',qo
         qr=qo
      endif
      lcf=index_l_nonblank(cf)
      lcc=index_l_nonblank(cc)
      lsl=index_l_nonblank(sitel)
      ls=index_l_nonblank(site)
      lm=index_l_nonblank(monu)
      l0=max0(index_1_nonblank(monu),1)
      if (sitel(1:1).eq.'?') then
         rec='$$ '
         write (rec(4:),'(a,1x,a)') site(:ls), monu(l0:lm)
         lrec=index_l_nonblank(rec)
         print *,'<puolre>d> site ? rec ->',rec(:lrec),'<-'
      else
         rec='$$ '//sitel(:lsl)
         lrec=index_l_nonblank(rec)
         print *,'<puolre>d> site known, rec ->',rec(:lrec),'<-'
      endif

      lrec=index_l_nonblank(rec)
       i=-999
      if (lcm.gt.0) then
         i=40-lcm
         if (i.gt.lrec+1) rec(i:)=comps(:lcm)
         lrec=i+lcm+1
      endif
      if (qpr) print *,'<puolre>d> lrec,lcm,i=',lrec,lcm,i

c      jjmx=min0(2,olnrw)
c      if (lsl.lt.37-5*jjmx.and.lcm.gt.0) then
c         i=max0(lsl+3,40-lcm)
c         rec(i:)=comps(:lcm)
c      endif

c      print *,'<puolre>>> lsl,jjmx,lcm,i=',lsl,jjmx,lcm,i
c      print *,'<puolre>>> sitel=',sitel

      rec(42:)='lon/lat:'
      write (rec(50:),'(2f10.4,f9.3)') xlon,ylat,salt
      lrec=index_l_nonblank (rec)
      lcfu=lcf
      print*,'<puolre>d> 1:: ',olampfac,lrec,lcfu
      print*,'<puolre>d> 1:: rec=',rec(:lrec)
      if (abs(olampfac-1.).gt.0.01) then
         write (cf(lcf+2:),'(4hSCF=,1p,e9.1)') olampfac
         call delete_in_string (cf,' ',lcf+2,0)
         lcfu=index(cf(lcf+2:),' ')+lcf+1
         cf(lcfu:)=' '
         print *,'<puolre>d> 2:: ',olampfac,lcf,lcfu
         print *,'<puolre>d> 3:: ',cf
      endif

      if (qpr) then
         buff=' <PuORec>>> site monu ->'
         write (buff(25:),f2102) site(:ls),monu(l0:lm)
         lb=index_l_nonblank(buff)+1
         buff(lb:)='<-'
         print '(a)',buff(:lb+2)
      endif
      if (qblq) then
         write (iun, 2101)
         write (iun,f2102) site(:ls),monu(l0:lm)
c2345c789012345678901234567890123456789012345678901234567890123456789012.......|
         write (iun, 2103) cf(:lcfu),cc(:lcc)
         write (iun,'(a)') rec(:lrec)
         if (qpr) print *,'<puorec>d> olnrw=',olnrw
         if (qri) then
            do 201 j=1,olnrw
 201        write (iun,2201) 
     &            (olampfac*blqama(i)*z(i+(j-1)*olmti),i=1,olmti)
         else
            do 101 j=1,olnrw
               do 101 i=1,olmti
                  if (.not.qo.and.j.gt.1) then
                     zv=polar(-z(i+(j-1)*olmti),phacut)
                  else
                     zv=polar(z(i+(j-1)*olmti),phacut)
                  endif
                  olamp(i,j)=real(zv)
                  if (olamp(i,j).lt.1.e-10) then
                     olpha(i,j)=720.0
                  else
                     olpha(i,j)=aimag(zv)
                  endif
 101        continue
            do 102 j=1,olnrw
 102        write (iun,2104) (olampfac*blqama(i)*olamp(i,j),i=1,olmti)
            do 103 j=1,olnrw
 103        write (iun,2105) (olpha(i,j),i=1,olmti)
         endif
         print *,'<PutOlR>>> Scaled by:',(olampfac*blqama(i),i=1,3)
      elseif (qharpos) then
         kun=0
 151     kun=kun+1
         if (kun.le.10) then
            if (iuns(kun).gt.0.and.iuns(kun).ne.iun) goto 151
            if (iuns(kun).le.0) then
               write (iun,'(1h#)') 
               write (iun,'(3h#F ,a)') cf
               write (iun,'(3h#C ,a)') cc
               iuns(kun)=iun
            endif
         endif
c         rs=dsqrt(xc*xc+yc*yc+zc*zc)
c         if (rs.lt.6.36d6.or.rs.gt.6.39d6) then
            dlon=xlon*rad
            dlat=ylat*rad
            dalt=salt
            call geog_to_xyz (dalt,dlon,dlat,xc,yc,zc)
c         else
            call xyz_to_geog (xc,yc,zc,dalt,dlon,dlat)
c         endif
         ls=index_l_nonblank(site)
         write (iun,3102) site(1:ls),xc,yc,zc,dlat/rad,dlon/rad,dalt
c         print *,'(polrec)d) ',olmti,olnrw
c         print '(3a,i3)',' (polrec)d) s ->',site(:ls),'<-',ls
         do j=1,olmti
            if (index_l_nonblank(oltides(j)).gt.0) then
c               call darwin_to_doodson (oltides(j),dsymb)
               call lcase (oltides(j),dsymb)             ! instead of doodson
               lds=max0(4,index_l_nonblank(dsymb))       ! min 4 to output
               write (iun,3101) dsymb(:lds),site(:ls)
     &,         (olampfac*osign(i)* real(z(j+(i-1)*olmti)),i=1,olnrw)
     &,         (olampfac*osign(i)*aimag(z(j+(i-1)*olmti)),i=1,olnrw)
            endif
         enddo
         write (iun,'(1h#)') 
      endif
2101  format ('$$')
c2102  format (2x,a,1x,a)
2103  format ('$$ ',a)
2104  format (1x,11(1x,f6.5))
2105  format (1x,11(1x,f6.1))
2201  format (22f9.4)

3101  format ('D  ',a,t14,a,t24,3f9.5,t53,3f9.5)
3102  format ('#'/'S  ',a,t13,3f14.4,t56,2f9.4,f7.1/'#')
      return

      entry put_ol_clear ()
      xc=0.d0
      yc=0.d0
      zc=0.d0
      return

      entry end_ol_rec (iun)
      if (qharpos) then
         write (iun,'(a)') 'HARPOS Format version of 2002.12.12'
      else
         write (iun,'(a)') '$$ END TABLE'
      endif
      return

      entry put_ol_orient (qq)
      if (qr.neqv.qq) print '(a)'
     &,  ' <PuOOri>w> Henceforth changed orientation'
      qr=qq
      return

      entry put_ol_phacut (vv)
      phacut=vv
      return

      entry put_ol_ampfac (vv)
      entry get_ol_ampfac (vv)
      olampfac=vv
      print *,'<..OlAm>>> Amplitude scale:' ,olampfac
      return
      
      entry put_ol_comps (jj,monu)
      olnrw=3
      if (jj.eq.1) olnrw=1
      comps=monu
      lcm=index_l_nonblank(comps)
      return

      entry put_ol_mti (jj)
      olmti=jj
      return

      entry store_ol_tsym (jj,opt4)
c
c jj   - integer - in/out  - location
c opt4 - char*4  - in      - symbol
c
c If jj >= 1, store tide symbol in location jj.
c If jj <= 0, find a new location, store the symbol, and return the location
c 
      if (jj.gt.0) then
         oltides(jj)=opt4
      else
         do i=1,olmti
            if (oltides(i).eq.opt4) return
            if (oltides(i).eq.'    ') then
               oltides(i)=opt4
               return
            endif
         enddo
         jj=i
      endif
      return

      entry put_ol_sitefmt (site)
      if (site(1:1).ne.' ') f2102=site
      return
      end

      subroutine enter_ol_blq (iun,tides,m,n)
c
c Reads the header of an ocean loading blq file and returns
c the tide symbols in the column order, and the number of
c components (GRAVITY: 1, other components (STRAIN or
c DISPLACEMENT: 3)
c
c To that end the subroutine spawns for $$ COLUMN ORDER and
c $$ OCEAN LOADING records.
c
c iun       - file unit number
c tides(11) - character*4 symbols
c m         - number of tides = columns
c n         - number of rows
c
c If ENTER_OL_BLQ is not successful, M is returned 0. Then you can
c CALL DEFAULT_OL_BLQ (TIDES,M,N) to get a default setup.
c That will set both the /CCOLRE/ OLTIDES array and the returned
c TIDES array. In DEFAULT_OL_BLQ (m,N) the N is to be set while
c ENTER_OL_BLQ will return it. The M is always returned.
c Actually, if you set N<=0, DEFAULT_OL_BLQ will return N=3.
c
      include 'colrec.fca'
      
      parameter (ndft=n_oltides-11)
      character tides*4,buff*80,dftides*4
      logical   qloc_in_file
      dimension tides(*),dftides(n_oltides)

      data dftides/'M2  ','S2  ','N2  ','K2  ',
     &             'K1  ','O1  ','P1  ','Q1  ',
     &             'MF  ','MM  ','SSA ',ndft*' '/

c
c Position at the first $$
      rewind (iun)
      m=0
      if (.not.qloc_in_file(iun,'$$','B')) then
         print '(a)',' <EOlBlq>E> Not a BLQ file'
         rewind (iun)
         return
      endif

      n=3
      do i=1,olmti
         tides(i)=' '
         oltides(i)=' '
      enddo

      read (iun,'(a)') buff
      call ucase (buff,buff)
      if (index(buff,'GRAVITY').gt.0) n=1
      if (qloc_in_file(iun,'$$ COLUMN ORDER',' ')) then
         backspace (iun)
         read (iun,'(a)') buff
         i=index(buff,':')+1
         do j=1,olmti
         i=index_1_nonblank(buff(i:))+i-1
         l=index(buff(i:)//' ',' ')+i-2
         if (i.eq.l) goto 9
         tides(j)=buff(i:l)
         oltides(j)=buff(i:l)
         i=l+1
         enddo
      else
         print '(a)'
     &,        ' <EOlBlq>E> Tide symbols (column order) not found'
         j=1
      endif
 9    m=j-1
      olmti=m
      olnrw=n

      rewind (iun)
      return
      
      entry default_ol_blq (tides,m,n)
      if (n.gt.0) then
         olnrw=n
      else
         olnrw=3
      endif
      
      olmti=11
      if (m.gt.0) olmti=m
      do i=1,olmti
         oltides(i)=dftides(i)
           tides(i)=dftides(i)
           print *,'<DOlBlq>>> Default tide:',i,' ',oltides(i)
      enddo
      return
      end

      block data bolrec
      include 'colrec.fca'

      data    blqama/n_oltides*1.0/
      data    olampfac/1.0/, olnrw/3/, olmti/11/
      data    oltides/n_oltides*'    '/
      end
C:Provides  subroutine get_ol_rec F=getolrec.f
C:Provides  subroutine put_ol_rec F=getolrec.f
C:Provides  subroutine enter_ol_blq F=getolrec.f
C:Provides  entry get_ol_locacc F=getolrec.f
C:Provides  entry end_ol_rec F=getolrec.f
C:Provides  entry put_ol_orient F=getolrec.f
C:Provides  entry put_ol_phacut F=getolrec.f
C:Provides  entry put_ol_ampfac F=getolrec.f
C:Provides  entry get_ol_ampfac F=getolrec.f
C:Provides  entry put_ol_comps F=getolrec.f
C:Provides  entry put_ol_mti F=getolrec.f
C:Provides  entry store_ol_tsym F=getolrec.f
C:Provides  entry put_ol_sitefmt F=getolrec.f
C:Provides  entry default_ol_blq F=getolrec.f
C:Provides  real function get_ol_distance F=getolrec.f
C:Provides  block data bolrec F=getolrec.f
