      INTEGER FUNCTION JULDAT (ID)
C UTIDE U * Julian date of date
C>
C This function returns the Julian date of the kalendar date ID(1..3) =
C Year, month, day with respect to 1899.12.31. The Julian date that begins at
C noon of the given Gregorian date is returned.
C_
      INTEGER ID(3)
      INTEGER KAL(12)
      COMMON /CY2KS/ IY2KS
      DATA KAL/0,31,59,90,120,151,181,212,243,273,304,334/
      DATA IY2KS/1900/
      JULDAT=0
      ISCL=0
      
      IF (ID(2).GT.12) THEN
         ID(1)=ID(1)+IFIX((ID(2)-1.0)/12.0+0.001)
         ID(2)=MOD(ID(2)-1,12)+1
      ENDIF
      IF (ID(1).GT.1900) THEN
         IHLP=ID(1)-1900
      ELSE
         IF (ID(1).LE.MOD(IY2KS,100)) ID(1)=ID(1)+100
         IHLP=MOD(ID(1),100)
         IF (ID(1).LT.200) ID(1)=ID(1)+1900
         ISCL=(19-ID(1)/100)*36525
      ENDIF
      IF (ID(1).LT.0.OR.ID(2).LT.0.OR.ID(3).LT.0) RETURN
      IF (ID(1).GE.2100) RETURN
      J=IHLP*365+nint((IHLP-1)/4.-.5)+ID(3)-ISCL
      IM=ID(2)
      IF (IM.GT.0) THEN
         J=J+KAL(IM)
         IF (ID(2).GT.2.AND.MOD(ID(1),4).EQ.0) J=J+1
      ENDIF
      JULDAT=J+15019
      RETURN
      END

      SUBROUTINE SET_Y2K_SWITCH (I)
      COMMON /CY2KS/ IY2KS
      IY2KS=I
      RETURN
      END

      double precision function djuldat (id,ihms)
      implicit real*8 (a-h,o-z)
      dimension id(3),ihms(4)

      jul=juldat(id)
      hms=((ihms(4)/1.d3+ihms(3))/60.d0+ihms(2))/60.d0+ihms(1)
      djuldat=jul+hms/24.d0
      return
      end

      SUBROUTINE DATJUL (JUL,IDAT)
C UTIDE U * Inverse of function JULDAT
C
      DIMENSION IDAT(3),JDAT(3)
      INTEGER MONDAY(12)
      DATA MONDAY/0,31,59,90,120,151,181,212,243,273,304,334/
      JH=JUL-15018
      IDAT(1)=IFIX(JH/365.25)
      IF (JH.LT.0) IDAT(1)=IDAT(1)-1
      IREM=IFIX(JH-IDAT(1)*365.25)
      IDAT(1)=IDAT(1)+1900
      IF (IREM.EQ.0) GOTO 9
      I=13
1     I=I-1
      M=MONDAY(I)
      IF (MOD(IDAT(1)+1000,4).EQ.0.AND.I.GE.3) M=M+1
      IF (IREM-M.LE.0) GOTO 1
      IDAT(2)=I
      IDAT(3)=IREM-M
      RETURN
9     IDAT(2)=12
      IDAT(3)=31
      IDAT(1)=IDAT(1)-1
      RETURN
      END
C
      SUBROUTINE DOYJUL (JUL,IDAT)
C UTIDE U * Inverse of function JULDAT
C
      DIMENSION IDAT(3),JDAT(3)
      INTEGER MONDAY(12)
      DATA MONDAY/0,31,59,90,120,151,181,212,243,273,304,334/
      JH=JUL-15018
      IDAT(1)=IFIX(JH/365.25)
      IF (JH.LT.0) IDAT(1)=IDAT(1)-1
      IREM=IFIX(JH-IDAT(1)*365.25)
      IDAT(1)=IDAT(1)+1900
      IF (IREM.EQ.0) GOTO 9
      I=13
1     I=I-1
      M=MONDAY(I)
      IF (MOD(IDAT(1)+1000,4).EQ.0.AND.I.GE.3) M=M+1
      IF (IREM-M.LE.0) GOTO 1

      IDAT(2)=I
      IDAT(3)=IREM-M
      JULD=JUL
      GOTO 2

      ENTRY DOYDAT (IDAT)
      JULD=JULDAT(IDAT)

 2    JDAT(1)=IDAT(1)
      JDAT(2)=0
      JDAT(3)=0
      JD=JULDAT(JDAT)
      IDAT(3)=JULD-JD
      IDAT(2)=0
      RETURN

9     IDAT(2)=0
      IDAT(3)=0

      RETURN
      END
C
      SUBROUTINE HMS (TIME,IT,ISG)
C UTIDE  U * Convert decimal hours to HH MM SS FFF
C>
C Given the time TIME in decimal hours, this subroutine returns in IT(1..4)
C the time in the form
C hours, minutes, seconds, rounded milliseconds in IT(1..4).
C The fractional seconds are rounded to 1/100 of one second.
C If TIME < 0, ISG=1 else ISG=2
C
C Leap-second processing:
C If the day ends in a leap second:
C    CALL Leap_Second_HMS (1)
C    CALL HMS (Time, IHMS, ISG)
C    CALL Leap_Second_HMS (0)
C
C or
C    CALL HMS_Leap_Second (Time, IHMS, ISG, 1)
C
C The range for |TIME| should be 0 .. 24.0d0 
C_
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER IT(4)
      DATA LS/0/

      ISG=2
      IF (TIME.LT.0.D0) ISG=1
      H=DABS(TIME)
      S=H*3.6d3
      IT(4)=IDINT(DMOD(S+0.5d-3,1.d0)*1000)

      IF (LS.NE.0) THEN
         IF (DABS(TIME-24.d0).lt.1.d0/3.6d3) THEN
            IT(1)=23
            IT(2)=59
            IT(3)=60
c            print *,'<JULDAT>>> This is a leap second!'
            RETURN
         ELSE IF (TIME.GT.24.d0) THEN
            H=H-LS/3.6d3
            S=S-LS
         ENDIF
      ENDIF
      IS=IDINT(S+0.5d-4)
      IT(3)=MOD(IS,60)
      IM=IDINT(H*60.d0+0.5d-6)
      IT(2)=MOD(IM,60)
      IH=IDINT(H+0.5d-8)
      IT(1)=IH
      RETURN

      entry Leap_Second_HMS (isg)
      ls=isg
      return
      END

      SUBROUTINE HMS_Leap_Second (TIME,IT,ISG,LEAP)
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER IT(4)
      CALL Leap_Second_HMS (LEAP)
      CALL HMS (TIME,IT,ISG)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION THMS (IT,ISG)
C UTIDE U * Convert HH MM SS FFF to decimal hours
C>
C Inverse of subroutine HMS: Converts HH,MM,SS,ff to decimal hours
C 
C OBS! This does not permit adding of leap-seconds. Use e.g.
C
C     Hour = THMS(IHMS, ISG) + Leap_Second(DATE, Is_Such_A_Day)
C                      ( ~hgs/sas/p/sasu01.f )
C_
      INTEGER IT(4)
      REAL*8 H
      H=IT(1)
     & +(DFLOAT(IT(2))+(DFLOAT(IT(3))+DFLOAT(IT(4))/1000.D0)/60.D0)
     & /60.D0
      IF (ISG.LE.1) H=-H
      THMS=H
      RETURN
      END

      SUBROUTINE KALENDAR (JUL,T,DATE,FTIME)
C UTIDE U * Julian Kalendar + hours
C>
C (1) Returns DATE and HH MM SS FFF of day of Julian day JUL + decimal hour T.
C
C (2) Returns JUL  and HH MM SS FFF of day DATE + decimal hour T  
C     if JUL <= -9999
C
C FTIME = "formatted time", Hour,Minute,Second, 1ms, integers.
C
C This routine does not work with leap-seconds
C Use sas_kalendar instead. 
C_
      INTEGER DATE(3),FTIME(4)
      REAL*8 T,TT,XT
      IF (JUL.LE.-9999) JUL=JULDAT(DATE)
      JULT=JUL+IDINT(T/24.D0)
      if (t.lt.0.) jult=jult-1
      CALL DATJUL (JULT,DATE)
      TT=DMOD(T+48.D0,24.D0)
      CALL HMS(TT,FTIME,ISIG)
      RETURN
      END


C:Provides  subroutine datjul F=juldat.f
C:Provides  subroutine hms F=juldat.f
C:Provides  subroutine kalendar F=juldat.f
C:Provides  integer function juldat F=juldat.f
C:Provides  double precision function thms F=juldat.f
