      program      test_tide
c
c If nsites=1 computes tide displacement (alt. other components) 
c time series at a location along components Radial East North (REN).
c
c If nsites=2 computes baseline changes along geocentric XYZ components.
c or, alt., baseline LTU components
c
c Can compute displacement tides
c             strain tides
c             tilt tides
c             gravity tides (rigid earth)
c             gravity tides (elastic earth and liquid core)
c             inertial accelerations due to displacements
c             coriolis       "        "  "       "
C
C 2014-06-18 Updated: Tilt added. Not tested yet.
C            If ocean loading is added, it's BLQ rows 2 & 3 
C            that are added to BT columns 2 & 3, and 1 remains empty.
C            OBS! signs in BT, signs in OLT not checked yet.
C            OBS! Are the functions PNM DPNM o.k. for n=1..6 ???
c _________________________________________________________
c
c Instruction file ['ttimms.ins']:
c Consists of one NAMELIST and one OPEN FILE BLOCK
c That's a minimum:
c _________________________________________________________ 
c  &param
c  &end
c    * Openf - block
c 21 B ttimm.tsf                   Output time series
c    Q
c _________________________________________________________
c
c Namelist &param       parameters                                 [defaults]
c ___________________________________________________________________________
c
c tpath     - char64  - Path/filename of the harmonic tide development        !
c                                                      ['etgtab/etcpot.dat '] !
c idate(3)  - integer - year, month, day  (e.g. idate=1993,12,6  
c                       for Dec 6, 1993)                         [1994,01,01]
c
c Defining the site(s):
c nsites    - integer  - 1 or 2 for single site or baseline mode,
c                        respectively.                                    [1]
c xsite(2)
c ysite(2)
c zsite(2)  - real*8  - coordinates for sites 1 and 2
c q_xyz     - logical - .true.  -  coordinates are geocentric XYZ in meters.
c                                                                   [.false.]
c q_lola    - logical - .true.  -  coordinates are longitude latitude height
c                                  in degrees                       [.false.]
c
c Defining the component:
c q_se_tide - logical - compute solid earth tide.                    [.true.]
c q_ol_tide - logical - compute ocean loading tide.                  [.true.]
c q_gravity - logical - gravity tides elast earth w core            [.false.]
c q_re_grav - logical - gravity tides rigid earth                   [.false.]
c q_strain  - logical - strain components                           [.false.]
c q_tilt    - logical - tilt components                             [.false.]
c q_permanent_tide    - logical -                                   [.false.]
c q_acc     - logical - compute linear accelerations                [.false.]
c q_corio   - logical - compute coriolis accelerations              [.false.]
c
c q_acc and q_corio cannot be combined with q_gravity, q_re_grav or q_strain 
c 
c
c The sites can be taken from the ocean loading table:
c snm_opt(2)- char*4  - option for finding site by name (call ocean_load_site):
c                       'M' - monument code
c                       'S' - site name
c                       'N' - site name AND monu code
c                       'U' - name comparison in upper case chars
c                       'R' - one rewind of file per call.
c                       ' ' - don't call ocean_load_site
c                             (q_lola or q_xyz and the coordinates must
c                             be given explicitly.)
c site_name(2) - char*16 - 
c monu_code(2) - char*6  -
c                       
c pmt_opt   - char*4  - options for print_major_tides
c                       'H' - show relative phase w.r.t. astronomical potential
c
c Ocean loading options:
c ol_opt    - char*4  - option parameter for subroutine Ocean_Load_tide,
c                       composed of
c                       'I' - interpolate spectrum
c                       'P' - extended print:  loading parameters
c                       'R' - even more print: response parameters
c                       'Q' - quiet, 'q' - nearly quiet
c                       '.' - use default.
c                       Opposed to 'I'-option, the sparse loading spectrum
c                       is returned, consisting only of the tabelled data.
c                                                                    ['I...']
c
c The output time series:
c nt        - integer  - requested length of time series output.        [300]
c dt        - real*8   - time interval in seconds.                  [3600.d0]
c t0        - real*8   - time from midnight in seconds                [0.0d0]
c
c ncomp_out - integer  - 1 or 3 
c                        For BLQ gravity with tilt, if you don't 
c                        want tilt, specify 1                   [BLQ decides]
c
c out       - char*3   -          default =                           ['UEN']
c                        'LTU'  - baseline tides:  Length Transverse Up
c                        'XYZ'  - baseline or single site, geocentric tides.
c
c                        For gravity or potential 'UEN' must be used.  
c                        'UEN'  - single site tides:  Up East North
c                                 'REN' can be used as a synonym.
c
c                        'UEN'  - baseline tides: Up East North;
c                                 you can select the reference site:
c n_refsite - integer  - 1 or 2 - the site where the unit vectors UEN
c                                 are computed
c                        other  - the point above the baseline center 
c                                 is used.                                [0]
c
c fmtout    - char*64  - fortran format code   
c                                      ['(i6,f6.5,1x,i4,2i3.2,#i3.2,3f10.4)']
c                        Inspect the default:
c                          i6,f6.5 is for a fractional Julian date
c                          i4,2i3.2 is for  year, month, day
c                          # will become the number khms
c                          the remainder is for the ordinate values.
c
c khms      - integer  - indicates whethert the hour(1) minute(2) second(3) 
c                        part is to be written                            [1]
c
c sco(3)    - real*8   - scale output                                 [1.0d0]
c
c File to open (subroutine openf)
c
c 21 - output, ASCII, time series, optional:
c      Time [h]  radial east north.
c
c 22 - output, ASCII, optional
c      horizontal vs vertical.
c
c 31 - input, optional
c      A previous output file, in order to produce differences.
c
c 32 - input, optional
c      First component (radial), added to this procedure's output
c 33,34 second and third component
c_
      implicit     real*8      (a-h,o-y)
     &,            complex*16  (z)
      complex*16   dpolar
      character*4  snm_opt(2), pmt_opt, snmo
      character*16 site_name(2), sn, ol_opt
      character*6  monu_code(2), mn
      character    tpath*64,out*3,fmt32*64, units*32, myunits*32
     &,            fmtout*64,buff*64, utc_file*64, fname*64, comps*32

      logical      q_ol_tide, q_se_tide, q_xyz, q_lola, q_daily_ave
     &,            qc,qp,qa,qltu,qxyz,qo11,qo21,qo22,qo32,q_abs_tide
     &,            q_gravity,q_re_grav,q_strain,q_tilt,q,qo41
     &,            q_permanent_tide, qzp21, qraw_amp, q_acc,q_corio
     &,            q_inject_leaps

      integer      get_video_mode, text_mode, prompted_pause
      integer*2    delete(0:9,100), select(0:9,100), tide_k1(0:9)

      dimension    idate(3), ihms(4), zdr(1200,3,2), zol(1200,3)
     &,            dxmax(3), dxo(3), dx(3), btsndr(3), sco(3)
     &,            ldate(3), leaps(100),leapj(100)

      real*8       xsite(2), ysite(2), zsite(2)

      namelist     /param/
     &             idate, xsite,ysite,zsite, q_xyz, q_lola, myunits
     &,            nsites, nt, crtdis, out, fmt32, a1,a2
     &,            q_permanent_tide, pmt_opt, n_refsite, ncomp_out
     &,            t0,dt, q_ol_tide, q_se_tide, ol_opt, q_daily_ave
     &,            site_name, monu_code, snm_opt, tpath, eff_ave
     &,            delete,select,ndel,nsel,q_abs_tide, utc_file
     &,            q_gravity,q_re_grav, q_strain, q_acc,q_corio
     &,            q_tilt
     &,            fmtout, khms, sco, n_print, msg_level
     &,            iun_spectrum, qraw_amp, amp_zp21, pha_zp21, btsndr
     &,            q_inject_leaps

      common       /mpar/ pi,rad

      integer*2    iaa
      common /castsp/ amp(1200),pha(1200),fre(1200),nv,iaa(0:9,1200)

      data         nt/300/, idate/1994,01,01/, t0/0.0d0/,dt/3600.0d0/
     &,            out/'UEN'/, pmt_opt/' '/, msg_level/4/
     &,            xsite/2*0.0d0/, ysite/60.0d0,20.0d0/, nsites/1/
     &,            zsite/2*0.0d0/
     &,            q_ol_tide/.false./, crtdis/0.5d0/, ol_opt/'I...'/
     &,            q_se_tide/.false./, q_xyz/.false./, q_lola/.false./
     &,            snm_opt/' ','?'/, tpath/'etgtab/etcpot.dat '/
     &,            fmt32/'(i6,f10.2)'/, a1/1.0d0/, a2/1.0d-3/
     &,            q_daily_ave/.false./, eff_ave/0.0d0/, ncomp/3/
     &,            comps/'RADI EAST NORTH'/, units/'[m] [m] [m]'/
     &,            delete,select/2000*-99/, ndel/0/,nsel/0/
     &,            q_abs_tide/.false./,q_acc,q_corio/2*.false./
     &,            q_gravity,q_re_grav,q_strain/3*.false./
     &,            fmtout/'(i6,f6.5,1x,i4,2i3.2,#i3.2,3f10.4)'/, khms/1/
     &,            sco/3*1.0d0/,q/.true./, zi/(0.0d0,1.0d0)/
     &,            utc_file/'./etgtab/ttutcf.dat '/, n_print/20/
     &,            q_permanent_tide/.false./, n_refsite/0/
     &,            iun_spectrum/-1/, qraw_amp/.false./
     &,            amp_zp21,pha_zp21/0.d0, 720.d0/
     &,            tide_k1/2,1,1,0,0,0,0,0,0,1/
     &,            btsndr/3*-9999.d0/, scale/1.d0/
     &,            ncomp_out/-1/, myunits/' '/, isc/1/
     &,            q_inject_leaps/.false./
     &,            leapj/100*99999/

           iro=26
           iuterm=10
           iuins=4


      call start_time_message ('/home/hgs/Ttide/p/m/ttimm.f$')
      call getenv ('term',buff)
      q=buff(1:1).ne.' '

      call open_term (iuterm)
      call open_ins  (iuins,'ttimm.ins')

           read (iuins, nml=param)
           ier=msglevel_ts(msg_level)

      call openf     (iuins)

      inquire (unit=21, opened=qo21)
      inquire (unit=22, opened=qo22)
      inquire (unit=32, opened=qo32)
      inquire (unit=31, opened=qc)

           iro = iro+1

           qzp21=pha_zp21.lt.361.d0.and.pha_zp21.gt.-361.d0

        if (.not.(q_re_grav.or.q_gravity)) then
      call show_prm          ('SHDO')
      call show_prm          ('WAHR')
      call show_prm          ('NDR ')
       end if

           open (unit=3, file=tpath)

      call set_epoch          (idate)
           julep = juldat     (idate)

      call tt_utc_file        (12, utc_file)
      ier= leap_seconds_from_file (12, utc_file, .false.)
      
      jlsd=0
      rewind (12)
        do j = 1,100
           read (12,'(i4,3i3)',end=181) ldate,leaps(j)
           if (ldate(1).le.0) goto 181
           leapj(j) = juldat(ldate)
           if (leapj(j).gt.julep.and.jlsd.eq.0) jlsd=j
       end do
 181  rewind (12)
      call datjul (leapj(jlsd),ldate)
      print *,'<Main-->>> Next Leap second: ',jlsd,ldate,leapj(jlsd)

      call set_ep_j2000       (idate)
      call set_permanent_tide (q_permanent_tide)
      call astronomic_thd     (3)
        if (iun_spectrum.gt.0) 
     &     call write_astronomic_thd (iun_spectrum,qraw_amp)
        if (amp_zp21.lt.1d-30) then
           call ndrose (0,(0.d0,0.0d0))
           call ndrose (1,(0.d0,0.0d0))
       end if
        if (qzp21) then
                zp21 = amp_zp21 * exp(zi*pha_zp21*rad)
           call ndrose (0,zolndr_val(0,tide_k1,zp21))
           call ndrose (1,zolndr_val(1,tide_k1,zp21))
       end if

       do i = 1,3
          if (btsndr(i).gt.-9000.d0) call ndrbts(i-1,btsndr(i))
      enddo

        if (snm_opt(2)(1:1).eq.'?') snm_opt(2)=snm_opt(1)

      call ucase (out,out)
           qltu = out.eq.'LTU' .and. nsites.gt.1
           qxyz = out.eq.'XYZ' .and. nsites.eq.1
           
           do 10 is=1,nsites

        if (q_xyz) then
           rs=v_2_norm_v (xsite(is),ysite(is),zsite(is))
      call xyz_to_geoc   (xsite(is),ysite(is),zsite(is), xs,ys)
      call xyz_to_geog   (xsite(is),ysite(is),zsite(is), hgs,xgs,ygs)
           dpsi=ygs+ys-pi/2.d0
           xgs = xgs/rad
           ygs = ygs/rad
      else if (q_lola) then
           xgs = xsite(is)*rad
           ygs = ysite(is)*rad
           hgs = zsite(is)
      call geog_to_xyz   (hgs,xgs,ygs, xsite(is),ysite(is),zsite(is))
           rs=v_2_norm_v (xsite(is),ysite(is),zsite(is))
      call xyz_to_geoc   (xsite(is),ysite(is),zsite(is), xs,ys)
           dpsi= ygs-ys+pi/2.d0
           xgs = xgs/rad
           ygs = ygs/rad
      endif

           sn = site_name(is)
           mn = monu_code(is)
           snmo=snm_opt(is)

      inquire (unit=11,opened=qo11)
        if (qo11.and.(.not.(q_xyz.or.q_lola)
     &           .or.snmo(1:1).ne.' ')
     &           .and.snmo(1:1).ne.'%') then ! (find site by name)
           ls = index_l_nonblank(sn)
           call ocean_load_site (11,snmo,sn(:ls),mn,xgs,ygs)
           xgs = xgs*rad
           ygs = ygs*rad
           hgs = 0.d0
      call geog_to_xyz   (hgs,xgs,ygs, xsite(is),ysite(is),zsite(is))
           rs=v_2_norm_v (xsite(is),ysite(is),zsite(is))
      call xyz_to_geoc   (xsite(is),ysite(is),zsite(is), xs,ys)
           dpsi= ygs+ys-pi/2.d0
           xgs = xgs/rad
           ygs = ygs/rad

       end if ! (find site by name)
      
      call blmult            ('YES')

         mcomp = 1

      call print_tides_opt (pmt_opt) 
        if      (q_se_tide) then
          if    (q_gravity) then

           call set_tide_type (1,'Gravity','[uGal]')
           call set_active_comp (1)
           call gravity_tide (xs,ys,zdr(1,1,is))
           call print_major_tides (n_print,zdr(1,1,is),1,1)

         elseif (q_re_grav) then

           call set_tide_type (1,'Gravity','[uGal]')
           call set_active_comp (1)
                q_ol_tide=.false.
           call re_gravity_tide (rs,ys,xs,dpsi,zdr(1,1,is))
           call print_major_tides (n_print,zdr(1,1,is),1,1)

         elseif (q_tilt) then

           isc=2
           call set_tide_type (3,'Tilt','[nrad]') 
           call set_active_comp (3)
           call tilt_earth (xs,ys,zdr(1,1,is))
           call print_major_tides (n_print,zdr(1,1,is),3,isc)
           scale=1.d9
           mcomp=2

         elseif (q_strain) then

           call set_tide_type (3,'Strain','[nano]')
           call set_active_comp (3)
           call strain_earth (xs,ys,zdr(1,1,is))
           call print_major_tides (n_print,zdr(1,1,is),3,1)
           scale=1.d9

         else

           call set_tide_type (3,'Displacements','[m]')
           call set_active_comp (3)
           call displacement_tide (xs,ys,zdr(1,1,is))
           call print_major_tides (n_print,zdr(1,1,is),3,1)

         endif
      else
           call setzd0 (zdr(1,1,is),1200,3)
       end if ! (q_se_tide)

        if      (q_ol_tide) then
           if (q_strain) scale=1.d9

           call ocean_load_tide   (xgs,ygs,crtdis,11,ol_opt,zol)
           call comb_earth_ocean  (zdr(1,1,is),zol)
       end if

      call get_tide_type (ncomp,comps,units)
      call set_active_comp (ncomp)
      
        do k=1,3
        if (abs(sco(k)-1.).gt.1e-8) then
           if (index_l_nonblank(myunits).gt.0) units=myunits
       end if
       end do

      call edit_zdr (zdr(1,1,is),ncomp,delete,ndel,select,nsel)

        if (.not.q_ol_tide) then
           print *,'<Main-->>> don''t forget the love numbers !'
           call bl_z              (zdr(1,1,is))
       end if 

      call inertial_tide (q_acc,q_corio,xs,ys,zdr(1,1,is),units(1:8))
           
      call print_major_tides (n_print,zdr(1,1,is),3,isc)

        if (q_daily_ave) eff_ave=24.d0
        if (eff_ave.gt.1.d-10) then
           call ave_tide (zdr(1,1,is),eff_ave)
       end if

        if (nsites.gt.1.or.qxyz) then
           call set_local_at      (xs,ys)
           do 30 iv=1,1200
30    call zv_local_to_geoc  (zdr(iv,1,is),zdr(iv,2,is),zdr(iv,3,is)
     &,                       zdr(iv,1,is),zdr(iv,2,is),zdr(iv,3,is))
c
c Test where UEN unit vectors at site 2 point when seen from site 1
c
        if (is.eq.2) then
      call zv_local_to_geoc  ((1.d0,0.d0),(0.d0,0.d0),(0.d0,0.d0)
     &,                           z11,        z12,        z13    )
      call zv_local_to_geoc  ((0.d0,0.d0),(1.d0,0.d0),(0.d0,0.d0)
     &,                           z21,        z22,        z23    )
      call zv_local_to_geoc  ((0.d0,0.d0),(0.d0,0.d0),(1.d0,0.d0)
     &,                           z31,        z32,        z33    )
       end if
c
       end if
           xsite(is) = xs
           ysite(is) = ys
10     continue              ! end loop over sites

        if (qltu) then
           call set_local_at (xsite(1),ysite(1))
           call set_chord_to (xsite(2),ysite(2))
           call make_ltu_rot
      else if (.not.qxyz) then
           if (n_refsite.le.0.or.n_refsite.gt.2) then

              call geog_to_xyz (0.d0,xsite(1),pi/2.d0-ysite(1)
     &,                         s1x,s1y,s1z)
              call geog_to_xyz (0.d0,xsite(nsites),pi/2.d0-ysite(nsites)
     &,                         s2x,s2y,s2z)
              call xyz_to_geoc ((s1x+s2x)/2.d0
     &,                         (s1y+s2y)/2.d0
     &,                         (s1z+s2z)/2.d0, sx,sy)

              call set_local_at (sx,sy)
           else
              call set_local_at (xsite(n_refsite),ysite(n_refsite))
           endif
           out='UEN'
       end if
       
        if (nsites.gt.1) then
           do 20 i=1,3
           do 20 iv=1,1200
              zdr(iv,i,1) = zdr(iv,i,2)-zdr(iv,i,1)
20         continue
           if (qltu) then
              print '(a)','0<Main-->>> Back to LTU'
              do 21 iv=1,1200
                 call zv_rotate_i (zdr(iv,1,1),zdr(iv,2,1),zdr(iv,3,1)
     &,                            zdr(iv,1,1),zdr(iv,2,1),zdr(iv,3,1))
21            continue
           elseif (.not.qxyz) then
              print '(a)','0<Main-->>> Back to UEN'
              do 31 iv=1,1200
31           call zv_geoc_to_local (zdr(iv,1,1),zdr(iv,2,1),zdr(iv,3,1)
     &,                             zdr(iv,1,1),zdr(iv,2,1),zdr(iv,3,1))

             call zv_geoc_to_local (z11,z12,z13, z11,z12,z13)
             call zv_geoc_to_local (z21,z22,z23, z21,z22,z23)
             call zv_geoc_to_local (z31,z32,z33, z31,z32,z33)

             print '(1p,3(d13.4,d12.4))', z11,z12,z13
             print '(1p,3(d13.4,d12.4))', z21,z22,z23
             print '(1p,3(d13.4,d12.4))', z31,z32,z33
              
           end if
           
           print '(4a,l2)','0<MAIN-->>> Output: ',out,' Baseline tide,'
     &,                    ' qltu =',qltu
      else
           print '(3a)','0<MAIN-->>> Output: ',out,' Local tide'
       end if

      call print_major_tides (n_print,zdr,3,isc)

      inquire (unit=41,opened=qo41)
      if (qo41) then
      do iv=1,nv
         av=abs(zdr(iv,1,1))+abs(zdr(iv,2,1))+abs(zdr(iv,3,1))
         av=av*scale
         zv=amp(iv)*cdexp(zi*pha(iv))
         if (av.gt.1.d-20) then
            zv=amp(iv)*cdexp(zi*pha(iv))/scale
            write (41,4101) fre(iv)/rad/15.d0
     &,                     (dpolar(zdr(iv,j,1)/zv,0.d0),j=1,3)
     &,                     av
         endif
      end do
      end if
 4101 format (f10.6,3(1p,d12.4,0p,f8.2),1p,d12.4)
         
           qp = .true.
           qa = .true.
           do j=1,3
              dxmax(j) = 0.d0
           end do
           
           k=index(fmtout,'#')
           if (k.gt.0) write (fmtout(k:k),'(i1)') khms

        if (ncomp_out.gt.0) ncomp=ncomp_out
      print *,'<Main-->>> Output ncomp=',ncomp
     &,'  Scale factors ',(sco(j),j=1,ncomp)

      print '(12a)',' |',('---------|',i=1,10),char(13)
      print '(1h ,$)'

      call   get_file_name (21,fname(22:))
        lf = index_l_nonblank(fname)
      print  '(2a)',' <Main-->>> Ready ',fname(22:lf)
      write  (fname(:21),'(1p,d20.12,1h_)') dt
      call   delete_in_string (fname,' ',1,0)
        lf = index_l_nonblank(fname)
      call   replace_in_string (fname,'_',' ',1,0)
      write  (21,'(a)') '> -rs'//fname(:lf)

        tleap=0.d0
        do 101 it=0,nt-1
           if (amod((100.*it)/float(nt-1),1.).lt.99./(nt-1))
     &        print '(a,$)','*'
    
           t = it*dt + t0 + tleap

           if (q_inject_leaps) then
              xjul = t/86400.d0+julep
              if (idint(xjul)-1.ge.leapj(jlsd)) then
                 t = t + leaps(jlsd)
                 tleap = tleap + leaps(jlsd)
                 print *,'<Main-->>> Leap second ',jlsd
     &,                  ' injected at ',xjul,' : ',leaps(jlsd)
                 jlsd = jlsd + 1
              endif
           endif
              
        if (q_abs_tide) then
           call abs_tide_at_time  (t,ncomp,zdr,dx)
      else
           call tide_at_time      (t,ncomp,zdr,0,dx)
       end if 

        if (qc) then
           read    (31,'(13x,3f10.4)')    dxo
             do    j=1,ncomp
             dx(j) = dx(j)-dxo(j)/sco(j)
           end do
       end if

        if (qo32) then
              do  iui = 1,ncomp
                  read (31+iui, fmt32, end=932)  kd, dxo(iui)
                  dx(iui) = a1*dx(iui) + a2*dxo(iui)
              end do
       end if

        if (qo22) then
              dxr = dx(1)*1.d3
              dxh = dsqrt(dx(2)**2+dx(3)**2)*1.d3
              write (22,'(2f10.1)') dxh,dxr
       end if
       
         if (qo21) then
              call sas_kalendar (t/3600.d0,jul,idate,ihms)
              if (ihms(1).eq.23.and.ihms(2).eq.59
     &           .and.ihms(3).eq.60) jul=jul+1
              write   (21, fmtout) 
     &                jul,dmod(t/86400d0,1.d0)
     &,               idate,(ihms(j),j=1,khms)
     &,               (sco(j)*scale*dx(j),j=mcomp,ncomp)
       end if
           
        do j=mcomp,ncomp
              dxmax(j) = dmax1(dxmax(j), dabs(scale*dx(j)))
       end do

c        if (qp) then
c              print  '(i8,2x,3f10.4)', it, (dx(j),j=mcomp,ncomp)
c           if (qa.and.mod(it+1,iro-5).eq.0) then
c              if (q) then
c                   key=prompted_pause (0,iro,'Stop Go ?  (S G)->')
c                   qp=index('Ss',char(key)).le.0
c                   qa=index('Gg',char(key)).le.0
c              else
c                   qp=.false.
c              end if
c          end if
c       end if

101   continue
      print '(1h )'

      call   get_file_name (21,fname(22:))
        lf = index_l_nonblank(fname)
      print  '(2a)',' <Main-->>> Ready ',fname(22:lf)
      write  (fname(:21),'(1p,d20.12,1h_)') dt
      call   delete_in_string (fname,' ',1,0)
        lf = index_l_nonblank(fname)
      call   replace_in_string (fname,'_',' ',1,0)
      write  (21,'(a)') '> -rs'//fname(:lf)

932   continue 
      lu=index_l_nonblank(units)
      print '(2a,1h:/t11,1p,3e12.4)'
     &,     ' Max Abs effect ',units(:lu),(dxmax(k),k=mcomp,ncomp)
 
      stop
      end

      subroutine setzd0 (z,m,n)
      complex*16 z(m,n),v0
      v0=(0.0d0,0.0d0)
      do 101 j=1,n
      do 101 i=1,m
101   z(i,j)=v0
      return
      end
 
      subroutine edit_zdr (zdr,nc,del,ld,sel,ls)
c
c ZDR(1200,NC)- tide spectrum, NC=3 for three-dimensional effects like 
c               displacements. NC=0 will default to INQ_ACTIVE_COMP()
c
c DEL(0:9,LD) - argument numbers for deleting some (amount = LD) 
c               specific tides.
c SEL(0:9,LS) - argument numbers for selecting some (amount = LS) 
c               specific tides.
c
c               DEL(j,.)=-99 will ignore the remainder of argument numbers
c               in the selection/delete process.
c               To select only the M2, a namelist section that sets appropriate
c               parameters may look like this:
c                   LD=0
c                   LS=1 
c                   SEL=2,2,0,0,0,0,0,0,0,-99
c               (nobody will hardly want to test on the last argument number 
c               which concerns the +- pi/2 "argument" (sine/cosine selector))
c
c               To select all degree two waves "near" the M2 ("far" = those 
c               frequencies that differ by at least one cycle per month)
c                   LD=0
c                   LS=1 
c                   SEL=2,2,0,7*-99
c
c               To delete all long-period tides (degrees 2 3 4)
c                   LD=3
c                   LS=0
c                   DEL=2,0,8*-99, 3,0,8*-99, 4,0,8*-99
c
c         
      implicit  real*8 (a-h,o-y), complex*16 (z)
      dimension zdr(1200,*)
      integer*2 iaa,del(0:9,*),sel(0:9,*)
      logical   qdel,qsel

      common /castsp/ amp(1200),pha(1200),fre(1200),nv,iaa(0:9,1200)

      kd=0
      ks=0
      n=nc
      if (n.le.0) n=inq_active_comp()
      if (ld.gt.0) then
         do i=1,nv
            do kk=1,ld
               qdel=.true.
               do k=0,9
                  if (del(k,kk).le.-99) goto 101
                  qdel=qdel.and.iaa(k,i).eq.del(k,kk)
                  if (.not.qdel) goto 101
               enddo
               do j=1,n
                  zdr(i,j)=(0.d0,0.d0)
               enddo
               kd=kd+1
               goto 102

 101           continue
            enddo
 102        continue
         enddo
      endif

      if (ls.gt.0) then
         do i=1,nv
            do kk=1,ls
               qsel=.true.
               do k=0,9
                  if (sel(k,kk).le.-99) goto 204
                  qsel=qsel.and.iaa(k,i).eq.sel(k,kk)
                  if (.not.qsel) goto 202
               enddo
 204           if (qsel) goto 201 ! qsel is true
 202           continue  
            enddo
c has been false each time, so delete
            do j=1,n
               zdr(i,j)=(0.d0,0.d0)
            enddo
            ks=ks+1 ! deleted via Select 
            goto 203
               
 201        print *,'selected:',(iaa(k,i),k=0,9)
 203        continue
         enddo
      endif
     
      print '(a,2i5)',' <EditZd>>> deleted by DEL, SEL: ',kd,ks
      print '(a,i5)',' <EditZd>>> active components: ',n
      return
      end


