c NaN routines:
c 
c DNAN() and RNAN() return double or single precision real NaN values, resp.
c
c QNAN(r) QDNAN(d) QCNAN(z) QCDNAN(c) are logical functions that
c return .true. if the argument is NaN (r real*4, d real*8, z complex*8,
c c complex*16). They also report that a NaN has been found so that the
c occurences are counted. The final total can be printed out on a file with
c unit number IUN using
c
c       CALL SUMMARY_NAN(IUN)
c
c NaN producing subprograms can be traced
c       IMPLICT LOGICAL (Q)
c       ...
c       CALL TRACE_NAN ('progname')
c       ...
c       QER=QNAN(r)
c       ...
c       CALL STOP_TRACE_NAN
c
c Up to 20 subprograms can be recorded. If more than 20 names are announced,
c the 20'th register cumulates the NaN counts of the excess requests.
 
      double precision function dnan ()
      real*8  y
      integer s(2),t
      equivalence (y,s(1))
      data t/'7FF80000'X/
      s(2)=t
      dnan=y
      return
      end

      function rnan ()
      real    y
      integer s,t
      equivalence (y,s)
      data t/'FFC00000'X/
      s=t
      rnan=y
      return
      end      
      
      logical function qdnan (x)
      real*8  x,y
      integer s(2),t
      logical q
      equivalence (y,s(1))
      data t/'7FF80000'X/
      y=x
      q=iand(s(2),t).eq.t
      call record_nan (q)
      qdnan=q
      return
      end
      
      logical function qnan (x)
      real    x,y
      integer s,t
      logical q
      equivalence (y,s)
      data t/'FFC00000'X/
      y=x
      q=iand(s,t).eq.t
      call record_nan (q)
      qnan=q
      return
      end
      
      logical function qcnan (z)
      complex*8 z,x
      real      s(2)
      logical   qnan,q
      equivalence (x,s(1))
      x=z
      q=qnan(s(1)).or.qnan(s(2))
      call record_nan (q)
      qcnan=q
      return
      end
      
      logical function qcdnan (z)
      complex*16 z,x
      real*8     s(2)
      logical    qdnan,q
      equivalence (x,s(1))
      x=z
      q=qdnan(s(1)).or.qdnan(s(2))
      call record_nan (q)
      qcdnan=q
      return
      end

      subroutine trace_nan (str)
      character str*(*),subpgm*8,caller*8
      logical   q,qop
      dimension caller(21),kount(0:21),ncall(0:21)

      data subpgm,caller/22*' '/, ncall,kount,iu,ku/46*0/

      if (str(1:1).eq.' ') then
         iu=0
         subpgm=' '
         return
      endif
         
      call ucase (str,subpgm)
      kuu=min0(ku,20)
      do i=1,kuu
         if (subpgm.eq.caller(i)) then
            iu=i
            goto 101
         endif
      enddo
      ku=ku+1
      iu=ku
 101  caller(iu)=subpgm
      return

      entry stop_trace_nan
      iu=0
      return

      entry record_nan (q)
      ncall(iu)=ncall(iu)+1
      if (q) kount(iu)=kount(iu)+1
      return

      entry summary_nan (iuo)
      iun=iuo
      inquire (unit=iun,opened=qop)
      if (.not.qop) iun=6
      write (iun,'(2a)') ' <SumNan>>> last called from ',subpgm
     &,' <SumNaN>>> routine  #NaN    #calls'
      do i=1,ku
         write (iun,'(2a,i5,i10)')
     &         ' <SumNan>>> ',caller(i),kount(i),ncall(i)
      enddo
      write (iun,'(2a,i5,i10)')
     &      ' <SumNan>>> ','nameless',kount(0),ncall(0)
      return
      end
