      SUBROUTINE Ocean_Load_Site (iun,opt,site,monu,xs,ys)
C BTIDE L * get site position from OL table
c
c
c iun       - integer - file unit number; file must be opened by
c                       calling program.
c
c opt       - char**  - options:
c                       'Q'     - quiet
c                       'X'     - return next site
c                       'S'     - find by site name
c                       'M'     - find by monument code
c                       'N'     - find by site name AND monument code
c                       'U'     - character case of site name (upper or
c                                 lower) is not significant.
c                       'L' ' ' - character case is significant
c                       'R'     - one rewind will be done per site
c                       'F'     - fail if the site cannot be found
c
c site      - char**  - significant left substring of site name.
c monu      - char*6  - monument code (optional). String must be right-
c                       justified !
c
c xs,ys     - real*8  - returned: site position, longitude, latitude
c                                 units=degrees.
c                                 Can be passed to Ocean_Load_Tide.
c                       returned: -999,-999  if no site did match specs.
c
      character site*(*), opt*(*),sitef*16, str*80, monu*6, monuf*6
     &,         comps*64
      logical quc,qrwd,qmd,qn,qbn,qbm,qfound,qor_index,qfirst,qnext
     &,       quiet,qer,qloc_in_file
      real*8 xs,ys
      SAVE
      data qfirst/.true./, ncompc/3/

      quc  = qor_index(opt,'uU',i)
      qrwd = qor_index(opt,'rR',i)
      qbm  = qor_index(opt,'nNmM',i)
      qbn  = qor_index(opt,'nNsS',i)
      qn   = qor_index(opt,'nN',i)
      qnext= qor_index(opt,'xX',i)
      quiet= qor_index(opt,'qQ',i)

      if (qrwd.or.qfirst) then
         rewind (iun)
         qer=qloc_in_file (iun,'$$','B')
         read (iun,2103) comps
         call ucase(comps,comps)
         if (index(comps,' GRAVITY').gt.0) ncompc=1
         if (index(comps,' GRAVITY+TILT').gt.0) ncompc=3
         call pos_header (iun)
         qfirst=.false.
      endif
2103  FORMAT (2X,A64)
      
      if (.not.qnext) then
         ls=index(site,']')-1
         if (ls.le.0) ls=index_l_nonblank(site)
         ls=min0(16,ls)
         l3=ls+2
      else
         ls=14
         l3=16
      endif
101   continue
      read (iun,2101,end=901)
      read (iun,2101,end=901) str
      if (quc) then
         call ucase (str(3:l3),sitef)
      else
         sitef=str(3:l3)
      endif
      qmd=.false.
      monuf=' '
      if (qbm) then
         im=index_l_nonblank(str)-5
         if (im.gt.10) then
            monuf=str(im:im+5)
            qmd=.true.
         endif
      endif
      if (qnext) then
         im=index_l_nonblank(str)-5
         if (im.gt.10) monuf=str(im:im+5)
         site=sitef
         monu=monuf
         qfound=.true.
      elseif (qn) then
         qfound  =   sitef(:ls).eq.site(:ls)
     &          .and.qmd.and.monuf.eq.monu
      else
         qfound  =   qbn.and.sitef(:ls).eq.site(:ls)
     &           .or.qbm.and.qmd.and.monuf.eq.monu
      endif
      if (qfound) then
c         print '(2a)',' <OLSite>>> Found: ',sitef(:ls)
         read (iun,2101)
         read (iun,2101)
         read (iun,2101) str
         i=index(str,'lon/lat:')+8
         read (str(i:),*) xs,ys
         do 103 i=1,2*ncompc
103      read (iun,2101)
         if (.not.quiet)
     &      print '(3a,2f9.4)',' <OLSite>>> ',sitef(:ls)//monuf
     &,        ' at lon/lat:  ',xs,ys
         return
      else
         do 102 i=1,3+2*ncompc
102      read (iun,2101)
         goto 101
      endif
901   if (.not.qnext) print '(4a,i2)',' <OLSite>E> ',site,monu
     &,     ' not found on file unit ',iun
      if (qor_index(opt,'Ff',i)) stop
      xs=-999.
      ys=-999.
      return
2101  format (a)
      end

      SUBROUTINE Ocean_Load_Tide (XS,YS,CRTDIS,IUN,OPT,ZOL)
C Loading tides * Compute spectrum for a site
C
C This routine computes the cospectrum of ocean loading effects at a site.
C The cospectrum can directly be used to create a time series of 3-D
C displacements
C              du(t,j) = Real ( ZOL(k,j) * exp (i*FRE(k)*t) )
C where  t  is the time after the initial epoch as specified to subr. ASTRO.
C Use Tide_S_T (SYNTHT.f) for that purpose.
C
C Calling parameters:
C
C XS,YS         real*8      Site longitude, latitude in degrees.
C CRTDIS        real*8      Critical distance to accept a site, deg.
C                           A value < 0 will accept any site.
C IUN           integer     Logical unit number for coefficients table.
C OPT           char**      'I' - Interpolate a complete spectrum,
C                                 otherwise: Return sparse spectrum.
C                           'P' - print the load parameters
C                           'R' - print the response parameters
c                           'Q' - quiet; only print eventual warnings.
c                           'O' - original orientation
C
C Returned:
C
C ZOL(1200,3)    complex*16  Cospectrum. First index: spectrum,
C                           second index: component (radial, east, north).
C
C
C Required:
C ASTRO must have been called.
C The ocean loading table (OC_LOAD file) on unit IUN must have been opened.
C
C The tabelled parameters are for radial, west and south components.
C The required flip to the conventional components is included in this
C programme.
C
C After
C              Ela_Ell_Earth (EARTH.f)    and   Ocean_Load_Tide
C you may call
C              Comb_Earth_Ocean
C (c.f. below) to obtain the spectrum of the combined Solid Earth and
C Ocean loading tide.
C
C If your compiler finds trouble with this code, examine comment records
C starting with "C!".
C
C Check whether your compiler supports INTEGER*1.
C
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      CHARACTER*10    COMP(3)
      CHARACTER*48    F2104
      CHARACTER*8     SITE,FSITE,SSITE
      CHARACTER*3     MTIDE(11),TIDES(11)
      CHARACTER       INSTR*80, BUFF*90, OPT*(*), COMPS*64, UNITS*32
      COMPLEX*16      ZOL(1200,*),ZMT(11,3),DPOLAR
C! You can use standard INTEGER in the following two type declarations:
C     INTEGER*1
      INTEGER*2       INBAND(0:2,3),IAA(0:9,1200),IAAMT(4,11),ITP(11)
     &,               ITPI(11),IIP(11,3),BANDIN(11,3)
      INTEGER*2       ISP(11)
      INTEGER         NA(3),NI(3),NBELOW(3),NBETWN(3),NABOVE(3)
      REAL*8          AMP(1200),PHA(1200),FRE(1200),FMT(11,3)
     &,               SA(3),SP(3)
      REAL            OTAMP(11,3),OTPHA(11,3),PHAP,BTAMP(11),BTPHA(11)
     &,               SCLF(3)
      LOGICAL         L1CALL,QP,qblm,qdispl,qor_index
     &,               QFS,QBS,q_substring,qrsc,qer,qloc_in_file
     &,               qsclfnotset
      COMMON /CASTSP/ AMP,PHA,FRE,NV,IAA
      COMMON /MPAR/   PI,RAD
      common /cttsts/ qblm,qdispl,ncompc

      EXTERNAL        GAMMA

      SAVE
      DATA F2104/'(2x,11f7.0)'/
      DATA QFS/.TRUE./,L1CALL/.FALSE./, NI/3*0/, SINITIAL/1.0D0/
     &,    qsclfnotset/.true./
     &,    TIDES/'SSA',' MM',' MF'
     &,          ' Q1',' O1',' P1',' K1'
     &,          ' N2',' M2',' S2',' K2'/
     &,    IAAMT/ 0, 0, 2, 0,   0, 1, 0,-1,   0, 2, 0, 0
     &,           1,-2, 0, 1,   1,-1, 0, 0,   1, 1,-2, 0,   1, 1, 0, 0
     &,           2,-1, 0, 1,   2, 0, 0, 0,   2, 2,-2, 0,   2, 2, 0, 0/
C
C Pointer arrays:
C ITP(i): if i is the column number in the coefficients table, ITP points to
C         the tide j in the sequence used in array TIDES.
C
C ITPI(j): Inverse pointer to ITP
C
C ISP(j): ISP of tide j points to tide constituent k in the astron. spectrum.
C
C IIP(j,c): Information about component c of tide j is stored in response
C           array ZMT(k,c), k=IIP(j,c)
C
C
C ______________________________________________________________________
C If not L1CALL: I. Prepare____________________________________________/
C
C I.1 Read file header and determine sequence of tides
C
C First record must contain a string denoting the component:
C DISPLACEment STRAIN or GRAVITY
C
C In the case of Gravity, the first or the second record may contain a 
C string denoting a scale factor for the amplitudes. 
C Example record two:
C $$ SCALE: 0.1 mGal
C stating that the amplitude coefficients are given in units of 
C 0.1 mGal = 1 um/s^2
C Since the procedure returns uGal, it will apply 0.1 * 1000 internally.
C mGal is the expected unit. However, um/s^2 is the default scaling (!). 
C The specification of the example can be shortened to $$ SCALE: 0.1
C or changed to $$ SCALE: 1 um/s^2 or omitted since it's the default. 
C The program will recognize these unit codes:
C um/s^2 um/s**2 um*s**-2 um*s^-2 mgal mGal ugal uGal
C
C One header record must contain the string '$$ COLUMN ORDER' 
C and the sequence of tides symbols as they appear in the table, n items, 
C starting at column 18, Format(T18,n(A3,1X)).
C
      QBS=Q_SUBSTRING(OPT,'S:',' ',SSITE,LO,LS)
      LO=LO-1
      IF (LO.LT.0) LO=LEN(OPT)
      QP=.NOT.QOR_INDEX(OPT(:LO),'Qq',i)
      QFS=.NOT.QOR_INDEX(OPT(:LO),'Oo',i)
      IF (L1CALL) GOTO 201
      REWIND IUN
      qer=qloc_in_file (iun,'$$','B')
      READ (IUN,2103) COMPS
      CALL UCASE (COMPS,COMPS)
      if     (index(comps,' DISPLACE').gt.0) then
             sclf(1)=1.
             sclf(2)=1.
             sclf(3)=1.
             sclp=1.
             phap=180.
             comp(1)='..Radial..'
             comp(2)='...Tg E...'
             comp(3)='...Tg N...'
             units='[m]'
             icomp=1
             ncomp=3
             qdispl=.true.
             comps='RADI EAST NORTH'
      elseif (index(comps,' STRAIN').gt.0) then
             sclf(1)=1.e-8
             sclf(2)=1.e-8
             sclf(3)=1.e-8
             if (Qloc_in_File(IUN,'$$ AMPLITUDES','U')) then
                backspace (iun)
                read (iun,'(a)') buff
                if (index(buff,'u-strain').gt.0) then
                   sclf(1)=1.e-6
                   sclf(2)=1.e-6
                   sclf(3)=1.e-6
                   qsclfnotset=.false.
                endif
             endif
             sclp=1.e9
             phap=360.
             comp(1)='...E nn...'
             comp(2)='...E ee...'
             comp(3)='...E ne...'
             units='[nano]'
             icomp=2
             ncomp=3
             qdispl=.false.
             comps='E_nn E_ee E_ne'
      elseif (index(comps,' GRAVITY').gt.0) then
             SINITIAL=-1.D0
             sclf(1)=1.e2 ! default factor   um/s^2 for output in uGal
             sclf(2)=1.e2 ! default factor 0.1 urad for output in nrad
             sclf(3)=1.e2 ! default factor 0.1 urad for output in nrad
             sclp=1.
             phap=360.
             comp(1)='...G n ...'
             if (index(comps,'TILT').gt.0) then
                comp(2)='..TILT E..'
                comp(3)='..TILT N..'
                units='[uGal] [nrad] [nrad]'
                icomp=2
                ncomp=3
                sclf(2)=1.e2
                sclf(2)=1.e2
             else
                comp(2)='..........'
                comp(3)='..........'
                units='[uGal]'
                icomp=2
                ncomp=1
             endif
             qdispl=.false.
             i=1
             do ii=1,2
c                print *,'olt: ii=',ii,' comps ->',comps
                io=i
                i=index(comps(io:),'SCALE:')+io+5
                qrsc=i.gt.io+5
                if (i.le.io+6) i=index(comps(io:),'FACTOR:')+io+6
                qrsc=qrsc.or.i.gt.io+6
                io=i
c                print *,'qrsc=',qrsc,io,i
                if (qrsc) then
                   qsclfnotset=.false.
                   read (comps(i:),*) sclf(ii)
                   if (index(comps(i:i+12),'UM/S**2' ).gt.0.or.
     &                 index(comps(i:i+12),'UM/S^2'  ).gt.0.or.
     &                 index(comps(i:i+12),'UM*S^-2' ).gt.0.or.
     &                 index(comps(i:i+12),'UM*S**-2').gt.0) then
                      sclf(1)=1.e2
                   elseif (index(comps(i:i+12),'URAD').gt.0) then
                      sclf(2)=1.e2
                   elseif (index(comps(i:i+12),'UGAL').gt.0) then
                      continue  ! use the factor unchanged
                   else
                      if (ii.eq.1) then
                         sclf(ii)=1.e3*sclf(ii) ! default spec in file: 'mGal'
                      else
                         sclf(ii)=1.e3*sclf(ii) ! default spec in file: 'urad'
                      endif
                   endif
                endif
                if (ii.eq.2) sclf(3)=sclf(2)
                read (iun,2103) comps
                call ucase (comps,comps)
             enddo
             comps='GRAV'
             if (ncomp.gt.1) comps='GRAV TILT-E TILT-N'
             if (Qloc_in_File(IUN,'$$ AMPLITUDES','U')) then
                backspace (iun)
                read (iun,'(a)') buff
                if (index(buff,'UM/S**2' ).gt.0.or.
     &              index(buff,'UM/S^2'  ).gt.0.or.
     &              index(buff,'UM*S^-2' ).gt.0.or.
     &              index(buff,'UM*S**-2').gt.0) then
                   sclf(1)=1.e2
                   qsclfnotset=.false.
                elseif (index(buff,'UGAL').gt.0) then
                   sclf(1)=1.0
                   qsclfnotset=.false.
                elseif (index(buff,'MGAL').gt.0) then
                   sclf(1)=1.e3    ! default spec in file: 'mGal'
                   qsclfnotset=.false.
                endif
                if (index(buff,'0.1 urad').gt.0 ) then
                   sclf(2)=1.e2
                   sclf(3)=1.e2
                   qsclfnotset=.false.
                endif
             endif
             sclp=1.0
      else
             print '(2a)',' <OLTIDE>T> ? ',comps
             print '(2a)'
     &,        ' <OLTIDE>I> First header record does not contain '
     &,        '"Strain" or "Displace"'
             stop
      endif
      print '(a,1p,4e12.4,a,e12.4)'
     &,' <OLTIDE>>> scale factors: Input * Conversion ',sclf
     &,' Phase: ',sclp
      
      ncompc=ncomp
      lu=index_l_nonblank(units)
      call set_tide_type (ncomp,comps,units)

      if (qloc_in_file(IUN,'$$ NUMFMT','U')) THEN
         backspace (iun)
         read (iun,'(9x,a)') f2104
      endif
      rewind (iun)

      IF (.NOT.QLOC_IN_FILE(IUN,'$$ COLUMN ORDER','U')) THEN
         PRINT '(A)',' <OLTIDE>T> "$$ COLUMN ORDER" record missing.'
         STOP
      ENDIF
      BACKSPACE (IUN)
c      READ (IUN,2101) MTIDE
      read (iun,'(a)') buff
      j=index(buff,'ORDER:')+6
      do i=1,11
         j1=index_1_nonblank(buff(j:))+j-1
         j2=index(buff(j1:),' ')+j1-1
         mtide(i)=buff(j1:j2)
         j=j2+1
      enddo
ccc      print *,'MTIDE: ',mtide
      DO 105  I=1,11
      call delete_in_string (mtide(i),' ',1,0)
      call delete_in_string (tides(i),' ',1,0)
      ISP(I)=0
      ITPI(I)=0
105   ITP(I)=0
      DO 101  I=1,11
      DO 1011 J=1,11
      lmt=index_l_nonblank(mtide(j))
      
      IF (index(TIDES(I),MTIDE(J)(:lmt)).eq.1) THEN
         DO 102 K=1,NV
         IF (IAA(0,K).NE.2) GOTO 102
         DO 103 L=5,8
         IF (IAA(L,K).NE.0) GOTO 102
103      CONTINUE
         DO 104 L=1,4
         IF (IAA(L,K).NE.IAAMT(L,I)) GOTO 102
104      CONTINUE
C
C Assign pointer arrays:
         ISP(I)=K
         ITP(J)=I
         ITPI(I)=J
         GOTO 101
102      CONTINUE
C
C The requested astron.argument has not been found in the spectrum:
C! User: Check the Harmonic Tide Development data file.
         PRINT '(A,4I2,3A)',' <OLTIDE>T> Astron.arg. (2'
     &,       (IAAMT(L,I),L=1,4),' 0 0 0 0) of tide ',MTIDE(J)
     &,       ' not found in spectrum.'
         STOP
C
      ENDIF
1011  CONTINUE
C
C The requested tide has not been found in array TIDES:
C! User: Check the header information in the OC_LOAD file.
      PRINT '(3A)',' <OLTIDE>T> Tide symbol ',TIDES(I),' not found.'
      print '(a,12(a4,1x))',' <OLTIDE>I> among:',MTIDE
      STOP
C
101   CONTINUE
C
      L1CALL=.TRUE.
      IF (QP) THEN
         PRINT '(A)'
     &,'0<OLTIDE>>> Ocean loading table contains the following tides:'

         print '(a,11i5)'     ,' <OLTIDE>>> Band pointer:',ITP
         print '(a,11(1x,a4))',' <OLTIDE>>>         Tide:',MTIDE
         print '(a,11i5)',' <OLTIDE>>> Tide devlp.#:'
     &,                     (ISP(ITP(J)),J=1,11)
         print '(a,3(i1,1h ,a10,:,3h | ))'
     &,        ' <OLTIDE>>> Components:',(J,COMP(J),J=1,NCOMP)
         print '(a,1p,3d11.3)'
     &,        ' <OLTIDE>>> Scale factor for file amplitudes:',sclf
      ENDIF
      print *,'<oltide>>> Find by site: >',opt,'< ',qbs,' ',ssite,ls
C______________________________________________________________________
C                                              /_____I. End prepare___/
C
C______________________________________________________________________
C II. Site info retrieval for the requested coordinates XS,YS_________/
C
201   REWIND IUN
C
      IF (CRTDIS.LT.0.D0) CRTDIS=180.D0
      CCD=DCOS(RAD*CRTDIS)
C
C II.1 Skip header; search the record containing END HEADER
      IF (.NOT.QLOC_IN_FILE(IUN,'$$ END HEADER','U')) GOTO 298
      DISMIN=-2.D0
C
C II.2 Find the ocean loading site information:
C
C      Each site record has to consist of:
C $$
C   Site name
C $$
C $$
C $$ ... lon/lat: value value     (in degrees)
C   radial amplitudes
C   west       "
C   south      "
C   radial phases
C   west     "
C   south    "
C
      IF (QP) PRINT '(A,16x,a,2F12.6)'
     &,             '0<OLTIDE>>> Requested site','lon/lat: ', XS,YS
c      print *,'oltide: ncomp=',ncomp
 203  RCAL=1.0
      RCALU=1.0
      READ (IUN,2102,END=299)
      READ (IUN,2105,END=299) FSITE
      READ (IUN,2106,END=299) INSTR
      print *,'INSTR 1 ->',instr
      I=INDEX(INSTR,'SCF=')+4
      IF (I.GT.6) READ (INSTR(I:),*) RCALU
cc      print *,' debug 1: rcal ',rcalu
      READ (IUN,2106,END=299) INSTR
      print *,'INSTR 2 ->',instr
      I=INDEX(INSTR,'SCF=')+4
      IF (I.GT.6) READ (INSTR(I:),*) RCALU
cc      print *,' debug 2: rcal ',rcalu
      READ (IUN,2106,END=299) INSTR
      print *,'INSTR 3 ->',instr
      I=INDEX(INSTR,'SCF=')+4
      IF (I.GT.6) READ (INSTR(I:),*) RCALU
cc      print *,' debug 3: rcal ',rcalu
      
      if (qsclfnotset) then
         rcal=rcalu 
         print '(a,1p,e12.4)',' <OLTIDE>>> Scale from site record: 1/'
     &,  rcal
      else
         print '(a)',' <OLTIDE>>> Scale from header'
      endif

      I=INDEX(INSTR,'lon/lat:')+8
      WRITE (BUFF,2106) INSTR(I:)
ccc      print '(2a)',' <OLTIDE>>> BUFF ->',BUFF
      READ (BUFF,*) XF,YF
      IF (QBS.AND.FSITE(:LS).EQ.SSITE(:LS)) THEN
         COSDIS=2.D0
C
C Compute cosine of distance for the site encountered:
      ELSE
         COSDIS=DSIN(YS*RAD)*DSIN(YF*RAD)+DCOS(YS*RAD)*DCOS(YF*RAD)
     &          *DCOS((XF-XS)*RAD)
      ENDIF
C
C---> Find the closest site in OC_LOAD file
C---> and store ocean loading coefficients,
C---> renormalised as response parameters ...
      IF (COSDIS.GT.DISMIN) THEN
         do 204 i=1,ncomp
         read (iun,'(a)') buff
ccc         print *,buff
         read (buff,f2104) (OTAMP(J,I),J=1,11)
c         READ (IUN,F2104) ((OTAMP(J,I),J=1,11),I=1,NCOMP)
c         do 204 i=1,NCOMP
         do 204 j=1,11
cc            print *,'debug: ',otamp(j,1),sclf,rcal
204      otamp(j,i)=otamp(j,i)*sclf(i)/rcal
         
         do 2041 i=1,ncomp
         read (iun,'(a)') buff
ccc         print *,buff
         read (buff,f2104) (OTPHA(J,I),J=1,11)
 2041    continue
c         READ (IUN,F2104) ((OTPHA(J,I),J=1,11),I=1,NCOMP)
         DISMIN=COSDIS
         XOLS=XF
         YOLS=YF
         SITE=FSITE
      ELSE
         DO 216 I=1,NCOMP*2
216      READ (IUN,2102)
      ENDIF
      GOTO 203
C
C Error branch:
298   PRINT '(2A)'
     &,' <OLTIDE>T> "END HEADER"-record in OC_LOAD table not found.'
      STOP
C
C Reset: Counters and pointers NA INBAND BANDIN & IIP, and arrays:
299      DO 214 I=1,3
         NA(I)=0
         DO 213 J=0,2
213      INBAND(J,I)=0
         DO 214 J=1,11
         BANDIN(J,I)=-1
         IIP(J,I)=0
         FMT(J,I)=0.0D0
         ZMT(J,I)=(0.0D0,0.0D0)
214      CONTINUE
C
C Do for the 11 possible tide frequencies:
         DO 211 I=1,11
         II=ITPI(I)
         IF (II.LE.0) GOTO 211
         K=ISP(I)
         N=IAA(0,K)
         M=IAA(1,K)
         GU=GAMMA(FRE(K),N,M)
         AU=AMP(K)
         PHU=PHA(K)/RAD
C
C Do for each component (radial, east, north):
         DO 212 JCOMP=1,ncomp
         S=SINITIAL
C
C Flip signs for the tangential components to get east and north displcm.
         IF (icomp.eq.1.and.JCOMP.GT.1.and.qfs) S=-1.D0
C
C---> Store if a value is avaiable:
         BTAMP(II)=AU*GU
         BTPHA(II)=PHU
         IF (OTPHA(II,JCOMP).LT.700.D0) THEN
C
C---> Compute admittances, ...
C! N.B.: EXP = CDEXP 
            Z=S*OTAMP(II,JCOMP)*EXP(DCMPLX(0.0D0,-OTPHA(II,JCOMP)*RAD))
C---> ... i.e. normalize them w.r.t. the amp. of the tide generating potential:
     &        /AU/GU
C
C Assign:
C Number of table values in each fundamental tide band:
            INBAND(M,JCOMP)=INBAND(M,JCOMP)+1
C Total number for each component:
            NA(JCOMP)=NA(JCOMP)+1
C Index in response arrays
            L=NA(JCOMP)
C Pointer from tide number I to response array:
            IIP(I,JCOMP)=L
C The response array itself:
            ZMT(L,JCOMP)=Z
C The associated tide frequency:
            FMT(L,JCOMP)=FRE(K)
C In which fundamental band is the response coefficient number L ?
            BANDIN(L,JCOMP)=M
         ENDIF
212      CONTINUE
211      CONTINUE
C ______________________________________________________________________
C                              \                                       /
C                               \________II. End site info retrieval__/
C
C Print site info
      IF (DISMIN.LT.CCD) THEN
         PRINT '(A,F8.2)',' <OLTIDE>E> All sites at distance > ',CRTDIS
         RETURN
      ENDIF
      IF (QP) THEN
         PRINT '(3A,2F12.6)'
     &,        '0<OLTIDE>>> Nearest site found: '
     &,         SITE,', lon/lat: ',XOLS,YOLS
         IF (.NOT.QBS) PRINT '(A,F10.2,2A)'
     &,        ' <OLTIDE>>> Distance to requ.point: '
     &,         DACOS(DISMIN)/RAD,' (deg). Option: ',OPT
         IF (ABS(1.-RCAL).gt.1.e-4) PRINT '(A,1P,E12.4)'
     &,        ' <OLTIDE>>> Amplitudes assumed scaled by ',RCAL
         PRINT '(A)'
     &,        ' <OLTIDE>>> Amplitudes taken at face value '
         IF (INDEX(OPT(:LO),'P').GT.0) THEN
            PRINT 1004, UNITS(:LU)
     &,       ('Amp'//COMP(I)//'Pha    ',I=1,NCOMP),'  Amp BT [m]'
            DO 223 I=1,11
223         PRINT 1003, I
     &,       OTAMP(I,1)*sclp,OTPHA(I,1)
     &,      (OTAMP(I,J)*sclp,AMOD(OTPHA(I,J)+phap,360.),J=2,NCOMP)
     &,       BTAMP(I),BTPHA(I)
         ENDIF
         IF (INDEX(OPT(:LO),'R').GT.0) THEN
            PRINT 1002, UNITS(:LU),(COMP(I),NA(I),I=1,3)
            DO 221 L=1,11
            DO 222 J=1,NCOMP
            ZCZ=dpolar(ZMT(L,J),0.0d0)
            SA(J)=dreal(zcz)
222         SP(J)=dimag(zcz)
ccc         write (7,'(1p,3(2d12.4,2x))') (zmt(l,j),j=1,NCOMP)
            PRINT 1001
     &,           (FMT(L,J)/RAD,SA(J)*sclp,SP(J),BANDIN(L,J),J=1,3)
            J=BANDIN(L,1)
            PRINT 1008
     &,           FMT(L,1)/RAD,GAMMA(FMT(L,1),2,J)
221         CONTINUE
            PRINT '(A)',' <OLTIDE>>>'
         ENDIF
      ENDIF
1001  FORMAT (' <OLTIDE>>>',3(0P,F10.6,1P,D10.2,0P,F10.1,2X,I1,2X))
1002  FORMAT (' <OLTIDE>>>'/' <OLTIDE>>> Response in '
     &,A,'/[m**2/s**2]:'/' <OLTIDE>>>'
     &/' <OLTIDE>>> ........',A10,':',I3,' ........../'
     &,         2('/........',A10,':',I3,' ........../')
     &/' <OLTIDE>>>',3(35H    Freq.   Amp..OLT-resp..Pha  M  )
     &/' <OLTIDE>>>')
1003  FORMAT (' <OLTIDE>>>',I3,1X,3(F9.5,F9.1,2X),F12.6,F8.1)
1004  FORMAT (' <OLTIDE>>>'/' <OLTIDE>>> Loading coefficients ',a
     &/       ' <OLTIDE>>> Seq# ',4a
     &/       ' <OLTIDE>>>')
1008  FORMAT (' <OLTIDE>>>',2F10.6,' = gamma')
C
C_______________________________________________________________________
C                                                                      /
C III. Produce the loading tide spectrum______________________________/
C
301   CONTINUE
C
C III.1 Principal waves: Adopt.
      NCOMPU=MIN0(NCOMP,INQ_ACTIVE_COMP())
      DO 302 JCOMP=1,NCOMPU
      NA(JCOMP)=0
      DO 303 K=1,NV
303   ZOL(K,JCOMP)=(0.0D0,0.0D0)
      DO 302 I=1,11
      K=ISP(I)
      N=IAA(0,K)
      M=IAA(1,K)
      L=IIP(I,JCOMP)
      IF (L.GT.0) THEN
C! N.B.: EXP = CDEXP 
         ZOL(K,JCOMP)=ZMT(L,JCOMP)*EXP(DCMPLX(0.0D0,PHA(K)))*AMP(K)
     &    *GAMMA(FRE(K),N,M)
         NA(JCOMP)=NA(JCOMP)+1
      ENDIF
302   CONTINUE
C
C III.2 Unmodelled waves: Interpolate.
      DO 411 I=1,NCOMPU
      NBELOW(I)=0
      NABOVE(I)=0
      NBETWN(I)=0
411   NI(I)=0
C-----------------------------------------------------------------------I
      IF (INDEX(OPT(:LO),'I').GT.0) THEN                                I
C-----------------------------------------------------------------------I
      DO 401 JCOMP=1,NCOMPU                                             I
      NCAA=-1                                                           I
      IT2=0                                                             I
      DO 401 K=1,NV                                                     I
      IF (CDABS(ZOL(K,JCOMP)).GT.1.D-20) GOTO 401                       I
      N=IAA(0,K)                                                        I
      M=IAA(1,K)                                                        I
      IF (N.NE.2) GOTO 401                                              I
      IF (M.NE.NCAA) THEN                                               I
C                                                                       I
C Skip into the next fundamental band, which has IL ocean models.       I
C                                                                       I
         NCAA=NCAA+1                                                    I
         IL=INBAND(NCAA,JCOMP)                                          I
         IT1=IT2+1                                                      I
         IT2=IT1+IL-1                                                   I
         ITU=IT1                                                        I
         IF (IL.EQ.0) GOTO 401                                          I
      ENDIF                                                             I
C                                                                       I
      IF (FRE(K).LT.FMT(IT1,JCOMP)) THEN                                I
C                                                                       I
C Take the coefficient of the lowest / highest modelled frequency in    I
C that band:                                                            I
C                                                                       I
C! N.B.: EXP = CDEXP 
         ZOL(K,JCOMP)=ZMT(IT1,JCOMP)*EXP(DCMPLX(0.0D0,PHA(K)))*AMP(K)   I
     &                *GAMMA(FRE(K),N,M)                                I
         NBELOW(JCOMP)=NBELOW(JCOMP)+1                                  I
      ELSE IF (FRE(K).GT.FMT(IT2,JCOMP)) THEN                           I
C! N.B.: EXP = CDEXP 
         ZOL(K,JCOMP)=ZMT(IT2,JCOMP)*EXP(DCMPLX(0.0D0,PHA(K)))*AMP(K)   I
     &                *GAMMA(FRE(K),N,M)                                I
         NABOVE(JCOMP)=NABOVE(JCOMP)+1                                  I
      ELSE                                                              I
C                                                                       I
C Interpolate between the nearest model freq. linearly                  I
C                                                                       I
402      IF (FRE(K).GT.FMT(ITU+1,JCOMP)) ITU=ITU+1                      I
         IF (FRE(K).GT.FMT(ITU+1,JCOMP)) GOTO 402                       I
         IF (ITU.GT.IT2-1) PRINT '(A)',' <OLTIDE>E> Band Problem ITU'   I
         IF (FMT(ITU,JCOMP).LE.1.D-20)                                  I
     &                     PRINT '(A)',' <OLTIDE>E> Band Problem FMT1'  I
         IF (FMT(ITU+1,JCOMP).LE.1.D-20)                                I
     &                     PRINT '(A)',' <OLTIDE>E> Band Problem FMT2'  I
         IF (abs(FMT(ITU+1,JCOMP)-FMT(ITU,JCOMP)).LE.1.D-20)            I
     &                     PRINT '(A)',' <OLTIDE>E> Band Problem FMT3'  I
         if (ITU.GT.IT2-1.or.
     &       FMT(ITU,JCOMP).LE.1.D-20.or.
     &       FMT(ITU+1,JCOMP).LE.1.D-20.or.
     &       abs(FMT(ITU+1,JCOMP)-FMT(ITU,JCOMP)).LE.1.D-20) then
            ZOL(K,JCOMP)=(0.d0,0.d0)
            goto 401
         else
            ZOL(K,JCOMP)=(ZMT(ITU,JCOMP)                                I
     &               +(ZMT(ITU+1,JCOMP)-ZMT(ITU,JCOMP))                 I
     &               /(FMT(ITU+1,JCOMP)-FMT(ITU,JCOMP))                 I
     &               *(FRE(K)          -FMT(ITU,JCOMP)))                I
C! N.B.: EXP = CDEXP                                                    I
     &                *EXP(DCMPLX(0.0D0,PHA(K)))                        I
     &                 *AMP(K)*GAMMA(FRE(K),N,M)                        I
            NBETWN(JCOMP)=NBETWN(JCOMP)+1                               I
         endif
      ENDIF                                                             I
      NI(JCOMP)=NI(JCOMP)+1                                             I
401   CONTINUE                                                          I
      IF (QP) THEN                                                      I
         PRINT '(2A)',' <OLTIDE>>> Interpolated ocean loading spectrum.'I
     &,               ' Partial''s location:'                           I
         PRINT '(A,3I4)'                                                I
     &,        ' <OLTIDE>>> Lower band corners.....:',NBELOW            I
     &,        ' <OLTIDE>>> Inside bands...........:',NBETWN            I
     &,        ' <OLTIDE>>> Upper band corners.....:',NABOVE            I
      ENDIF                                                             I
C-----------------------------------------------------------------------I
      ELSE
C-----------------------------------------------------------------------S
      IF (QP) PRINT '(A)'                                               S
     &,     ' <OLTIDE>>> Sparse ocean loading spectrum returned.'       S
C-----------------------------------------------------------------------S
      ENDIF
C
      IF (QP.OR.INDEX(OPT(:LO),'q').LE.0) PRINT '(3A,3I5,A,3I5)'
     &,     ' <OLTIDE>>> Site ',SITE,'. Nonzero Ocean Loading coeff''s:'
     &,       NA,'. Interpolated:',NI
      RETURN
2101  FORMAT (T18,11(A3,1X))
2102  FORMAT (1HX)
2103  FORMAT (2X,A64)
c2104  FORMAT (2X,11F7.0)
2105  FORMAT (2X,A8)
2106  FORMAT (A)
      END

      SUBROUTINE Comb_Earth_Ocean (ZSE,ZOL)
C Loading tides * Combine body and loading tide spectra
C
C Combines the solid earth (ZSE) and ocean loading (ZOL) spectra:
C
C     BL(k) * ZSE(,k) + ZOL(,k) -> ZSE(,k)
C
C where BL is the Basic Love number for component k.
c
c BL(k) is applied if component is displacement (parameter set by
c       Ocean_Load_Tide)
c
c If BL(k) is to be ignored (already applied before call), use
c Call BLMULT ('NO') before calling this procedure.
C
C The BLMULT-option of Tide_S_T is turned off automatically.
C
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
C      INTEGER*1 IAA
      INTEGER*2  IAA
      COMPLEX*16 ZSE(1200,*),ZOL(1200,*)
      logical qblm,qdispl
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW
      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IAA(0:9,1200)
      common /cttsts/ qblm,qdispl,ncompc
      SAVE
      blv=1.d0
      ncompu=min0(ncompc,inq_active_comp())
      DO 101 J=1,ncompu
      K=MIN0(J,2)
      DO 101 I=1,NV
      if (qdispl) blv=bl(k)
101   ZSE(I,J)=ZSE(I,J)*BLV+ZOL(I,J)
      PRINT '(A)'
     &,'0<CEO--->>> Solid earth and ocean tides have been combined.'
      if (qdispl) then
         print '(a,2f10.4,a)'
     &,        ' <CEO--->>> Basic Love numbers '
     &,          BL(1),BL(2),' have been applied on SE-tide.'
      else
         print '(a)'
     &,        ' <CEO--->>> Basic Love numbers have been ignored'
      endif
      CALL BLMULT ('NO')
      RETURN
      END


C:Provides  subroutine ocean_load_site F=oltide.f
C:Provides  subroutine ocean_load_tide F=oltide.f
C:Provides  subroutine comb_earth_ocean F=oltide.f
