      subroutine open_ins (iun,default_name) 
C File utility * Open *.ins file 
C>
C OBS! OBS! OBS!
C since this routine calls GETARG, it must be compiled with g77
C =============================================================
C
C iun - integer - file unit number (in, out).  
C Might be changed to 5 on return.  
C default_name - char** - file name 
C 
C Some programs read parameters from INStruction files.  At the same time, 
C output may be redirected to a file or device as the screen needs to be 
C reserved for graphics etc.  
C In order to keep user prompt for instruction file specification on 
C the console ...  (see the html manual: 
C ftp://gere.oso.chalmers.se/pub/hgs/hgs.man/openins.html )
C_ 
C ...this routine opens an instruction file while keeping the prompt on the 
C screen.  The screen device name is read from the environment variable 
C $tty. The tcsh or ksh may be instructed to set these variables:  
C 
C term=$TERM 
C tty=pty/`ps | fgrep " ps" | cut -c 8-16` 
C export term tty 
C
C Recognized terminals (3 significant characters):
C vt1** vt2** vt3** xte** hpt**
C 
C PROMPT:  
C -------
C The prompt asks for a file name: 
C
C   > *.ins file (default_name.ins) >
C
C The user enters a file name. Other responses: $ and ?
C
C Rules for filenames:
C The program appends ".ins" at the name entered 
C  * unless the last four characters of the name contain "." or 
C  * unless the whole name is given enclosed within quotes ('). 
C 
C The user may enter the name of a directory, in which case default_name 
C is appended to the user entry.  
C Example:  default_name is "INSTRF"
C User enters ABCD/ 
C File name will be "ABCD/INSTRF.ins"
C 
C The file is opened with STATUS='OLD'; in the case of ERRor, the program 
C STOPs.  
C
C If the user enters ':'  or 'TER:'  'TRM:'  or 'CON:', OPEN is skipped 
C and IUN=5 will be used.
C
C Any of this can be given on the program launch line; c.f. below, "$".
C
C
C BEGIN READING *.ins-FILE AFTER xyz
C ----------------------------------
C The point from which file reading commences can be shifted. At the
C file name prompt, add a blank and >SEARCH_WORD, like
C
C        grav/olfg.ins >LEPR>
C
C In this example, the *.ins-file will be read after a record which starts 
C with LEPR> (reckoned from column 1)
C 
c
c The dialogue is exchanged via default logical unit 10.  That allocation 
c can be redefined: 
C 
c        Call Open_Term (iun) 
c
c where  iun > 0 . 
c This call will open the terminal for dialogue. The terminal unit can be
c used for subsequent console input and output while the standard output
c might have been redirected to a file.  
c 
c To inquire the actual file unit, 
c       iun=-1                          or        
c       Call Open_Term (iun)                           iun=Terminal_Out(i)
c This call will do nothing else                 The value iun returned is a
c but return the actual iun.                     recommended output unit.
c If iun=-1 on return, no special terminal       i=0: "sample once and hold"
c unit is open. Subsequent instruction           i=1: "fresh measurement"
c file prompt under                              Use i=1 if you know/suspect
c       Call Open_ins (...)                      intervening Call Close_Term()
c will be processed on units 
c 5 (read) and 6 (write).
c 
c To close the terminal, 
c       Call Close_Term () 
c
c The default terminal unit is 2. If environment parameters $tty and $term
c are not set, terminal unit will be set to -1. 
c
C USER RESPONSE IS  $
C $ - Taking the launch line arguments using getarg(.,.).
C The prompt response can be given already on the program launch line, in 
C which case a Dollar sign ($) is to be entered at the prompt. 
C However, observe the shell's effect on string quotes and redirection 
C symbols (>); quotes and escaping backslashes will be required.
C
C Example: 
C Running the ocean loading program. User's response is underlined.
C
C   G[hgs]~/Oload> olfg \'/geo/hgs/Oload/grav/olfg-instructions\' '>HERE>'
C                  ------------------------------------------------------- 
C   0<StTiMe>>> olfg.f at 1997-Jan-15 16:06:55
C    <OpenIn>>> Open Terminal = /dev/ttyq1, unit was   2
C    > *.ins file (olfg.ins)> $
C                             -
C 
C USER RESPONSE IS  ?
C This enters a help utility provided by the main program source code. 
C The main program must contain a statement
C     CALL START_TIME_MESSAGE ('/path/with/main.f$')
C All comments in the main program will be listed on the terminal
C
      character ans*64, default_name*(*), ts*32, tty*32, term*32, a4*4
     &,         buff*80, ans1*1, ds*10, cs*8, loctxt*16, temp*24
     &,         fn*128
      logical qe,qo,qt,qhelp,qer,qloc,q_substring,qloc_in_file,qtalk
      common /cncoi/ ncoi,llt,loctxt
      save
      data iuterm/2/, qhelp/.true./, fn/' '/, iuns/-1/

      qe=.false.  
      ts=default_name 
      ldn=index_l_nonblank (ts) 
      if (ts(ldn:ldn).eq.'/') then 
         stop '<OpenIn>T> Illegal default name' 
      endif 
ccc      print *,'Default name ts(:ldn): ',ts(:ldn)
ccc      print *,'Default name ts      : ',ts
ccc      print *,'Default name         : ',default_name
ccc      print *,'Default name: ',ts(:ldn)
      qtalk=.true.
      goto 1

      entry open_term (iut) 
      qtalk=.true.
      goto 111
      entry open_term_q (iut) 
      qtalk=.false.

 111  if (iut.gt.0) iuterm=iut 
      if (iut.eq.-1) then 
         qo=.false.
         if (iuterm.gt.0) inquire (unit=iuterm,opened=qo) 
         if (qo) then
            iut=iuterm 
         elseif (qtalk) then
            print '(a,i3)',' <OpenTe>>> Not open, unit was ',iuterm
         endif
         return 
      endif
      qe=.true.

1     qo=.false.
      iuu=iuterm
      if (iuterm.le.0) iuu=5
      inquire (unit=iuu,opened=qo)
      qt=qo.and.iuu.ne.5 
      if (qe.and.qo) then
         return 
      endif
      iarg=1
      call getarg (iarg,ans)
      la=index_l_nonblank(ans)
cc      print *, 'openins: ',iarg,' ->',ans(:la),'<-'
      if (qe.or..not.(ans(1:1).eq.'!'.or.ans(1:1).eq.'@')) then
         if (.not.qo) then
            call getenv ('tty',tty) 
            qt=index_l_nonblank(tty).gt.0.and.iuu.ne.5
            if (tty(1:5).ne.'/dev/') then
               tty='/dev/'//tty
               lt=index_l_nonblank(tty)
               print '(3a)',' <OpenIn>>> Your terminal: ',tty(:lt)
     &,                     ' (I suppose)'
            endif
            call replace_in_string (tty,'pty/pty','pty',1,0)
            call replace_in_string (tty,'pts/pty','pts',1,0)
            call replace_in_string (tty,'pts/pts','pts',1,0)
            if (qt) then
               call getenv ('term',term) 
               if (index('vt1_vt2_vt3_xte_hpt_hp _',term(1:3)).lt.1
     &              .and.term(1:2).ne.'hp') then 
                  print '(3a)'
     &,                 ' <OpenIn>E> Unknown term type: "',term,'"' 
                  print '(a)'
     &,' <OpenIn>I> Issue the following unix command in this terminal wi
     &ndow:'
     &,'            setenv term vt100'
     &,' <OpenIn>I> or anyway set term in environment.'
c     stop 
c2345678901234567890123456789012345678901234567890123456789012345678901234567890
c
               endif 
               lt=index_l_nonblank(tty)
               open (unit=iuterm,file=tty(:lt),err=999,iostat=ioerr)
               if (qtalk) print '(2a,i3)'
     &,                   ' <OpenIn>>> Open Terminal: '//tty(:lt)
     &,                   ', unit was ',iuterm
               goto 989

 999           lt=index_l_nonblank(tty)
               print '(a,i5,a,i3,2a)',' <OpenIn>E> ERR ',ioerr
     &,              ' Terminal unit:',iuterm
     &,              ' reset -> -1.  Name ->'
     &,              tty(:lt)//' <&> '//term//'<-' 
               iuterm=-1

 989           continue
            else
               if (qtalk) print '(2a,i3)'
     &,     ' <OpenIn>>> No tty declared. We will use units 5 and 6 '
     &,     'instead of ',iuterm
               iuterm=-1
            endif 
         endif
         iutw=iuterm
         iuu=iuterm
         if (iuterm.eq.-1) iutw=6
         if (iuterm.eq.-1) iuu=5
      endif 
      if (qe) then
         return 
      endif

10    call getarg (iarg,ans)
      if (ans(1:1).eq.'!'.or.ans(1:1).eq.'@') then
         ipa=2
         if (ans(2:2).eq.' ') then
            iarg=iarg+1
            call getarg (iarg,ans(2:))
         endif
         if (ans(ipa:ipa).eq.'?') then
            ans='? '
         else
            ans='$ '
         endif
         iutw=6
      else
         write (iutw,'(1h ,a,3h)> ,$)') '> *.ins file ('//ts(:ldn) 
         read (iuu,'(a)') ans 
      endif
      if (ans(1:2).eq.'? ') then
         call get_file_name (0,ans)
         call openfs ('21 R '//ans)
 171     read (21,'(a)',end=997) buff
         lb=index_l_nonblank(buff)
         if (index('cC',buff(1:1)).gt.0) write (iutw,'(a)') buff(2:lb)
         if (index('c_C_',buff(1:2)).gt.0) goto 997
         goto 171
cc 998     la=index_l_nonblank(ans)
cc         write (iutw,'(2a)') ' <OpenIn>T> No help for ',ans(:la)
 997     stop
      endif
      if (ans(1:2).eq.'$ ') then
         iarg=1
         call getarg (iarg,ans)
         if (ans(1:1).eq.'!'.or.ans(1:1).eq.'@') then
            if (ans(2:2).ne.' ') then
               ans=ans(2:)
            else
               iarg=iarg+1
               call getarg (iarg,ans)
            endif
         endif
         i=index_l_nonblank(ans)+2
         iarg=iarg+1
         call getarg (iarg,ans(i:))
         if (ans(i:i).ne.'>'.and.ans(i:i).ne.':') then
            ans(i:)=' '
         else
            ans(i:i)='>'
            la=index_l_nonblank(ans)
            if (ans(la:la).ne.'>') then
               la=la+1
               ans(la:)='>'
            endif
         endif
         print '(3a)', 
     &         ' <OpenIn>>> From command line ->',ans(:la),'<-'
      endif
         
      qloc=q_substring(ans,'>','$] ',loctxt,ilt,llt)
      if (qloc) then
         ans=ans(:ilt-2)
         llt=min0(16,llt)
      endif
      
      i=index_l_nonblank (ans)
      j=index(ans//' ',' ')-1
      ij=1
      if (ans(1:1).eq.' ') then
         ans=ts(:ldn) 
      elseif (ans(j:j).eq.'/') then 
         ans=ans(:j)//ts(:ldn) 
         ij=j                         ! we'll ignore any "." in path head
      endif 
ccc      print *,'1 ans=',ans
      call ucase(ans(:4),a4) 
      if (index('CON:TRM:TER:TTY:',a4).gt.0.or.ans(1:1).eq.':')  
     & then
         iun=5
         write (iutw, '(a)')
     & ' <OpenIn>>> Expecting instruction input from the terminal !'
         return
      endif 
c      print '(1h )' 
      j=index(ans//' ',' ')-1
      ans1=ans(j:j)
      if (index('"''',ans1).gt.0) then
         call delete_in_string (ans,ans1,1,j)
         j=index(ans//' ',' ')-1
      else
         i=index(ans(ij:),'.')  
         if (i.le.0) ans=ans(:j)//'.ins' 
      endif
ccc      print *,'2 ans=',ans
      open (unit=iun,file=ans,status='OLD',err=99) 
      ncoi=0
      if (qloc) then
343      qer=qloc_in_file (iun,loctxt(:llt),' ')
         if (.not.qer) then
            WRITE (iutw,'(2a)') ' <OpenIn>E> Not found: ',loctxt(:llt)
            write (iutw,'(a)') 
     &            ' <OpenIn>?> Stop | Rewind | Cont | >text'
            read  (iuu,'(a)') temp
            if (index('cCrR>',temp(1:1)).le.0) then
               stop '<OpenIn>T> USER'
            elseif (temp(1:1).eq.'>') then
               qer=q_substring(temp,'>',']$ ',loctxt,ilt,llt)
               rewind (iun)
               goto 343
            endif  
            rewind (iun)
         else
            if (iutw.ne.6) 
     .      WRITE (iutw,'(2a)') ' <OpenIn>>> Located at ',loctxt(:llt)
            PRINT '(2a)',       ' <OpenIn>>> Located at ',loctxt(:llt)
            ncoi=located_in_file ()
         endif
      endif
      i=index_l_nonblank (ans) 
      call my_date(ds)
      call my_time(cs)
ccccc      print *,'openins=',time()
      PRINT '(A,i2,3h:  ,A,t54,6A)' 
     &,     ' <OpenIn>>> ',IUN,ans(:i),' - on ',ds,' ',cs
      WRITE (iutw,'(A,i2,3h:  ,A,t54,3h - ,A)') 
     &      ' <OpenIn>>> ',IUN,ans(:i),'INSTRUCTION FILE' 
      call save_file_name (IUN,ans(:i))
      iuns=iun
      fn=ans(:i)
      return

      entry get_insfile_name (default_name)
      i=min0(index_l_nonblank(fn),len(default_name))
      if (i.le.0.or.iuns.le.0) then
         default_name=' '
      else
         default_name=fn(:i)
      endif
      return

      entry close_term () 
      if (iuterm.gt.0.and.iuterm.ne.5) close (iuterm) 
      iuterm=-9
      return 

99    if (.not.qhelp) then 
         print '(3a,3i4)',' <OpenIn>T> ERR: >',ans
     &,        '<  Dot, length, tail=', j,i,ij 
         stop '<OpenIn>T> ERR for the second time.'
      endif
      qhelp=.false.
c23456789012345678901234567890123456789012345678901234567890123456789012 
      if (iutw.gt.0.and.iuterm.gt.0) then
         i=index_l_nonblank(ans)
         write (iutw,1001) ans(:i)
1001  format (' <OpenIn>W> ERR:  ',a 
     &/' <OpenIn>I> At the prompt, you have the following alternatives:'  
     &/' <OpenIn>I> (1) Specify entire name of the instruction file '
     &/' <OpenIn>I>     (criterion: name contains "." or use '' '' as de
     .limiters).'  
     &/' <OpenIn>I> (2) Specify first part (no dot) - ".ins" will be app
     .ended.'
     &/' <OpenIn>I> (3) Specify a path (must be right-delimited by "/")' 
     &,' -' 
     &/' <OpenIn>I>     the default name shown will be appended.'
     &/' <OpenIn>I> (4) Specify TER: or TRM: for terminal input;'
     &/' <OpenIn>W>     Not all programs allow that though.'
     &/' <OpenIn>I> (5) Specify "$"; the string alt.(1)...(4) will be'
     &/' <OpenIn>I>     from the argument list of the program launch.')       
         goto 10 
      endif
      print *,'<OpenIn>T> Trouble ',ans(:i),' ',tty,' ',term
     &,       ' ',iuu,iutw,iuterm
      stop '<OpenIn>T> Trouble'

      end

      subroutine rewind_ins (iun,*)
c
c Rewinds the instruction file and positions it immediately after the
c section label if such has been declared at the prompter / command line
c
      character loctxt*16
      logical qloc_in_file
      common /cncoi/ ncoi,llt,loctxt

      rewind (iun,err=901)
      if (llt.le.0) return

      if (.not.qloc_in_file(iun,' ',loctxt(:llt))) then
         print '(a,i2.2,3a)'
     &,        ' <RewIns>T> Unit #',iun
     &,        ' Cannot locate label: "',loctxt(:llt),'"'
         stop
      endif
      return

 901  print '(a,i2,a)',' <RewIns>E> Cannot rewind #',iun
      return 1
      end

      integer function terminal_out (i)
c
c iun=terminal_out(0)
c returns a unit number suitable for output to the screen.
c If iuterm is 5 the output unit will be 6, else equal to iuterm.
c
      data iuterm/-999/
      if (i.ne.0) iuterm=-999
      if (iuterm.le.-999) then
         iuterm=-1
         call open_term (iuterm)
         if (iuterm.eq.5.or.iuterm.le.0.or.iuterm.gt.99) iuterm=6
      endif
      terminal_out=iuterm
      return
      end

      block data bncoi
      character loctxt*16
      common /cncoi/ ncoi,llt,loctxt
      data ncoi/0/, llt/0/, loctxt/' '/
      end
C:Provides  subroutine open_ins F=openins.f
C:Provides  subroutine rewind_ins F=openins.f
C:Provides  entry open_term F=openins.f
C:Provides  entry open_term_q F=openins.f
C:Provides  entry close_term F=openins.f
C:Provides  integer function terminal_out F=openins.f
C:Provides  block data bncoi F=openins.f
