      LOGICAL FUNCTION QLOC_IN_FILE (IUN,STR,OPT)
C
C iun   - integer - file unit number.
C str   - char**  - text to locate
C opt   - char**  - options (blank N O A U AC P B)
C
C Locate (Move forward to) text in a file. 
C
C ( )  position an ASCII file immediately after a line that starts with
C      the indicated text STR:  OPT =/= 'O'
C
C (O)  position within an OPEN FILE block, at a record  '   L '//STR: 
C           OPT == 'O'
C 
C (E)  replace environment parameters
C
C (U)  Upper case conversion: OPT == 'U'
C
C (A)  Search anywhere in target string: OPT == 'A' 
C
C (AC) Search anywhere in target string and demand match at exact column: 
C           QER=QCOLLOC_IN_FILE(ICOL)
C           IF (QLOC_IN_FILE(...,'AC')) THEN ...
C
C (N)  Normally, file is rewound one time. To suppress this: OPT= 'N'
C (B)  Backspace to stay at the located line (default is next line)
C
C (P)  Print protocol messages: OPT = 'P'; this option is remanent at return
c (PP) more extensive print.
C (Q)  Quiet, no messages: OPT = 'Q'
C
C
C     The line read most recently can be copied to STR using
C           QER=QLOCLINE_IN_FILE (STR,ICOL)
C     where ICOL is the position of the first character in the line.
C     Returns .TRUE. if the last location operation has been successful.
C     (A negative number for ICOL will copy from the first nonblank character
C     at or to the right of |ICOL| .)
C     For instance, consider the target line 'TSF EDIT    MYLAB'
C     where LABEL is preceeded by one plus an unknown number of blanks
C     can be processed as follows
C 10     IF (QLOC_IN_FILE (22,'TSF EDIT',' ')) THEN
C           QER=QLOCLINE_IN_FILE (LABEL,-9)
C           IF (LABEL(1:6).NE.'MYLAB ') GOTO 10
C        ENDIF
C     Thus, the search will go on until the target has been found. 
C
C More entries:
C
C        QER=QEOI_IN_FILE(STR,ICOL)
C     defines a read-termination string. For reading to terminate 
C     the criterion is an exact match at column ICOL of the input line.  
C     ICOL < 1 switches the feature off.
C     ICOL = 0 leaves the internally stored value unchanged.
C     QEOI_IN_FILE returns .TRUE. if the feature is enabled.
C
C        NREADS=LOCATED_IN_FILE()
C     returns the line number that produced the match.
C
C     Print options can also be set externally:
C        QER=QLOC_IN_FILE_DBG (OPT)
C     where OPT is 'P' or 'PP' for normal or extensive diagnostic printing. 
C
C
      CHARACTER STR*(*),OPT*(*),RSTR*(*)
     &,         INSTR*256,TSTR*256,EOISTR*256,ANS*9
      LOGICAL   QUC,QOF,QRWD,QANYWH,QPR,QPRPP,QCOL,QLIF,QBCK,QUCH_IN
     &,         QCOLLOC_IN_FILE,QEOI_IN_FILE,QLOC_IN_FILE_DBG
     &,         QLOCLINE_IN_FILE, QRENV
      INTEGER   LOCATED_IN_FILE
      SAVE
      DATA JEOI/-1/, LEOI/1/, QPR,QPRPP/2*.FALSE./
      
      QRWD=.NOT.QUCH_IN('N',OPT,I)
      QBCK=QUCH_IN('B',OPT,I)
      QUC=QUCH_IN('U',OPT,I)
      QRENV=QUCH_IN('E',OPT,I)
      IF (INDEX(OPT,'Q').GT.0) QPRPP=.FALSE.
      IF (INDEX(OPT,'Q').GT.0) QPR=.FALSE.
      QPR=QPR.OR.QUCH_IN('P',OPT,I)
      QPRPP=QPRPP.OR.INDEX(OPT,'PP').GT.0
      QANYWH=QUCH_IN('A',OPT,I)
      QCOL=QUCH_IN('C',OPT,I)
      LS=LEN(STR)
      QOF=QUCH_IN('O',OPT,I)
      IF (QOF) THEN
         TSTR(6:)=STR
         TSTR(1:5)='   L '
         LT=LS+5
      ELSE
         TSTR=STR
         LT=MIN0(LS,INDEX_L_NONBLANK(TSTR))
      ENDIF
      lls=index_l_nonblank(str)
      IF (QPR) PRINT '(A/A,I5,I3,7L2/3A)'
     &,       ' <QInFil>>> LT,IUN,QPRPP,QRWD,QUC,QANYWH,QCOL,QOF,QBCK'
     &,       ' <QInFil>>> ',lt,iun,qprpp,qrwd,quc,qanywh,qcol,qof,qbck
     &,       ' <QinFil>>> ::',tstr(:lls),'::'
      IF (QPR.AND.QCOL) PRINT '(A,I5)'
     &,       ' <QInFil>>> COLLOC enabled at column ',jcol
      IF (QPR.AND.JEOI.GT.0) PRINT '(A,I5,A/3A)'
     &,       ' <QInFil>>> EOI    enabled at column ',jeoi
     &,       ' for string'
     &,       ' <QInFil>>> ::',eoistr(:leoi),'::'
      NREC=0
      
101   READ (IUN,'(A)',END=109) INSTR
      if (qrenv) call replace_envars_l (instr)
      if (jeoi.gt.0) then
         if (index(instr,eoistr(:leoi)).eq.jeoi) goto 109
      endif
      NREC=NREC+1
      IF (QUC) THEN
         LI=INDEX_L_NONBLANK(INSTR)
         CALL UCASE (INSTR(:LI),INSTR(:LI))
      ENDIF
      
      ICOL=INDEX(INSTR,TSTR(:LT))
      IF (QPRPP) THEN
         LIN=INDEX_L_NONBLANK(INSTR)
         PRINT '(5A,2I5)'
     &,        ' <QlocIF>d> ',TSTR(:LT),'->',INSTR(:LIN),'<-',ICOL,LT
      endif
      IF (ICOL.LE.0) GOTO 101
      IF (QANYWH) THEN
         IF (QCOL.AND.ICOL.LT.JCOL) GOTO 101
      ELSE
         IF (QCOL.AND.ICOL.NE.JCOL) GOTO 101
         IF (.NOT.QCOL.AND.ICOL.NE.1) GOTO 101
      ENDIF
      QLIF=.TRUE.
      QLOC_IN_FILE=.TRUE.
      ANS='after '
      IF (QBCK) THEN
         ANS='right at '
         BACKSPACE (IUN)
      ENDIF
      IF (QPR) PRINT '(2A,2h->,a,3h<->,a)'
     &,   ' <QInFil>>> ',ANS,TSTR(:LT),INSTR(:LI)
      RETURN
109   IF (QRWD) THEN
         REWIND (IUN)
         NREC=0
         IF (QPR) PRINT '(A,i2)',' <QInFil>>> Rwd ',iun
         QRWD=.FALSE.
         GOTO 101
      ENDIF
      QLIF=.FALSE.
      QLOC_IN_FILE=.FALSE.
      IF (QPR) PRINT '(A,i2,l2,2a)'
     &,              ' <QInFil>E> ',iun,qcol,' failed ',TSTR(:LT)
      RETURN 

      ENTRY QLOCLINE_IN_FILE (RSTR,JJ)
      LR=MIN0(256,LEN(RSTR))
      IF (JJ.LT.0) THEN
         JJJ=INDEX_1_NONBLANK(RSTR(-JJ:))-JJ-1
      ELSE
         JJJ=MAX0(1,JJ)
      ENDIF
      RSTR=INSTR(JJJ:LR)
      QLOCLINE_IN_FILE=QLIF
      RETURN

      ENTRY LOCATED_IN_FILE ()
      LOCATED_IN_FILE=NREC
      RETURN

      ENTRY QCOLLOC_IN_FILE (JJ)
      QCOLLOC_IN_FILE=ICOL.EQ.JJ
      JCOL=JJ
      RETURN

      ENTRY QEOI_IN_FILE (STR,JJ)
      IF (JJ.GT.0) JEOI=JJ
      QEOI_IN_FILE=JEOI.GT.0
      EOISTR=STR
      LEOI=INDEX_L_NONBLANK(EOISTR)
      RETURN

      ENTRY QLOC_IN_FILE_DBG (OPT)
      QPR=QUCH_IN('P',OPT,I)
      QPRPP=INDEX(OPT,'PP').GT.0
      QLOC_IN_FILE_DBG=QPR.OR.QPRPP
      RETURN
      END

