      SUBROUTINE TDT_UTC (DTUJD,DDT)
C----------------------------------------------------------------------C
C                                                                      C
C     ROUTINE ETMUTC, VERSION 900301 FORTRAN 77.                       C
C                                                                      C
C     === VERSION FOR IBM-AT ===                                       C
C                                                                      C
C     ALL VARIABLES WITH D AS FIRST CHARACTER ARE DOUBLE PRECISION.    C
C                                                                      C
C     INPUT PARAMETER DESCRIPTION...                                   C
C     ==============================                                   C
C                                                                      C
C     DTUJD...     JULIAN DATE OF EPOCH (UNIVERSAL TIME).              C
C                                                                      C
C     OUTPUT PARAMETER DESCRIPTION...                                  C
C     ===============================                                  C
C                                                                      C
C     DDT...       ET - UTC   RESP. TDT - UTC IN SECONDS FROM 1955.5   C
C                  TO 1988.0 . FOR EPOCHS LESS 1955.5, DDT=31.59 SEC   C
C                  WILL BE RETURNED. FOR EPOCHS EXCEEDING 1988.0,      C
C                  DDT=56.184 SEC WILL BE RETURNED.                    C
C                  ET  IS EPHEMERIS TIME.                              C
C                  TDT IS TERRESTRIAL DYNAMICAL TIME.                  C
C                  UTC IS COORDINATED UNIVERSAL TIME, USED IN RADIO    C
C                  BROADCAST.                                          C
C                                                                      C
C     THE TABLES DTX, DTJULD, DTY HAVE TO BE EXTENDED, WHEN NEW DATA   C
C     ARE AVAILABLE. CHANGE PARAMETER NTAB AND DIMENSIONS !!!          C
C                                                                      C
C     ROUTINE CREATION...    R.A. BROUCKE,                             C
C                            JET PROPULSION LABORATORY,                C
C     ROUTINE MODIFICATION...900301 BY H.-G. WENZEL,                   C
C                            GEODAETISCHES INSTITUT,                   C
C                            UNIVERSITAET KARLSRUHE,                   C
C                            ENGLERSTR. 7,                             C
C                            D-7500 KARLSRUHE 1.                       C
C                                                                      C
C     UPDATE TABLES...       940131 BY H.-G. SCHERNECK,                C
C                            CHALMERS UNIVERSITY OF TECHNOLOGY         C
C                            ONSALA SPACE OBSERVATORY,                 C
C                            S-439 92  ONSALA                          C
C**********************************************************************C
      IMPLICIT DOUBLE PRECISION (D)
      DIMENSION DTJULD(91),DTX(91),DTY(91)
C----------------------------------------------------------------------C
C     TABLE DJULD DEFINES THE EPOCH OF DDT DEFINED IN TABLE DTY.       C
C     DTJULD IS JULIAN DATE.                                           C
C----------------------------------------------------------------------C
      DATA  DTJULD/   2435289.500D0, 2435655.500D0, 2436020.500D0,
     1 2436385.500D0, 2436750.500D0, 2437116.500D0, 2437481.500D0,
     2 2437665.500D0, 2437846.500D0, 2438030.500D0, 2438211.500D0,
     3 2438395.500D0, 2438577.500D0, 2438761.500D0, 2438942.500D0,
     4 2439126.500D0, 2439307.500D0, 2439491.500D0, 2439672.500D0,
     5 2439856.500D0, 2439947.500D0, 2440038.500D0, 2440130.500D0,
     6 2440222.500D0, 2440312.500D0, 2440403.500D0, 2440495.500D0,
     7 2440587.500D0, 2440677.500D0, 2440768.500D0, 2440860.500D0,
     8 2440952.500D0, 2440983.500D0, 2441011.500D0, 2441042.500D0,
     9 2441072.500D0, 2441103.500D0, 2441133.500D0, 2441164.500D0,
     * 2441195.500D0, 2441225.500D0, 2441256.500D0, 2441286.500D0,
     1 2441317.499D0, 2441317.500D0, 2441499.499D0, 2441499.500D0,
     2 2441683.499D0, 2441683.500D0, 2442048.499D0, 2442048.500D0,
     3 2442413.499D0, 2442413.500D0, 2442778.499D0, 2442778.500D0,
     4 2443144.499D0, 2443144.500D0, 2443509.499D0, 2443509.500D0,
     5 2443874.499D0, 2443874.500D0, 2444239.499D0, 2444239.500D0,
     6 2444786.499D0, 2444786.500D0, 2445151.499D0, 2445151.500D0,
     7 2445516.499D0, 2445516.500D0, 2446247.499D0, 2446247.500D0,
     8 2447161.499D0, 2447161.500D0,                               ! 88.001
     9 2447892.499D0, 2447892.500D0,                               ! 90.001
     * 2448257.499D0, 2448257.500D0,                               ! 91.001
     1 2448804.499D0, 2448804.500D0,                               ! 92.182
     2 2449169.499D0, 2449169.500D0,                               ! 93.181
     3 2449534.499D0, 2449534.500D0,
     4 2449899.499D0, 2449899.500D0, 
     5 2450265.499D0, 2650265.500D0,                               ! 96.182
     6 2450630.499D0, 2450630.500D0,                               ! 97.181
     7 2451544.499D0, 2451544.500D0/                             ! 2000.0 
C----------------------------------------------------------------------C
C     TABLE DTX DEFINES THE EPOCH OF DDT DEFINED IN TABLE DTY.         C
C     DTX IS COUNTED IN YEARS FROM 1900.                               C
C----------------------------------------------------------------------C
      DATA  DTX/  55.5000D0, 56.5000D0, 57.5000D0,
     1 58.5000D0, 59.5000D0, 60.5000D0, 61.5000D0,
     2 62.0000D0, 62.5000D0, 63.0000D0, 63.5000D0,
     3 64.0000D0, 64.5000D0, 65.0000D0, 65.5000D0,
     4 66.0000D0, 66.5000D0, 67.0000D0, 67.5000D0,
     5 68.0000D0, 68.2500D0, 68.5000D0, 68.7500D0,
     6 69.0000D0, 69.2500D0, 69.5000D0, 69.7500D0,
     7 70.0000D0, 70.2500D0, 70.5000D0, 70.7500D0,
     8 71.0000D0, 71.0850D0, 71.1620D0, 71.2470D0,
     9 71.3290D0, 71.4140D0, 71.4960D0, 71.5810D0,
     * 71.6660D0, 71.7480D0, 71.8330D0, 71.9150D0,
     1 71.9990D0, 72.0000D0, 72.4958D0, 72.4959D0,
     2 72.9999D0, 73.0000D0, 73.9999D0, 74.0000D0,
     3 74.9999D0, 75.0000D0, 75.9999D0, 76.0000D0,
     4 76.9999D0, 77.0000D0, 77.9999D0, 78.0000D0,
     5 78.9999D0, 79.0000D0, 79.9999D0, 80.0000D0,
     6 81.4958D0, 81.4959D0, 82.4958D0, 82.4959D0,
     7 83.4958D0, 83.4959D0, 85.4958D0, 85.4959D0,
     8 87.9999D0, 88.0000D0, 89.9999D0, 90.0000D0,
     9 90.9999D0, 91.0000D0, 92.4958D0, 92.4959D0,
     * 93.4958D0, 93.4959D0, 94.4958D0, 94.4959D0,
     1 95.4958d0, 95.4959D0, 96.4958D0, 96.4959D0,
     2 97.4958D0, 97.4959D0, 99.9999d0, 100.0000d0/ 
C----------------------------------------------------------------------C
C     TABLE DTY DEFINES THE DIFFERENCE EPHEMERIS TIME MINUS UTC, OR    C
C     TERRESTRIAL DYNAMICAL TIME MINUS UTC, AS TAKEN FROM THE          C
C     ASTRONOMICAL EPHEMERIS.                                          C
C----------------------------------------------------------------------C
      DATA  DTY/  31.590D0,  32.060D0,  31.820D0,
     1 32.690D0,  33.050D0,  33.160D0,  33.590D0,
     2 34.032D0,  34.235D0,  34.441D0,  34.644D0,
     3 34.950D0,  35.286D0,  35.725D0,  36.160D0,
     4 36.498D0,  36.968D0,  37.444D0,  37.913D0,
     5 38.390D0,  38.526D0,  38.760D0,  39.000D0,
     6 39.238D0,  39.472D0,  39.707D0,  39.946D0,
     7 40.185D0,  40.420D0,  40.654D0,  40.892D0,
     8 41.131D0,  41.211D0,  41.284D0,  41.364D0,
     9 41.442D0,  41.522D0,  41.600D0,  41.680D0,
     * 41.761D0,  41.838D0,  41.919D0,  41.996D0,
     1 42.184D0,  42.184D0,  42.184D0,  43.184D0,
     2 43.184D0,  44.184D0,  44.184D0,  45.184D0,
     3 45.184D0,  46.184D0,  46.184D0,  47.184D0,
     4 47.184D0,  48.184D0,  48.184D0,  49.184D0,
     5 49.184D0,  50.184D0,  50.184D0,  51.184D0,
     6 51.184D0,  52.184D0,  52.184D0,  53.184D0,
     7 53.184D0,  54.184D0,  54.184D0,  55.184D0,
     8 55.184D0,  56.184D0,  56.184D0,  57.184D0,
     9 57.184D0,  58.184D0,  58.184D0,  59.184D0,
     * 59.184D0,  60.184D0,  60.184D0,  61.184D0,
     1 61.184D0,  62.184D0,  62.184D0,  63.184D0,
     2 63.184D0,  64.184D0,  65.184D0,  65.184D0 /
      DATA NTAB/91/
      IF(DTUJD.LT.DTJULD(NTAB)) GOTO 11
C----------------------------------------------------------------------C
C     DTUJD EXCEEDS LAST TABULATED EPOCH DTJULD(NTAB).                 C
C----------------------------------------------------------------------C
   12 DDT=DTY(NTAB)
      PRINT 7051, NTAB,DTJULD(NTAB)
      RETURN

   11 DO  20  I=1,NTAB
      IF(DTUJD-DTJULD(I))   21,22,20
   22 DDT=DTY(I)
      RETURN
   21 N=I-1
      GOTO  23
   20 CONTINUE
   23 DDT=(DTY(N+1)*(DTUJD-DTJULD(N))-DTY(N)*(DTUJD-DTJULD(N+1)))/
     1 (DTJULD(N+1)-DTJULD(N))
      RETURN
C----------------------------------------------------------------------C
C     FORMAT STATEMENTS.                                               C
C----------------------------------------------------------------------C
 7051 FORMAT(' <TdtUtc>E> Table exhausted. Using ',i6,f12.3)
      END
