C This file contains the subprograms for tilt tides computation.
C The output columns in ZDR are 2 and 3; perhaps column 1 can 
C be used for gravity in the future.
C The results have not been tested - 2014-06-18
C
C     -------------------------------
      SUBROUTINE Tilt_Earth (X,Y,ZDR)
C     -------------------------------
C BTIDE Tilt * Elastic elliptic earth model
C>
C After CALL ASTRO(...), Tilt_Earth can be called to return ZDR(1200,3).
C
C ENTRY          Tilt_Anela_Earth (X,Y,ZDR)
C incorporates anelasticity by multiplication of the Basic Love numbers
C with anelastic relaxation factor.
C
C ENTRY          D_Tilt_D_Phi (X,Y,ZDR)
C returns the tilt spectrum derivative w r t the phase parameter of the
C anelasticity model.
C
C ZDR(i,k) - complex*16 - is the tide tilt spectrum for a station at...
C X        - real*8     - east longitude and...
C Y        - real*8     - colatitude
C                         (geocentric coordinates, units = radians).
C
C i   = frequency index, i=0..NV. NV is found in common /CASTSP/.
C k   = component index (unused, NS, EW for k=1..3).
C
C In the tilt tide spectrum, the Basic Love numbers are already applied.
c
c ZDR should be initialized as  3600*(0.0d0,0.0d0) to avoid problems
c with eventual NaN's.
C
C Earth model: Elastic, elliptical earth with liquid core, WAHR (1981),
C refined core resonance parameters according to NEUBERG et al. (1987).
C The coefficients of the leading terms turn out practically
C identical to the Love numbers:
C S1 = h, S2 = l; only S3 = WS3(/WAHRP/) is not equal Wahr's y
C
C
C ENTRY      D_Tilt_Earth (X,Y,ZDR,ZO,ZM)
C
C Returns a linear combination   ZO*ZDR.in + ZM*ZDR.new = ZDR.out
C supposedly after an earth or site parameter has been changed. This call
C provides a way to obtain derivatives w r t a parameter.
C
C
C In order to obtain tidal tilt of component k at time T, perform
C
C   eps(k) = Real ( Sum (i=1,NV)  ZDR(i,k) * Exp(ZI*FRE(i)*T) )
C
C where
C       k = component, 2 - NS, 3 - EW
C      ZI = imaginary unit
C     FRE = Frequency array, found in common /CASTSP/
C      NV = length of spectrum, found in common /CASTSP/
C       T = time from initial epoch in hours.
C More help ...
C_
c
c Multiple calls to Tilt_Earth and too much printing? 
c Print option is turned off automatically after the first call.
c
c Call  Tilt_Earth_Msg  to set print option ON again.
C
      IMPLICIT        REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      PARAMETER       (NMAX=6)
      INTEGER*2       IAA,IATR(0:9)
      INTEGER         ICO(1:4,0:4)
      REAL*8          P(1:6,0:6), DP(1:6,0:6), DDP(1:6,0:6)
     &,               CORDH(2:4,0:4), CORDL(2:4,0:4), CORDK(2:4,0:4)
      REAL*8          WLP0(0:2), WWP0(0:2), WZ0(0:2)
      COMPLEX*16      ZLO(0:4), ZDR(1200,3), ZBL(3)
c      LOGICAL         QTR
      LOGICAL         QANEL,QDANEL,qp
C
C Tide potential spectrum:
      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IAA(0:9,1200)
C
C Maths:
      COMMON /MPAR/   PI,RAD
      COMMON /CWPNM/  WNM(2:5,0:5)
      COMMON /CYWNM/  ywnm(0:nmax,0:nmax),aynm(0:nmax,0:nmax)
C WNM are the scaling factors: GeodCoeff(n,m) = P(n,m)/W(n,m)
C     computed in Astronomic_THD (astros.f)
c
c YWNM are the scaling factors for Ynm(n,m,x) = YWnm(n,m) * Pnm(n,m,x)
C
C Earth parameters:   Mean radius, equat.radius, equat.gravity, rot.velocity,
C                     Basic Love numbers:
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE
C
C Near diurnal resonance (NDR) parameters:
      COMMON /COREP/  ONDR,QNDR,FREFC
     &,               SEHNDR,SELNDR,SEKNDR,ZOHNDR,ZOLNDR,ZOKNDR
C
C Love number ratios for spherical harmonic degrees and orders w r t  n=2, m=1
      COMMON /SHDOP/  CORDH, CORDL, CORDK
C
C Spherical harmonic coupling parameters; Wahr-model
      COMMON /WAHRP/  WWP0,WLP0,WZ0,WY,WWP1,WLP1,WZ1,WS3
     &,               WKP0(0:2), WSLP(0:2)
     &,               WTS(0:3), WTE(0:2)
C
      DATA            ZI/(0.0D0,1.0D0)/
     &,     IATR/2,2,0,0,0,0,0,0,0,0/
     &,     qp/.true./
      QANEL=.false.
      QDANEL=.FALSE.
      GOTO 2
C
      ENTRY Tilt_Anela_Earth (X,Y,ZDR)
C BTIDE Tilt * ... + anelastic earth
C
      QANEL=.true.
      QDANEL=.false.
      GOTO 2
C
      ENTRY D_Tilt_D_Phi (X,Y,ZDR)
C BTIDE Tilt * Derivative w.r.t. phase (anelast.) parameter
C
      QANEL=.true.
      QDANEL=.true.
2     ZO=(0.0D0,0.0D0)
      ZM=(1.0D0,0.0D0)
      GOTO 1
C
      ENTRY D_Tilt_Earth (X,Y,ZDR,ZOD,ZMD)
C BTIDE Tilt * Lin.comb. after parameter redefinition
C
      ZO=ZOD
      ZM=ZMD
C
    1 IF (Y/RAD.LT.1.D-3.OR.Y/RAD.GT.179.99D0) THEN
         PRINT '(A,F10.4)','0<TILT-->W> Invalid colatitude :'
     &,   Y/RAD,'. Adjusted.'
         Y=DMIN1(179.99*RAD,DMAX1(1.D-3*RAD,Y))
      ENDIF
      FRENDR=(1.D0+1.D0/ONDR)*EROT
      ZFRNDR=CMPLX(FRENDR,FRENDR/QNDR/2.D0)
C                                       (N.B. use DCMPLX if available)
      TNSMIN=1.D10
      TEWMIN=1.D10
      TNEMIN=1.D10
C
C Spherical harmonics for the site:
      if (qp) then
         PRINT '(A,2F12.6)'
     &,   '0<TILT-->>> Site longitude / colatitude:',X/RAD,Y/RAD
     &,   ' <TILT-->>> Assigning columns 1..3: eps_NS, eps_EW, eps_NE'
         PRINT '(A)',' <TILT-->>>'
     &,              ' <TILT-->>>  N       P(N,M), DP(N,M)'
     &,              ' <TILT-->>>        DDP(N,M)           M=1..N'
      endif
C
      SINY=DSIN(Y)
      COSY=DCOS(Y)
      CTGY=COSY/SINY
C
      DO 20 N=1,6
      DNN1=DFLOAT(N*(N+1))
      DO 21 M=0,N
      DM=DFLOAT(M)
      ICO(N,M)=0
      P(N,M)=PNM(N,M,Y)
      DP(N,M)=DPNM(N,M,Y)
      DDP(N,M)=(DM*DM/SINY/SINY-DNN1)*P(N,M)-CTGY*DP(N,M)
c      print *,n,m,p(n,m),dp(n,m),ddp(n,m)
21    CONTINUE
      if (qp) then
         print 1003
         PRINT 1001, N,(P(N,M),DP(N,M),M=0,N)
         PRINT 1002,(DDP(N,M),M=0,N)
      endif
20    CONTINUE
      RS=R0*(1.D0-2.D0/3.D0*0.00334D0*P(2,0))
      if (qp) PRINT 1003
      DO 22 M=0,4
      DM=DFLOAT(M)
22    ZLO(M)=EXP(ZI*DM*X)

c      print *,'BL::',zbl(1),bl(1),zbl(3),bl(3)

1001  format (' <TILT-->>>',I3,7(2X,2F9.4))
1002  format (' <TILT-->>>',3X,7(2X,F9.4,9X))
1003  format (' <TILT-->>>')
C                          (N.B. use CDEXP instead of EXP if available)
C
C ------------------------- Step through the astron.spectrum -->>>
      DO 101 IV=1,NV
      N=IAA(0,IV)
      DNN1=DFLOAT(N*N+N)
C
C Discard sph.harm.degree > 4
      IF (N.GT.4) THEN
         ZDR(IV,1)=(0.0D0,0.0D0)
         ZDR(IV,2)=(0.0D0,0.0D0)
         ZDR(IV,3)=(0.0D0,0.0D0)
         GOTO 101
      ENDIF
      M=IAA(1,IV)
      ICO(N,M)=ICO(N,M)+1
      DM=DFLOAT(M)
      IF (QDANEL) THEN
         ZBL(1)=(0.0D0,0.0D0)
         ZBL(3)=(0.0D0,0.0D0)
      ELSE
         ZBL(1)=BL(1)
         ZBL(3)=BL(3)
      ENDIF
C
      IF (N.EQ.2) THEN
C
C Leading terms
C
         IF (M.EQ.1) THEN
C
C Liquid core effects
C
            ZCLQCH=SEHNDR*(FRE(IV)-FREFC)/(FRE(IV)-ZFRNDR)
            ZCLQCK=SEKNDR*(FRE(IV)-FREFC)/(FRE(IV)-ZFRNDR)
            ZOLQCH=ZOHNDR*EROT/(FRE(IV)-ZFRNDR)
            ZOLQCK=ZOKNDR*EROT/(FRE(IV)-ZFRNDR)
C
         ELSE
C
C Long period or semidiurnal
C
            ZCLQCH=(0.0D0,0.0D0)
            ZCLQCK=(0.0D0,0.0D0)
            ZOLQCH=(0.0D0,0.0D0)
            ZOLQCK=(0.0D0,0.0D0)
         ENDIF
C
         IF (QANEL) THEN
            Z0=ZANEL(0,FRE(IV),ZD0)
            Z1=ZANEL(2,FRE(IV),ZD1)
            IF (QDANEL) THEN
               ZBL(1)=BL(1)*ZD0
               ZBL(3)=BL(3)*ZD1
            ELSE
               ZBL(1)=BL(1)*Z0
               ZBL(3)=BL(3)*Z1
            ENDIF
         ENDIF
         S3U=WS3
      ELSE
C
C Spherical harmonic degree > 2
         ZCLQCH=(0.0D0,0.0D0)
         ZCLQCK=(0.0D0,0.0D0)
         ZOLQCH=(0.0D0,0.0D0)
         ZOLQCK=(0.0D0,0.0D0)
         S3U=0.0D0
C
      ENDIF
C
C Put together: potential ampl. / g / r * (spherical eff. + ellips.perturb.)
      GAM=1.d0-BL(1)+BL(3)
      XAMP=AMP(IV)/GE/RS
c
c Note: The Wahr formulation uses the Ynm functions.
c       AMP*GeodCoeff = AMP/Wnm Pnm = AMP/Wnm/YWnm Ynm
c
      ZNS= XAMP*( 1.D0
     &   + (1.D0+ZCLQCH+ZOLQCH) * ZBL(1)*CORDH(N,M)*DP(N,M)
     &   + ZBL(3)*CORDK(N,M)*(1.D0+ZCLQCK+ZOLQCK) * DP(N,M))
      IF (N.EQ.2) THEN
         ZNS= ZNS +XAMP/GAM/YWNM(N,M) *(
     &        WTS(1)*DYNM(N+2,M,Y) 
     &      + ( WTS(2)*YNM(N+1,M,Y)
     &        + WTS(3)*YNM(N-1,M,Y) )*DM/SINY )
      ENDIF

c      print *,'NN:',iv,':: ',zns,xamp,CORDH(N,M),ZCLQCH,ZOLQCH
c     &,        CORDL(N,M),ZCLQCL,ZOLQCL,zbl(1),zbl(2),s3u
c      print *,'n,m,xamp,zi,dm,zbl(1),zbl(3),p(n,m)'
c     &,n,m,xamp,zi,dm,zbl(1),zbl(3),p(n,m)
      ZEW= ZI*DM/SINY* XAMP*( 1.D0
     &   -  ZBL(1)*CORDH(N,M)*(1.D0+ZCLQCH+ZOLQCH)
     &   +  ZBL(3)*CORDK(N,M)*(1.D0+ZCLQCK+ZOLQCK)) * P(N,M)
c      print *,'n,m,zew:',n,m,zew
      IF (N.EQ.2) THEN
         ZEW= ZEW + WNM(N,M)*ZI *XAMP/GAM/YWNM(N,M) *(
     &        WTE(1)*DYNM(N+1,M,Y)+WTE(2)*DYNM(N-1,M,Y) )
      ENDIF
C
      ZPHA=EXP(ZI*PHA(IV))
C                          (N.B. use CDEXP instead of EXP if available)
c      print *,iv,':: ',ZNS,ZLO(M),ZPHA,ZM,ZO,ZDR(IV,1)
      ZDR(IV,2) = ZNS*ZLO(M)*ZPHA*ZM + ZO*ZDR(IV,2)
      ZDR(IV,3) = ZEW*ZLO(M)*ZPHA*ZM + ZO*ZDR(IV,3)
C
C Check smallest amplitude (N.B. use CDABS if available):
      ANS=ABS(ZDR(IV,1))
      AEW=ABS(ZDR(IV,2))
      TNSMIN=DMIN1(ANS,TNSMIN)
      TEWMIN=DMIN1(AEW,TEWMIN)
C ------------------------------------------------ End loop ---<<<
101   CONTINUE

      if (qp) then
         PRINT '(A,1P,2D10.2)'
     &,   ' <TILT-->>> Min. tide amplitudes (NS, EW) :'
     &,   TNSMIN, TEWMIN
         PRINT '(A)',' <TILT-->>>'
     &,   ' <TILT-->>> Number of tides in the different bands:'
     &,   ' <TILT-->>>  N / M=0    1    2    3    4'
         DO 111 N=2,4
111      PRINT 1004, N,(ICO(N,M),M=0,N)
         PRINT 1005,' BLMULT-option turned off.'
cc      else
cc         print 1005,' Ready.'
      endif
      qp=.false.
      CALL BLMULT ('NO ')
      RETURN
1004  format (' <TILT-->>> ',I2,1X,5I5)
1005  format (' <TILT-->>>',A)

      entry tilt_earth_msg
c BTIDE Tilt * Revive message option
c 
      qp=.true.
      return
      END

