      Subroutine Tide_At_Time (t,mm,z,kaddc,x)
c
c t           - real*8   - time [s] after epoch
c
c mm          - integer  - number of components (Displacement: 3)
c
c z(1200,mm)  - cmplx*16 - tide spectrum (frequency setup in COMMON /CASTSP/)
c
c kaddc       - integer  - typically -1, 0 or +1
c                          add kaddc*tide(z) to x and return result in x.
c                          0: return tide(z) in x
c
c x(mm)       - real*8   - result
c
c Basic Love numbers will be applied on tide(z) -> x unless Comb_Ocean_Load 
c or BLMULT('OFF') have been called.
c
c The partial w.r.t. a tide amplitude can be obtained simply by setting
c the corresponding z(i,j)=cexp(zi*pha(i))
c
c Executive entries:
c ------------------
c
c Entry Abs_Tide_At_Time (t,mm,z,x)
c will return the envelope value of the tide at time t.  
c
c Entry Fast_Tide_At_Time (t,mm,z,isk,nsk,x)
c Uses a Fourier interpolation skeleton spectrum z(nsk) of length nsk, pointer
c array isk(nsk). It can be obtained using call Fast_Tide_Spectrum
c (cf fitides.f)
c
c Parameter setting entries:
c --------------------------
c
c Entry dt_Tide_At_Time (q)
c logical q
c Any of the executive entries will compute the
c time derivative of tide (q=.true.) else tide.
c 
      implicit real*8 (a-h,o-y), complex*16 (z)
      integer*2 iaa
      dimension z(1200,*), x(*), y(3), zsk(nsk,*), isk(*)
      logical qblm,qdispl,qabs,qmsg,qfast,qdt,qq
      common /cttsts/ qblm,qdispl,ncompc
      common /castsp/ amp(1200),pha(1200),fre(1200),nv,iaa(0:9,1200)
      COMMON /EPAR/   R0,RA,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG
     &,               RHOW
      data qmsg/.true./, qdt/.false./, zi/(0.0d0,1.0d0)/
      SAVE
      qabs=.false.
      qfast=.false.
      kadd=kaddc
      goto 1

      Entry Abs_Tide_At_Time (t,mm,z,x)
      qabs=.true.
      kadd=0
      qfast=.false.
      goto 1

      Entry Fast_Tide_At_Time (t,mm,zsk,isk,nsk,kaddc,x)
      kadd=kaddc
      qfast=.true.

1     qmsg=.true.
      m=mm
      zi36=zi/3.6d3
      if (m.le.0) m=inq_active_comp()
      zt=cmplx(0.d0,t/3600.d0)
      if (kadd.eq.0) then
         do 103 j=1,m
103      x(j)=0.0d0
         iadd=1
      else
         iadd=kadd
      endif
      if (qabs) then
         do 113 j=1,m
113      y(j)=0.0d0
      endif
  
      nk=nv
      if (qfast) then
         nk=nsk
         if (isk(1).eq.-1) qfast=.false.
      endif

      do 101 kk=1,nk
      if (qfast) then
         k=isk(kk)
         if (k.le.0) goto 109
      else
         k=kk
      endif
      za=exp(zt*fre(k))
      if (qdt) za=zi36*za*fre(k)
      do 101 j=1,m
      if (qfast) then
         zv=zsk(kk,j)
      else
         zv=z(kk,j)
      endif
      if (qabs) then
         y(j)=y(j)+iadd*dimag(zv*za)
      endif    
      x(j)=x(j)+iadd*dreal(zv*za)
101   continue

 109  if (qabs) then
         do 114 j=1,m
114      x(j)=dsqrt(x(j)**2+y(j)**2)
      endif

      if (qblm.and.qdispl) then
         if (qmsg) then
            qmsg=.false.
            print '(a)',' <TideAT>>> Applying "Basic" Love numbers'
         endif
         do 104 j=1,m
         jj=min0(2,j)
104      x(j)=x(j)*bl(jj)
      endif
      return

      entry dt_tide_at_time (qq)
      qdt=qq
      return
      end

