      complex*16 function zolndr_val (itype,iaa_t,zp21)
c
c Returns the value for ZOHNDR ZOLNDR ZOKNDR  (ITYPE = 0,1,2) 
c given the spherical harmonic coefficient ZP21 [metres] 
c of ocean tide IAA_T.
c
c This formulation assumes ZP21 = C_{21+} * exp (i*EPS_{21+})
c
c The result can be implemented into the parmset database by calling
c CALL NDROSE (itype, zolndr_val(itype,iaa_t,zp21))
c
c iaa_t can be obtained from a Darwin symbol using
c CALL argn_of_symbol (symbol,iaa_t,9,.false.)
c
      implicit real*8 (a-h,o-y), complex*16 (z)
      integer*2  iaa_t(0:9), iaa
      
      common /mpar/ pi,rad
      COMMON /EPAR/ R0,RE,GE,EROT,BL(0:2),GM,GRVLAT,ESQUAR,FE,BG,RHOW
      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IAA(0:9,1200)
      COMMON /WAHRP/  WWP0(0:2), WLP0(0:2), WZ0(0:2)
     &,               WY, WWP1, WLP1, WZ1, WS3
     &,               WKP0(0:2), WSLP(0:2)
     &,               WT(0:5)

      EXTERNAL GAMMA

      data zi/(0.0d0,1.0d0)/

      if (iaa_t(0).ne.2.or.iaa_t(1).ne.1) then
         print '(a,10i3.2)',' <Zolndr>E> Not a tesseral: ',iaa_t
         return
      endif

      nt   = nr_of_tide (iaa_t,9)
      nref = nr_of_tidesymbol ('O1',.false.)

      zolndr_val=4*pi*bg*rhow*re/5.d0/bl(itype)*wslp(itype)
     &           *gamma(fre(nref),2,1)/gamma(fre(nt),2,1)
     &           *zi*zp21/amp(nt)
c
c N.B. GE is eliminated since amp is in [m^2/s^2], not in [m] as assumed
c in Scherneck's 1991 Eq (2.1).
      return
      end

